#ifndef CXXOMFORT_ARRAY_HPP
#define CXXOMFORT_ARRAY_HPP
/**
 * @file
 * @brief Implementations and Backports for Standard header <tt>array</tt>.
 * @ingroup std0array
 * @xrefitem std0array "<array>" "Header" ""
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/config.hpp>
#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>1)
	#pragma message CXXO_NOTICE("enabled <array> support.")
#endif

// 
// <array> header
// 

#if (CXXOMFORT_CXX_STD>=2011 && CXXO_CXXSTDHEADER_array)
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<array> C++11 native")
	#endif
	#include <array>
#elif (CXXOMFORT_CXX_STD>=1997 && CXXO_CXXSTDHEADER_array)
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<array> C++0x native")
	#endif
	#include <array>
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=300) && CXXOMFORT_CXX_STD<2011 && !defined(_LIBCPP) )
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<array> C++/TR1 Clang")
	#endif
	#include <tr1/array>
#elif (CXXOMFORT_CXX_STD<2011 && CXXO_CXXSTDHEADER_tr1_array)
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<array> header <tr1/array>")
	#endif
	#include <tr1/array>

#elif defined(CXXOFLAG_USE_INTERNAL_TR1)
	#if (CXXOMFORT_NOTICES > 1)
		#pragma message CXXO_NOTICE("internal TR1: <array>")
	#endif
	// not even C++03
	#include "impl/07-array.hpp"


#else // unrecognized
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<array> request external")
	#endif
	#include <array>
#endif // CXX_STD

// 
// std::array, etc
// 

#if (CXXOMFORT_CXX_STD>=2011)
#elif (CXXO_COMPILER_SUPPORT_array)
#elif (CXXO_COMPILER_SUPPORT_tr1_array)
namespace std {
	using tr1::array;
}
#elif (defined(CXXOFLAG_USE_INTERNAL_TR1))
namespace std {
	using ::cxxomfort::cxxostd::array;
}
#else
#endif


// 
// fix::array 
// 

#include "utils/array.hpp"

namespace cxxomfort { namespace fix {
	#if (CXXOMFORT_CXX_STD >= 2017)
	using std::array;
	#elif (CXXOMFORT_CXX_STD >= 2014)
	using cxxomfort::array;
	#elif (CXXOMFORT_CXX_STD >= 1997)
	using std::array;
	#endif
	
} } // cxxomfort::fix

// 
// make_array (experimental/lfv2)
// 

#include "impl/n4031_make_array.hpp"

namespace std { namespace experimental {
    using ::cxxomfort::make_array;
} }

// 
// to_array (C++20)
// 


#if defined(DOXYGEN_DOC)
namespace std {

/**
 * @cxx20b Creates a <tt>std::array</tt> from a native C array.
 * @xrefitem std0array "" "" Creates a <tt>std::array</tt> from a native C array.
 * 
 * @note This is backported down to full C++11 support only. For a more generic solution in cxxomfort, see @c make_array() or @c make_cxxo_array() .
 * 
 * */ 
template <typename T, size_t N>
constexpr auto to_array(T (&arr)[N])
-> std::array<typename std::remove_cv<T>::type , N> ;

}

#elif defined(__cpp_lib_to_array)

#elif ( ((CXXO_COMPILER_SUPPORT_auto>0 && CXXO_COMPILER_SUPPORT_rvref>0 && CXXO_COMPILER_SUPPORT_variadic>0 && CXXO_COMPILER_SUPPORT_constexpr>0)) )
    #include <type_traits>
    #include <cxxomfort/utility.hpp> // index_sequence

namespace cxxomfort { namespace cxxostd {
namespace detail {

    template <typename T, size_t N, size_t... I>
    constexpr std::array< typename std::remove_cv<T>::type , N> 
    _to_array(T (&arr)[N], std::index_sequence<I...> ) {
        return { { arr[I]... } };
    }

    template <typename T, size_t N, size_t... I>
    constexpr std::array< typename std::remove_cv<T>::type , N> 
    _to_array(T (&&arr)[N], std::index_sequence<I...> ) {
        return { { std::move(arr[I])... } };
    }
 
} // detail_array

template <typename T, size_t N>
constexpr std::array<typename std::remove_cv<T>::type , N> 
to_array (T (&arr)[N]) {
    return detail::_to_array(arr, std::make_index_sequence<N>{} );
} 

template <typename T, size_t N>
constexpr std::array< typename std::remove_cv<T>::type , N>
to_array (T (&&arr)[N]) {
    return detail::_to_array(std::move(arr), std::make_index_sequence<N>{} );
}

} }

namespace std {
    using ::cxxomfort::cxxostd::to_array;
}
    #define __cpp_lib_to_array (CXXOMFORT_CXX_STD * 100)

#else // c++03

namespace cxxomfort { namespace cxxostd {

template <typename T, size_t N>
std::array< typename std::remove_cv<T>::type, N >
to_array (T (&arr)[N] ) ;

template <typename T>
std::array< typename std::remove_cv<T>::type, 1>
to_array (T (&arr)[1] ) {
    std::array<typename std::remove_cv<T>::type, 1> a 
    = { arr[0] };
    return a;
}

template <typename T>
std::array< typename std::remove_cv<T>::type, 2>
to_array (T (&arr)[2] ) {
    std::array<typename std::remove_cv<T>::type, 2> a 
    = { arr[0], arr[1] };
    return a;
}

template <typename T>
std::array< typename std::remove_cv<T>::type,  3>
to_array (T (&arr)[3] ) {
    std::array<typename std::remove_cv<T>::type, 3> a 
    = { arr[0], arr[1], arr[2] };
    return a;
}

template <typename T>
std::array< typename std::remove_cv<T>::type,  4>
to_array (T (&arr)[4] ) {
    std::array<typename std::remove_cv<T>::type, 4> a 
    = { arr[0], arr[1], arr[2], arr[3] };
    return a;
}


}}

    #define __cpp_lib_to_array 199703
#endif


#endif

#ifdef DOXYGEN_DOC
namespace cxxomfort { namespace cxxostd {
	/**
	 * @brief C++11 array class
	 * */
	 template <typename T, size_t N> class array {};
}}
namespace std {
	using ::cxxomfort::cxxostd::array;
}
	
#endif

/**
 * @page std0array <array>
 * @brief Backports related to Standard header <tt><array></tt> 
 * 
 * @code
 * #include <cxxomfort/array.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * 
 * Including the header brings up @c std::array , accounting for eg.: location being @c std::tr1::array where set up in C++03+TR1.
 * 
 * If no TR1 is found and the user has defined the macro <tt>CXXOFLAG_USE_INTERNAL_TR1</tt>, cxxomfort uses this header to provide its own implementation of <tt>array</tt>.
 * 
 * An @c std::array is much like a native C-array in that it has a type and size set up at compile time, but it also adds an interface similar to most of C++'s sequence containers: it has a <tt>begin/end</tt>, an <tt>iterator</tt> type, and accepts indexing of elements via the <tt>operator[]</tt>.
 * 
 * @section Interfaces
 * 
 * Interfaces defined in this section:
 * 
 * * @c std::array (C++11) for compilers that don't have eg.: C++03/TR1.
 * * @c std::to_array() (C++20) - converts native arrays to <tt>std::array</tt>.
 * * the @c __cpp_lib_to_array macro if the above is backported.
 * * @c make_array() (exp) - from LFv2, creates <tt>std::array</tt> from a list of arguments.
 * 
 * Interfaces from TR1 repointed in this section:
 * 
 * * @c @ref get(std::array)
 * 
 * Interfaces not in std:
 * 
 * * @c to_cxxo_array() - converts native arrays to <tt>cxxomfort::array</tt>.
 * 
 * @section array_cxxomfort_array array (cxxomfort)
 * 
 * This header also makes available cxxomfort's implementation of array, found at <tt>cxxomfort::array</tt>. This implementation differs from std mainly in enabling some @c constexpr -capable member functions in compilers that were capable but the stdlib did not mark array so, such as modifying <tt>operator[]</tt> in C++14.
 * 
 * 
 * @section array_examples Examples 
 * 
 * For other examples see the following entries in <a href="https://notabug.org/lmachucabezzaza/cxxomfort-examples">cxxomfort-examples</a>:
 * 
 * * @c array.cpp : usage of array and related functions.
 * * @c algorithms_constexpr.cpp : usage of @c cxxomfort::array in constexpr context.
 * 
 * @section also See Also 
 * 
 * @see @cppref{header/array} (cppreference)
 * @see @cppref{array}
 * @see @link libarrayref cxxomfort\'s @c array_ref @endlink
 * @see @ref cxxomfort::array - cxxomfort's own version of @c array .
 * @see @cppref{experimental/make_array} on @c make_array .
 * 
 * */

