#ifndef CXXOMFORT_BASE_ALIGNOF_HPP
#define CXXOMFORT_BASE_ALIGNOF_HPP
/**
 * @file
 * @brief Implementation details for "alignof" emulation.
 * @ingroup cxxo-base
 */

#include "../config.hpp"

#if (CXXOMFORT_CXX_STD < 2011 || CXXO_COMPILER_SUPPORT_alignof==0)
    #define CXXOMFORT_NEEDS_alignof
#endif


#if defined(CXXOMFORT_NEEDS_alignof) || defined(DOXYGEN_DOC)

// from https://stackoverflow.com/questions/364483/determining-the-alignment-of-c-c-structures-in-relation-to-its-members

namespace cxxomfort { namespace cxxostd {

template <typename T>
struct alignof_impl {
	private:
	struct helper {
		char _;
		T t;
	};
	public:
	enum { value = sizeof(helper)-sizeof(T) };
	
};



} }

/* alignof */
    #ifdef alignof
    #error " cxxomfort - conflicting #define directive for 'alignof()'!"
    #endif

/**
 * @def alignof
 * @brief Provides <code>alignof(x)</code> in platforms lacking the directive.
 * @ingroup cxx11-backports
 * @note This definition relies on the existence of an existing intrinsic 
 * <code>__alignof</code> that would have similar semantics.
 * 
 * Related utilities: @c alignas , @c aligned_storage .
 * */
	#if (CXXO_COMPILER_SUPPORT___alignof)
		#define alignof(x) __alignof( x )
	#else
		#define alignof(x) (cxxomfort::cxxostd::alignof_impl<x>::value)
	#endif

#endif // using emulation


#endif // file
