#ifndef CXXOMFORT_BASE_CSTDINT_HPP
#define CXXOMFORT_BASE_CSTDINT_HPP
/**
 * @file
 * @ingroup cxxo-base
 */

/*
 * This header file redirects to the header providing
 * the standard integer types; in C++11 this is
 * <cstdint>, whereas in C++03 this is <stdint.h>
 */
#include "../config.hpp"
#if (CXXOMFORT_CXX_STD<2011 && !defined(__STDC_LIMIT_MACROS))
	// #pragma message CXXO_NOTICE("note pre-C++11 mode __STDC_LIMIT_MACROS is not defined")
#endif

#if (CXXOMFORT_CXX_STD>=2011 || CXXO_CXXSTDHEADER_cstdint)
    #if defined(CXXOMFORT_NOTICES)  && (CXXOMFORT_NOTICES > 1)
        #pragma message CXXO_NOTICE("Including <cstdint> in C++11 mode or native")
    #endif
    #include <cstdint>
    #include <stdint.h>
    #define CXXOMFORT_USING_NATIVE_stdint
    #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
    #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()
#else
    #if 0
        #error ("Unreachable.");
    
    #elif (CXXO_CXXSTDHEADER_cstdint > 0)
        #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
            #pragma message CXXO_NOTICE("Including native <cstdint>")
        #endif
        #include <cstdint>
        #include <stdint.h>
        #define CXXOMFORT_USING_NATIVE_stdint
        #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
        #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()
    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_CLANG && !defined(_LIBCPP))
    // GCC and clang have a <stdlib.h>
        #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
            #pragma message CXXO_NOTICE("cstdint forwarded to tr1/ <cstdint> [clang]")
        #endif
        //#define __STDC_LIMIT_MACROS
        #include <tr1/cstdint>
        #include <stdint.h>
        #define CXXOMFORT_USING_NATIVE_stdint
        #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
        #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()

    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_GCC)
        #if (CXXOMFORT_COMPILER_VERSION >= 406)
            #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
                #pragma message CXXO_NOTICE("cstdint forwarded to tr1/ <cstdint> [gcc]")
            #endif
            //#define __STDC_LIMIT_MACROS
            #include <tr1/cstdint>
            #include <stdint.h>
            #define CXXOMFORT_USING_NATIVE_stdint
            #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
            #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()
        #else
            #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
                #pragma message CXXO_NOTICE("cstdint forwardrd to <stdint.h> [gcc]")
            #endif
            //#define __STDC_LIMIT_MACROS
            #include <stdint.h>
            #define CXXOMFORT_USING_NATIVE_stdint
            #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
            #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()
        #endif
    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_MSC)
        #if (CXXOMFORT_COMPILER_VERSION>=1600)
            #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
                #pragma message CXXO_NOTICE("cstdint forwarded to MSVC:<cstdint>")
            //#define __STDC_LIMIT_MACROS
            #include <cstdint>
            #define CXXOMFORT_USING_NATIVE_stdint
            #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
            #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_NATIVE()
            #endif
        #else
            //#include <stdint.h>
        #endif
    #elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_DMC && CXXOMFORT_COMPILER_VERSION<0x850)
		#if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
			#pragma message CXXO_NOTICE("cstdint forwarded to DMC:<stdint.h>")
		#endif
		#include <stdint.h>
		// There is a bug in older DMC versions where having stdint.h and wchar.h included leads to a macro already define hard error.
		#undef WCHAR_MAX
		#undef WCHAR_MIN
		#include <inttypes.h>
		#define CXXOMFORT_USING_NATIVE_stdint
		#undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
		#define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_BACKPORT()
    
    #else // other compilers
		#if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
			#pragma message CXXO_NOTICE("cstdint forwarded to default stdint.h")
		#endif
		#include <stdint.h>
		#include <inttypes.h>
		#define CXXOMFORT_USING_NATIVE_stdint
		#undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
		#define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_BACKPORT()
    #endif
#endif // c++11

#if (!defined(CXXOMFORT_USING_NATIVE_stdint))
    #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
        #pragma message CXXO_NOTICE("cstdint forwarded to portable-stdint")
    #endif
    //#define __STDC_LIMIT_MACROS
    #include "../vendor/pstdint.h"
    #define CXXOMFORT_USING_PSTDINT_stdint
    #undef  CXXOMFORT_IMPLEMENTS_cstdint_integrals
    #define CXXOMFORT_IMPLEMENTS_cstdint_integrals CXXO_IMPLSTATUS_EMULATION()
    // use up the macro guard of stdint.h to prevent a collission
    #define _MSC_STDINT_H_ // win32
#endif // using pstdint

#include <climits>
#include <stddef.h>
#endif // header

