#ifndef CXXOMFORT_SASSERT_HPP
#define CXXOMFORT_SASSERT_HPP
/**
 * @file
 * @brief Implementation details for "static_assert" backport.
 * @ingroup cxxo-base
 */

#include "../config.hpp"

#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_COMPILER_SUPPORT_static_assert)
#else
	#define CXXOMFORT_USING_static_assert
#endif

// implementation
/////////////////

#if defined(CXXOMFORT_USING_static_assert)
    #if defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 2)
    #pragma message CXXO_NOTICE("enabled base 'static_assert' emulation.")
    #endif


// trick from https://stackoverflow.com/questions/3046889/optional-parameters-with-c-macros 
// for allowing a macro with an optional argument

/**
 * @def static_assert
 * @cxx1b 
 * @brief Provides an assertion at compile-time
 * @param test The boolean test to run, must be computable at compile-time
 * @param msg The string literal to provide as a compiler error message if the assertion does not hold
 * @see https://en.cppreference.com/w/cpp/language/static_assert
 * 
 */
//    #define static_assert(test,msg) enum { CXXO_JOIN(sassert_e_,__COUNTER__) = (int)(msg[0])/(test) }
    #if defined(_MSC_VER)
		#define static_assert(test,msg) typedef void* CXXO_JOIN(sassert_expr_,__COUNTER__)[(test)?1:-1]
    #elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_OW || CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_DMC)
		#define static_assert(test,msg) enum { CXXO_JOIN(sassert_expr_,__LINE__) = 1 /(test) }
    #else
		#define static_assert(test,msg) enum { CXXO_JOIN(sassert_e_,__COUNTER__) = 1/(test) }
    #endif

    #define CXXOMFORT_IMPLEMENTS_n1381 CXXO_IMPLSTATUS_EMULATION()

#else 
    #define CXXOMFORT_IMPLEMENTS_n1381 CXXO_IMPLSTATUS_NATIVE()
#endif // USING
#define CXXOMFORT_IMPLEMENTS_n1604 CXXOMFORT_IMPLEMENTS_n1381
#define CXXOMFORT_IMPLEMENTS_n1720 CXXOMFORT_IMPLEMENTS_n1604

#undef  CXXOMFORT_IMPLEMENTS_static_assert
#define CXXOMFORT_IMPLEMENTS_static_assert CXXOMFORT_IMPLEMENTS_n1720

// 
// n3928: static_assert without message
// partial implementation, using a different macro name
// 

#if (defined(_cpp_static_assert) && (__cpp_static_assert>=201411))
    #define static_assert0(test) static_assert(test)
	#undef  CXXOMFORT_IMPLEMENTS_n3928 
	#define CXXOMFORT_IMPLEMENTS_n3928 CXXO_IMPLSTATUS_NATIVE()
#else
    #define static_assert0(test) static_assert(test,#test)
	#undef  CXXOMFORT_IMPLEMENTS_n3928
	#define CXXOMFORT_IMPLEMENTS_n3928 CXXO_IMPLSTATUS_EMULATION()
#endif
#undef  CXXOMFORT_IMPLEMENTS_static_assert_17
#define CXXOMFORT_IMPLEMENTS_static_assert_17 CXXOMFORT_IMPLEMENTS_n3928 

#endif
