#ifndef CXXOMFORT_CONFIG_HPP
    #error "This file should only be #included via <cxxomfort/config.hpp>"
#endif
#ifndef CXXOMFORT_CONFIG_PRE_HPP
#define CXXOMFORT_CONFIG_PRE_HPP

//! @file 

//! Expands to nothing
//! @ingroup cxxomfort-codegen
#define CXXO_NONE() 

//! @xrefitem cxxomfort-codegen "" "Parsing" Expands to a comma literal (\"<tt>,</tt>\").
#define CXXO_COMMA() ,

//! @xrefitem cxxomfort-codegen "Parsing" "" Expands to its arguments.
#define CXXO_EXPAND(...) __VA_ARGS__

#define CXXO_EXPAND_AND_QUOTE(a1) CXXO_STRINGIZE1(a1)

//! @xrefitem cxxomfort-codegen "Parsing" "" Unpacks parentheses
#define CXXO_UNPACK2(a1,a2) a1, a2

#define CXXO_STRINGIZE_IMPL(...) #__VA_ARGS__
//! @xrefitem cxxomfort-codegen "Parsing" "" Stringizes tokens: <tt>CXXO_STRINGIZE(\_\_FILE\_\_, "+", \_\_LINE\_\_)</tt> expands to for example <tt>\"myfoo.cpp+18\"</tt>.
#define CXXO_STRINGIZE(...) CXXO_STRINGIZE_IMPL(__VA_ARGS__)

#define CXXO_STRINGIZE1_IMPL2(x) #x
#define CXXO_STRINGIZE1_IMPL1(x) CXXO_STRINGIZE1_IMPL2(x)
//! @xrefitem cxxomfort-codegen "Parsing" "" Stringizes one token.
#define CXXO_STRINGIZE1(x) CXXO_STRINGIZE1_IMPL1(x)

#define CXXO_WIDEN1_IMPL(x) L#x
//! @xrefitem cxxomfort-codegen "Parsing" "" Stringizes one token into a wide string literal.
#define CXXO_WIDEN(x) CXXO_WIDEN1_IMPL(x)

#define CXXO_JOIN_IMPL2(x,y) x##y
#define CXXO_JOIN_IMPL1(x,y) CXXO_JOIN_IMPL2(x,y)
//! @xrefitem cxxomfort-codegen "Parsing" "" Joins two tokens.
#define CXXO_JOIN(x,y) CXXO_JOIN_IMPL1(x,y)

#define CXXO_ZCHECK_HELPER(xm,sf) ( (#xm[0] != sf) )
//! @xrefitem cxxomfort-codegen "" "" Tests if the macro with the given name exists, using prefix char @c safety to detect it.
#define CXXO_IF_MACRO(x,safety) CXXO_ZCHECK_HELPER(x, safety)

#define CXXO_IF___cpp_feature(name) CXXO_IF_MACRO(name,'_')

//! @xrefitem cxxomfort-codegen "Compile Info" "" The current year as a number, as exposed in @c __DATE__ .
#define CXXOMFORT_ISOYEAR() ((__DATE__[7]-'0')*1000 + (__DATE__[8]-'0')*100 + (__DATE__[9]-'0')*10 + (__DATE__[10]-'0') )
//! @xrefitem cxxomfort-codegen "Compile Info" "" The current month as a number.
#define CXXOMFORT_ISOMONTH() ( \
 ((__DATE__[0]=='J' ? \
 (__DATE__[1]=='a' ? 1 : (__DATE__[2]=='n' ? 6 : 7)   )\
 :0)) | \
 ((__DATE__[0]=='F' ? 2:0)) | \
 ((__DATE__[0]=='M' ? \
 (__DATE__[2]=='r' ? 3 : 5)\
 :0)) | \
 ((__DATE__[0]=='A' ? \
 (__DATE__[1]=='p' ? 4 : 8)\
 :0)) | \
 ((__DATE__[0]=='S' ? 9:0)) | \
 ((__DATE__[0]=='O' ? 10:0)) | \
 ((__DATE__[0]=='N' ? 11:0)) | \
 ((__DATE__[0]=='D' ? 12:0)) \
 )
//! @xrefitem cxxomfort-codegen "Compile Info" "" The current day as a number, as exposed in @c __DATE__ .
#define CXXOMFORT_ISODAY() (( __DATE__[4]==' ' ?  0 : __DATE__[4]-'0')*10 + (__DATE__[5]-'0'))
//! @xrefitem cxxomfort-codegen "Compile Info" "" Expands to the current date as a number in @c YYYYMMDD format.
#define CXXOMFORT_ISODATE() ( CXXOMFORT_ISOYEAR() * 10000UL + CXXOMFORT_ISOMONTH() * 100 + CXXOMFORT_ISODAY() )

#endif
