#ifndef CXXOMFORT_CONFIG_HPP
    #error "This file should only be #included via <cxxomfort/config.hpp>"
#endif
#ifndef CXXOMFORT_CONFIG_SUPPORT_HPP
#define CXXOMFORT_CONFIG_SUPPORT_HPP

/**
 * @file
 */

#include <cstddef>

#define CXXOMFORT_CXX_STD (__cplusplus/100)
#define CXXOMFORT_CXX_EMULATION 0

// 
// compilers supported
// 

#define CXXO_COMPILERID_GCC 101
#define CXXO_COMPILERID_MSC 102
#define CXXO_COMPILERID_CLANG 103
#define CXXO_COMPILERID_DMC 104
#define CXXO_COMPILERID_OW 105
#define CXXO_COMPILERID_DJGPP 106

// 
// per-standard version features
// 

#define CXXO_COMPILER_SUPPORT__Pragma 0
#define CXXO_COMPILER_SUPPORT_alignN (CXXOMFORT_CXX_STD>=2011)
#define CXXO_COMPILER_SUPPORT_expression_sfinae (CXXOMFORT_CXX_STD>=2011)
#define CXXO_COMPILER_SUPPORT_rtti (defined(__cpp_rtti))

#define CXXO_COMPILER_SUPPORT___alignof 0 
#define CXXO_COMPILER_SUPPORT_type_intrinsics 0 
#define CXXO_COMPILER_SUPPORT_functional_oldstyle (CXXOMFORT_CXX_STD<2017)
#define CXXO_COMPILER_SUPPORT_tr1_array 0
#define CXXO_COMPILER_SUPPORT_tr1_cstdint 0
#define CXXO_COMPILER_SUPPORT_tr1_random 0
#define CXXO_COMPILER_SUPPORT_tr1_tuple 0
#define CXXO_COMPILER_SUPPORT_tr1_type_traits 0
#define CXXO_COMPILER_SUPPORT_typeof 0
#define CXXO_COMPILER_SUPPORT_va_args 0


#define CXXOFLAG_tr1 0
#define CXXO_STDTR1_OPEN() namespace std  {
#define CXXO_STDTR1_CLOSE()  }
#define CXXO_STDTR1_NAME() std
#define CXXOFLAG_directory_tr1 0
#define CXXOFLAG_namespace_tr1 0

// 
// c++11 stuff
// 

#define CXXO_CXXSTDHEADER_array (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_cinttypes (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_cstdint (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_cuchar (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_random (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_ratio (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_system_error (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_tuple (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_typeindex (CXXOMFORT_CXX_STD>=2011)
#define CXXO_CXXSTDHEADER_type_traits (CXXOMFORT_CXX_STD>=2011)

//! If compiler provides support for @c alignof(T) .
#define CXXO_COMPILER_SUPPORT_alignof 0
//! If compiler provides support for <tt>std::array</tt>
#define CXXO_COMPILER_SUPPORT_array (CXXOMFORT_CXX_STD >= 2011)
//! If compiler provides support for <tt>[[__attribute__]]</tt>.
#if (defined(__cpp_attributes))
	#undef  CXXO_COMPILER_SUPPORT_attribute
	#define CXXO_COMPILER_SUPPORT_attribute 1
#else
	#undef  CXXO_COMPILER_SUPPORT_attribute
	#define CXXO_COMPILER_SUPPORT_attribute 0
#endif
//! If compiler supports new (type-deduction) semantics for @c auto .
#define CXXO_COMPILER_SUPPORT_auto (CXXOMFORT_CXX_STD>=2011)
#if (defined(__cpp_auto))
	#undef  CXXO_COMPILER_SUPPORT_auto
	#define CXXO_COMPILER_SUPPORT_auto 1
#endif
//! If compiler provides support for the new C++11 char types.
#if (defined(__cpp_unicode_characters))
	#define CXXO_COMPILER_SUPPORT_charn_t 1
#else
	#define CXXO_COMPILER_SUPPORT_charn_t 0
#endif
//! If the above types are defined as typedef of integral types (such as unsigned short).
#define CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF 0
//! If compiler provides support for @c constexpr .
#if (CXXOMFORT_CXX_STD>=2011 || defined(__cpp_constexpr) )
	#define CXXO_COMPILER_SUPPORT_constexpr 1
#else
	#define CXXO_COMPILER_SUPPORT_constexpr 0
#endif
//! If the compiler supports dereferencing a const pointer in constexpr
#if (defined(__cpp_constexpr))
	#if (__cpp_constexpr>=201304L) // c++14 relaxed
		#define CXXO_COMPILER_SUPPORT_constexpr_dereference 1
	#else
		#define CXXO_COMPILER_SUPPORT_constexpr_dereference 0
	#endif
#else
	#define CXXO_COMPILER_SUPPORT_constexpr_dereference 0
#endif
//! If compiler supports @c decltype(expr) .
#if defined(__cpp_decltype)
	#define CXXO_COMPILER_SUPPORT_decltype 2343
#else
	#define CXXO_COMPILER_SUPPORT_decltype 0
#endif

//! If compiler provides support for <code>=default</code>-ed and <code>=delete</code>-d members.
#define CXXO_COMPILER_SUPPORT_default_delete (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for scoped enumerations ie.: <code>enum class...</code>.
#define CXXO_COMPILER_SUPPORT_enum_class (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for <code>explicit operator ...</code> casts.
#define CXXO_COMPILER_SUPPORT_explicit_operator (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for range-for.
#if (CXXOMFORT_CXX_STD >= 2011 || defined(__cpp_range_based_for))
	#define CXXO_COMPILER_SUPPORT_foreach 1
#else
	#define CXXO_COMPILER_SUPPORT_foreach 0
#endif

//! If compiler provides support for inheritance's "final" keyword
#define CXXO_COMPILER_SUPPORT_inheritance_final (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for inheritance's "override" keyword
#define CXXO_COMPILER_SUPPORT_inheritance_override (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for inheriting constructors.
#define CXXO_COMPILER_SUPPORT_inheriting_constructors (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for @c std::initializer_list .
#define CXXO_COMPILER_SUPPORT_initializer_list (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for lambda-expressions
#define CXXO_COMPILER_SUPPORT_lambda_expressions (CXXOMFORT_CXX_STD>=2011)
#if (CXXO_COMPILER_SUPPORT_lambda_expressions==0 && defined(__cpp_lambdas))
	#undef  CXXO_COMPILER_SUPPORT_lambda_expressions
	#define CXXO_COMPILER_SUPPORT_lambda_expressions
#endif
//! If compiler provides support for local and unnamed types as template arguments.
#define CXXO_COMPILER_SUPPORT_local_types (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for @c noexcept *AND* is_noexcept-traits.
#define CXXO_COMPILER_SUPPORT_noexcept (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for @c nullptr  and @c std::nullptr_t .
#define CXXO_COMPILER_SUPPORT_nullptr (CXXOMFORT_CXX_STD>=2011)

#define CXXO_COMPILER_SUPPORT_override_final (CXXOMFORT_CXX_STD>=2011)

//! If compiler provides support for <em>raw string literals</em>.
#if (CXXOMFORT_CXX_STD >= 2011 || defined(__cpp_raw_strings))
	#define CXXO_COMPILER_SUPPORT_raw_strings 1
#else
	#define CXXO_COMPILER_SUPPORT_raw_strings 0
#endif

//! If compiler provides support for @c T&& .
#define CXXO_COMPILER_SUPPORT_rvref (CXXOMFORT_CXX_STD>=2011)
#if (CXXOMFORT_CXX_STD >= 2011 || defined(__cpp_rvalue_references) )
	#undef  CXXO_COMPILER_SUPPORT_rvref
	#define CXXO_COMPILER_SUPPORT_rvref 1
#else
	#undef  CXXO_COMPILER_SUPPORT_rvref
	#define CXXO_COMPILER_SUPPORT_rvref 0
#endif

//! If compiler provides support for <tt>shared_ptr<T></tt>.
#define CXXO_COMPILER_SUPPORT_shared_ptr (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for @c static_assert .
#define CXXO_COMPILER_SUPPORT_static_assert (CXXOMFORT_CXX_STD>=2011)
#if defined(__cpp_static_assert) 
	#undef  CXXO_COMPILER_SUPPORT_static_assert
	#define CXXO_COMPILER_SUPPORT_static_assert 1
#endif
//! If compiler provides support for the <code>std::is_[trivially|nothrow]_...</code> traits.
#define CXXO_COMPILER_SUPPORT_std_is_trivially (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for template aliases (template<> using)
#define CXXO_COMPILER_SUPPORT_template_aliases (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for trailing return types.
#define CXXO_COMPILER_SUPPORT_trailing_returns (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for type traits (basic category)
#define CXXO_COMPILER_SUPPORT_type_traits_basic (CXXOMFORT_CXX_STD>=2011 || CXXOMFORT_CXX_EMULATION>=2011)
//! If compiler provides support for type traits (type intrinsics category)
#define CXXO_COMPILER_SUPPORT_type_traits_intrinsic (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for Unicode literals
#define CXXO_COMPILER_SUPPORT_unicode_string_literals (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for user-defined-literals.
#define CXXO_COMPILER_SUPPORT_user_defined_literals (CXXOMFORT_CXX_STD>=2011)
#if  defined(__cpp_user_defined_literals)
	#undef  CXXO_COMPILER_SUPPORT_user_defined_literals
	#define CXXO_COMPILER_SUPPORT_user_defined_literals 1
#endif
//! If compiler supports user defined literal declarations without the lexer space after ""
#define CXXO_COMPILER_SUPPORT_udl_lexer_nospace 0
//! If compiler provides support for variadic templates.
#define CXXO_COMPILER_SUPPORT_variadic (CXXOMFORT_CXX_STD>=2011)
#if defined(__cpp_variadic_templates)
	#undef  CXXO_COMPILER_SUPPORT_variadic
	#define CXXO_COMPILER_SUPPORT_variadic 1
#endif


//! If compiler provides support for alignment tools (@c alignas , @c aligned_storage , etc).
#define CXXO_COMPILER_SUPPORT_alignment_tools 0
//! If compiler provides support for variadic common_type
#define CXXO_COMPILER_SUPPORT_common_type (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for the new C++11 helpers std::enable_if, std::conditional.
#define CXXO_COMPILER_SUPPORT_conditional_enableif (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for the new C++11 helpers std::begin, std::end.
#define CXXO_COMPILER_SUPPORT_iterator_accessors_11 (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for the new C++11 std algorithms.
#define CXXO_COMPILER_SUPPORT_std_cxx11_algorithms (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for the C++11-onward std::hash
#define CXXO_COMPILER_SUPPORT_std_hash (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for std::hash<std::string> etc
#define CXXO_COMPILER_SUPPORT_std_hash_basicstring (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for std::hash<std::error_code>
#define CXXO_COMPILER_SUPPORT_std_hash_errorcode (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for std::hash<std::type_index> etc
#define CXXO_COMPILER_SUPPORT_std_hash_typeindex (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for std::next, std::prev.
#define CXXO_COMPILER_SUPPORT_std_iterator_helpers_next_prev (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for the "is_constructible", "is_assignable", and similar traits.
#define CXXO_COMPILER_SUPPORT_std_is_constructible_traits (CXXOMFORT_CXX_STD>=2011)
//! If compiler provides support for @c unique_ptr<T> .
#define CXXO_COMPILER_SUPPORT_unique_ptr (CXXOMFORT_CXX_STD>=2011)

// 
// c++14 stuff
// here we can mostly use SD-6
// 

//! If compiler provides support for C++14's relaxed form of @c constexpr .
#define CXXO_COMPILER_SUPPORT_constexpr_relaxed (CXXOMFORT_CXX_STD>=2014)
#if (defined(__cpp_constexpr) && __cpp_constexpr >= 201304)
	#undef  CXXO_COMPILER_SUPPORT_constexpr_relaxed
	#define CXXO_COMPILER_SUPPORT_constexpr_relaxed 1
#endif

//! If compiler provides support for C++14's auto return type deduction
#if (defined(__cpp_return_type_deduction))
	#define CXXO_COMPILER_SUPPORT_auto_return 1
#else
	#define CXXO_COMPILER_SUPPORT_auto_return 0
#endif

//! If compiler provides support for C++14's "decltype(auto)"
#if defined(__cpp_decltype_auto)
	#define CXXO_COMPILER_SUPPORT_decltypeauto 1
#else
	#define CXXO_COMPILER_SUPPORT_decltypeauto 0
#endif

//! If compiler provides support for generic lambdas
#if (defined(__cpp_generic_lambdas))
	#define CXXO_COMPILER_SUPPORT_lambda_expressions_generic  1
#else
	#define CXXO_COMPILER_SUPPORT_lambda_expressions_generic  0
#endif

#define CXXO_COMPILER_SUPPORT_attribute_deprecated 0
#if (CXXOMFORT_CXX_STD>=2011 && defined(__has_cpp_attribute))
	#if (__has_cpp_attribute(deprecated))
		#undef  CXXO_COMPILER_SUPPORT_attribute_deprecated
		#define CXXO_COMPILER_SUPPORT_attribute_deprecated 1
	#endif
#endif

//! If compiler provides support for C++14's nonstatic data member init (NSDMI)
#define CXXO_COMPILER_SUPPORT_nsdmi (CXXOMFORT_CXX_STD>=2014)

//! If compiler provides support for "integer_sequence".
#define CXXO_COMPILER_SUPPORT_integer_sequence (CXXOMFORT_CXX_STD>=2014)
//! If compiler supports C++14's nonmember iterator accessors (rbegin, rend)
#define CXXO_COMPILER_SUPPORT_iterator_accessors_14 (CXXOMFORT_CXX_STD>=2014)
//! If compiler provides support for transparent std::plus, etc.
#define CXXO_COMPILER_SUPPORT_transparent_operators_functional (CXXOMFORT_CXX_STD>=2014)
//! If compiler provides support for C++14's tuple get<type>
#define CXXO_COMPILER_SUPPORT_tuple_get_type 0
//! If compiler provides support for C++14's operator"" s
// use __cpp_lib_string_udls for detection
#define CXXO_COMPILER_SUPPORT_udl_s_basic_string 0
//! If compiler provides support for the two-form full ranges equal/mismatch 
#define CXXO_COMPILER_SUPPORT_equal_mismatch_full_ranges 0

// 
// c++17 stuff
// we can use SD-6 for almost everything that is lib-provided, too
// 

// __has_include
#define CXXO_COMPILER_SUPPORT_macro_has_include (CXXOMFORT_CXX_STD>=2017)

//! If compiler provides a <charconv> header
#define CXXO_CXXSTDHEADER_charconv 0
//! If compiler provides a <string_view> header
#define CXXO_CXXSTDHEADER_string_view 0
//! If compiler provides a <any> header
#define CXXO_CXXSTDHEADER_any 0
//! If compiler provides a <optional> header
#define CXXO_CXXSTDHEADER_optional 0
//! If compiler provides a <variant> header
#define CXXO_CXXSTDHEADER_variant 0

//! If compiler provides support for attribute [[nodiscard]]
#define CXXO_COMPILER_SUPPORT_attribute_nodiscard 0
#if (CXXOMFORT_CXX_STD>=2014 && defined (__has_cpp_attribute))
	#if (__has_cpp_attribute(nodiscard))
		#undef  CXXO_COMPILER_SUPPORT_attribute_nodiscard
		#define CXXO_COMPILER_SUPPORT_attribute_nodiscard 1
	#endif
#endif
//! If compiler supports @c static_assert with no message (n3928)
#define CXXO_COMPILER_SUPPORT_static_assert_17 (CXXOMFORT_CXX_STD >= 2017)
#if (defined(__cpp_static_assert) && (__cpp_static_assert>=201411))
	#undef  CXXO_COMPILER_SUPPORT_static_assert_17
	#define CXXO_COMPILER_SUPPORT_static_assert_17 1
#endif
//! If compiler supports C++17's bool_constant 
#define CXXO_COMPILER_SUPPORT_bool_constant 0
//! if compiler supports inline variables as per p0386
#define CXXO_COMPILER_SUPPORT_inline_variables 0
//! If compiler supports C++17's clamp()
#define CXXO_COMPILER_SUPPORT_std_clamp 0
//! If compiler supports C++17's nonmember iterator accessors
#define CXXO_COMPILER_SUPPORT_iterator_accessors_17 (CXXOMFORT_CXX_STD>=2017)
//! If compiler supports C++17's std::byte
#define CXXO_COMPILER_SUPPORT_std_byte 0
#if (CXXOMFORT_CXX_STD >= 2011 && defined(__cpp_lib_byte))
	#undef  CXXO_COMPILER_SUPPORT_std_byte
	#define CXXO_COMPILER_SUPPORT_std_byte 1
#endif
//! If compiler supports C++17's std::invoke
#define CXXO_COMPILER_SUPPORT_invoke 0
//! If compiler supports C++17's operator"" sv
#define CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view (CXXOMFORT_CXX_STD>=2017)
//! If compiler provides support for std::hash<std::string_view> etc
#define CXXO_COMPILER_SUPPORT_std_hash_basicstringview (CXXOMFORT_CXX_STD>=2017)
//! If compiler provides support for std::hash<std::optional>
#define CXXO_COMPILER_SUPPORT_std_hash_optional 0
//! If compiler provides support for std::hash<std::variant>
#define CXXO_COMPILER_SUPPORT_std_hash_variant 0

// c++20 stuff

//! If compiler provides a <span> header
#define CXXO_CXXSTDHEADER_span (CXXOMFORT_CXX_STD>=2020)
//! If compiler provides a <endian> header
#define CXXO_CXXSTDHEADER_endian (CXXOMFORT_CXX_STD>=2020)

//! If compiler supports signed integer as twos complement (p1236)
#define CXXO_COMPILER_SUPPORT_twos_complement (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports shift algorithms
#define CXXO_COMPILER_SUPPORT_algorithm_shift 0
//! If compiler supports attribute [[likely]]
#define CXXO_COMPILER_SUPPORT_attribute_likely (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports @c bit_cast() 
#define CXXO_COMPILER_SUPPORT_bit_cast (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports @c constexpr for algorithms in <numeric>
#define CXXO_COMPILER_SUPPORT_constexpr_numeric (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports destroying @c delete .
#define CXXO_COMPILER_SUPPORT_destroying_delete (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports <tt>explicit(condition)</tt>
#define CXXO_COMPILER_SUPPORT_explicit_condition (CXXOMFORT_CXX_STD>=2020)
//! If compiler supports spaceship operator
#define CXXO_COMPILER_SUPPORT_spaceship (CXXOMFORT_CXX_STD>=2020)

//! If compiler supports @c bind_front() 
#define CXXO_COMPILER_SUPPORT_bind_front 0
//! If compiler supports @c endian 
#define CXXO_COMPILER_SUPPORT_endian (CXXOMFORT_CXX_STD>=2020)


// c++23 stuff

//! If compiler supports C++17's std::invoke
#define CXXO_COMPILER_SUPPORT_invoke_r 0


//------------------------------------------------
/*
 * List of C++ features for documentation
 * 
 * Good source: https://wg21.link/index.txt
 * 
 * c++03:
 * c++11:
 * 
 * n1377 - move semantics
 * n1381, n1604, n1720 - static_assert
 * n1382 - tuple
 * n1424 - type_traits
 * n1431 - shared_ptr
 * n1436, n1453 - reference_wrapper<> with function call operator
 * n1443 - std::hash and hash_set
 * n1454 - result_of
 * n1478, n1527 - decltype
 * n1479 - array
 * n1488, n1601, n2214, n2431 - null pointer constant
 * n1511 - literals for user-defined types https://wg21.link/n1511
 * n1811 - long long
 * n1836 - built-in type traits
 * n1978, n2343 - decltype
 * n1984, N2546 - new 'auto' semantics
 * n1986 - delegating constructors
 * n1990 - minmax
 * n2070 - decay
 * n2118 - rvalue references, std::move
 * n2235 - constexpr (c++11)
 * n2240 - enable_if and conditional
 * n2243, n2930 - range-based for loop
 * n2249 - new character types
 * n2258 - template aliases ("using =")
 * n2333, n2437 - explicit conversion operators
 * n2341 - alignment (alignof, alignas, aligned_storage, aligned_union)
 * n2346 - defaulted and deleted members
 * n2347 - scoped enums
 * n2349 - new character types (char16, char32)
 * n2439 - rvalue references for *this
 * n2442 - raw string literals
 * n2540 - inheriting constructors
 * n2541 - trailing return type for functions (->)
 * n2555 - variadic templates
 * n2569 - any, all, none algorithms; lexicographical compare 3-way
 * n2657 - local types as template arguments
 * n2672 - initializer_list
 * n2761 - attributes
 * n2765 - user-defined literals
 * n2844 - rvalue references 2.0
 * n2947 - additional C++0x traits (is_trivially... and is_literal...)
 * 
 * c++14:
 * 
 * n3421 - functional's transparent operators
 * n3584, n3670 - get<>(std:tuple) by type
 * n3652 - relaxed constexpr (c++14)
 * n3656 - make_unique
 * n3638 - return type deduction for normal functions
 * n3654 - quoted
 * n3658 - integer_sequence
 * n3668 - exchange
 * n3671 - improved equal, mismatch
 * n3829 - apply (call a function with a tuple as pack of arguments)
 * 
 * c++17:
 * 
 * n3840, n4282
 * n3840, n4282 - observer_ptr, the world's dumbest smart pointer
 * n3928 - extended static_assert
 * n3911 - make_void, void_t
 * n4061 - gcd, lcm
 * n4076 - not_fn
 * n4380 - as_const
 * 
 * external proposals as of Aug 2017:
 * 
 * n1878, n3527 - optional<T>, nullable type wrapper
 * n2648 - dynarray<T>, ctor-sized vector (functionally later fixed_vector)
 * n3334 - array_ref<T>, range view over a contiguous sequence
 * n3350 - std::range (minimum proposal)
 * n3500 - new assert variants
 * n4017 - nonmember size (<iterator>)
 * n3928 - static_assert without message
 * p0005r2 - not_fn
 * p0007r1 - as_const
 * 
 
 * 
 */



// basic

#define CXXOMFORT_IMPLEMENTS_explicit_operator 0
#define CXXOMFORT_IMPLEMENTS_cstdint_integrals 0
#define CXXOMFORT_IMPLEMENTS_nullptr 0
#define CXXOMFORT_IMPLEMENTS_static_assert 0

// compiler details

#define CXXOMFORT_IMPLEMENTS_decltype 0
#define CXXOMFORT_IMPLEMENTS_typeof 0

// standards

// c++03 / TR1
#define CXXOMFORT_IMPLEMENTS_array 0
#define CXXOMFORT_IMPLEMENTS_mem_fn 0
#define CXXOMFORT_IMPLEMENTS_minmax 0
#define CXXOMFORT_IMPLEMENTS_ranlux 0
#define CXXOMFORT_IMPLEMENTS_reference_wrapper 0
#define CXXOMFORT_IMPLEMENTS_tuple 0

// c++11
#define CXXOMFORT_IMPLEMENTS_aligned_storage 0
#define CXXOMFORT_IMPLEMENTS_attribute 0
#define CXXOMFORT_IMPLEMENTS_algorithm_allanynoneof 0
#define CXXOMFORT_IMPLEMENTS_algorithm_minmax 0
#define CXXOMFORT_IMPLEMENTS_algorithm_sorted 0
#define CXXOMFORT_IMPLEMENTS_common_type 0
#define CXXOMFORT_IMPLEMENTS_common_type_fix 0
#define CXXOMFORT_IMPLEMENTS_decay 0
#define CXXOMFORT_IMPLEMENTS_integral_constant 0
#define CXXOMFORT_IMPLEMENTS_integral_constant_callable 0
#define CXXOMFORT_IMPLEMENTS_typeindex 0
#define CXXOMFORT_IMPLEMENTS_underlying_type 0
#define CXXOMFORT_IMPLEMENTS_unique_ptr 0
#define CXXOMFORT_IMPLEMENTS_stdhash_string 0

// c++14
#define CXXOMFORT_IMPLEMENTS_equal_mismatch_full_ranges 0
#define CXXOMFORT_IMPLEMENTS_heterogeneous_operator_functors 0
#define CXXOMFORT_IMPLEMENTS_integer_sequence 0
#define CXXOMFORT_IMPLEMENTS_make_unique 0
#define CXXOMFORT_IMPLEMENTS_tuple_get_type 0
#define CXXOMFORT_IMPLEMENTS_udl_s 0
#define CXXOMFORT_IMPLEMENTS_stdhash_enum 0

// c++17
#define CXXOMFORT_IMPLEMENTS_bool_constant 0
#define CXXOMFORT_IMPLEMENTS_byte 0
#define CXXOMFORT_IMPLEMENTS_destroy_at 0
#define CXXOMFORT_IMPLEMENTS_invoke 0
#define CXXOMFORT_IMPLEMENTS_make_from_tuple 0
#define CXXOMFORT_IMPLEMENTS_ssize 0
#define CXXOMFORT_IMPLEMENTS_string_view 0
#define CXXOMFORT_IMPLEMENTS_std_void_t 0
#define CXXOMFORT_IMPLEMENTS_timespec_get 0
#define CXXOMFORT_IMPLEMENTS_tuple_apply 0
#define CXXOMFORT_IMPLEMENTS_udl_sv 0
#define CXXOMFORT_IMPLEMENTS_static_assert_17 0
#define CXXOMFORT_IMPLEMENTS_stdhash_string_view 0

// c++20
#define CXXOMFORT_IMPLEMENTS_algorithm_shift 0
#define CXXOMFORT_IMPLEMENTS_bind_front 0
#define CXXOMFORT_IMPLEMENTS_std_remove_cvref 0
#define CXXOMFORT_IMPLEMENTS_constexpr_numeric 0

// c++23
#define CXXOMFORT_IMPLEMENTS_is_scoped_enum 0

// fixes
#define CXXOMFORT_IMPLEMENTS_smartptr_accessors 0
#define CXXOMFORT_IMPLEMENTS_typetrait_accessors 0
#define CXXOMFORT_IMPLEMENTS_random_substract_fix 0

#endif
