#ifndef CXXOMFORT_CONFIG_CLANG_HPP
#define CXXOMFORT_CONFIG_CLANG_HPP
#ifndef CXXOMFORT_CONFIG_HPP
    #error "This file must not be included directly (see cxxomfort/config.hpp)"
#endif

/*
 * Clang version history
 * https://en.wikipedia.org/wiki/LLVM
 * 
 * 6.0  	March 8, 2018
 * 4.0  	March 13, 2017
 * 3.8  	March 8, 2016
 * 3.6.2 	July 16, 2015 
 * 3.4.2 	June 19, 2014 **preferred, in Trusty dev**
 * 3.2 	December 20, 2012 **preferred**
 * 3.0 	December 1, 2011
 * 2.8 	October 5, 2010 
 * 2.6 	October 23, 2009 
 * 2.4  ????
 * * 
 * 
Here follows a list of C++11 features for Clang:

TODO
 */

#define CXXOMFORT_COMPILER_ID CXXO_COMPILERID_CLANG
#define CXXOMFORT_COMPILER_VERSION 100*__clang_major__+__clang_minor__
#define CXXOMFORT_COMPILER_VERSION_FULL 1000*__clang_major__+10*__clang_minor__+__clang_patchlevel__


#define CXXO_PRAGMA(x) _Pragma (#x)
//#define CXXO_NOTICE(msg) GCC message (" cxxomfort warning -- " msg )
//#define CXXO_WARNING(msg) CXXO_NOTICE_IMPL_( "cxxomfort warning: " msg)

#undef  CXXO_NOTICE_IMPL_
#define CXXO_NOTICE_IMPL_(msg) msg
#undef  CXXO_NOTICE
#define CXXO_NOTICE(msg) CXXO_NOTICE_IMPL_( "cxxomfort notice: " msg)
#undef  CXXO_WARNING
#define CXXO_WARNING(msg) CXXO_NOTICE_IMPL_( "cxxomfort warning: " msg)

#if defined(CXXOMFORT_NOTICES)
    #pragma message CXXO_NOTICE("Found the Clang C++ Compiler.")
#endif

#if (defined(CXXOMFORT_NOTICES) && defined(_LIBCPP_VERSION))
	#pragma message CXXO_WARNING("with libcxx")
#elif (defined(CXXOMFORT_NOTICES) && defined(__GLIBCXX_))
	#pragma message CXXO_WARNING("with gnu stdlibc++")

#endif

//
// compatibility detail for older clang as shown in 
// http://clang.llvm.org/docs/LanguageExtensions.html#feature-checking-macros
//
#if (CXXOMFORT_COMPILER_VERSION<300)
    #ifndef __has_extension
        #define __has_extension __has_feature
    #endif
#endif

#if (CXXOMFORT_COMPILER_VERSION>=300)
	#undef  CXXO_COMPILER_SUPPORT_expression_sfinae
	#define CXXO_COMPILER_SUPPORT_expression_sfinae 1
	#undef  CXXO_COMPILER_SUPPORT_macro_has_include
	#define CXXO_COMPILER_SUPPORT_macro_has_include 1
#endif

#undef  CXXO_COMPILER_SUPPORT___alignof
#define CXXO_COMPILER_SUPPORT___alignof 1
#undef  CXXO_COMPILER_SUPPORT_type_intrinsics
#define CXXO_COMPILER_SUPPORT_type_intrinsics 1
#undef  CXXO_COMPILER_SUPPORT_typeof
#define CXXO_COMPILER_SUPPORT_typeof 1
#undef  CXXO_COMPILER_SUPPORT_va_args
#define CXXO_COMPILER_SUPPORT_va_args 1

// c++03

#if (CXXOMFORT_CXX_STD >= 1997 && CXXOMFORT_CXX_STD < 2011 && CXXOMFORT_COMPILER_VERSION >= 300)
    // here it should use the same paths as GCC in theory?
    // (pending check against libcpp mode however)
    #undef  CXXOFLAG_tr1
    #define CXXOFLAG_tr1 1
    #undef  CXXOFLAG_directory_tr1
    #define CXXOFLAG_directory_tr1 1
    #undef  CXXOFLAG_namespace_tr1
    #define CXXOFLAG_namespace_tr1 1

    #undef  CXXO_STDTR1_NAME
    #define CXXO_STDTR1_NAME() std::tr1
    #undef  CXXO_STDTR1_OPEN
    #define CXXO_STDTR1_OPEN() namespace std { namespace tr1 {
    #undef  CXXO_STDTR1_CLOSE
    #define CXXO_STDTR1_CLOSE() } }
    
    #undef  CXXO_CXXSTDHEADER_tr1_array
    #define CXXO_CXXSTDHEADER_tr1_array (__has_include(<tr1/array>))
    #undef  CXXO_CXXSTDHEADER_tr1_tuple
    #define CXXO_CXXSTDHEADER_tr1_tuple (__has_include(<tr1/tuple>))
    #undef  CXXO_CXXSTDHEADER_tr1_type_traits
    #define CXXO_CXXSTDHEADER_tr1_type_traits (__has_include(<tr1/type_traits>))

#endif

#undef  CXXO_COMPILER_SUPPORT_tr1_array
#define CXXO_COMPILER_SUPPORT_tr1_array CXXO_CXXSTDHEADER_tr1_array
#undef  CXXO_COMPILER_SUPPORT_tr1_tuple
#define CXXO_COMPILER_SUPPORT_tr1_tuple CXXO_CXXSTDHEADER_tr1_tuple
#undef  CXXO_COMPILER_SUPPORT_tr1_type_traits
#define CXXO_COMPILER_SUPPORT_tr1_type_traits CXXO_CXXSTDHEADER_tr1_type_traits


// c++11

#if (CXXO_CXXSTDHEADER_array==0 && defined(__has_include) && __has_include(<array>))
    #undef  CXXO_CXXSTDHEADER_array
    #define CXXO_CXXSTDHEADER_array (CXXOMFORT_CXX_STD>=2011)
#endif

#undef  CXXO_COMPILER_SUPPORT_alignof
#define CXXO_COMPILER_SUPPORT_alignof (CXXOMFORT_CXX_STD >= 2011 || CXXOMFORT_COMPILER_VERSION>=209)
#undef  CXXO_COMPILER_SUPPORT_alignment_tools
#define CXXO_COMPILER_SUPPORT_alignment_tools __has_extension(cxx_alignas)
#undef  CXXO_COMPILER_SUPPORT_attribute
#define CXXO_COMPILER_SUPPORT_attribute __has_feature(cxx_attributes)
#undef  CXXO_COMPILER_SUPPORT_auto
#define CXXO_COMPILER_SUPPORT_auto __has_feature(cxx_auto_type)
#undef  CXXO_COMPILER_SUPPORT_constexpr
#define CXXO_COMPILER_SUPPORT_constexpr __has_feature(cxx_constexpr)
#if (CXXOMFORT_CXX_STD>=2011 && CXXOMFORT_COMPILER_VERSION>=308 && CXXO_COMPILER_SUPPORT_constexpr_dereference==0)
	#undef  CXXO_COMPILER_SUPPORT_constexpr_dereference
	#define CXXO_COMPILER_SUPPORT_constexpr_dereference 1
#endif
#undef  CXXO_COMPILER_SUPPORT_decltype
#define CXXO_COMPILER_SUPPORT_decltype __has_feature(cxx_decltype_incomplete_return_types)
#undef  CXXO_COMPILER_SUPPORT_default_delete
#define CXXO_COMPILER_SUPPORT_default_delete (__has_extension(cxx_defaulted_functions) * __has_extension(cxx_deleted_functions))
#undef  CXXO_COMPILER_SUPPORT_enum_class
#define CXXO_COMPILER_SUPPORT_enum_class __has_feature(cxx_strong_enums)
#undef  CXXO_COMPILER_SUPPORT_explicit_operator
#define CXXO_COMPILER_SUPPORT_explicit_operator __has_feature(cxx_explicit_conversions)
#undef  CXXO_COMPILER_SUPPORT_foreach
#define CXXO_COMPILER_SUPPORT_foreach __has_feature(cxx_range_for)
#undef  CXXO_COMPILER_SUPPORT_initializer_list
#define CXXO_COMPILER_SUPPORT_initializer_list __has_feature(cxx_inheriting_constructors)
#undef  CXXO_COMPILER_SUPPORT_local_types
#define CXXO_COMPILER_SUPPORT_local_types __has_extension(cxx_local_type_template_args)
#undef  CXXO_COMPILER_SUPPORT_noexcept
#define CXXO_COMPILER_SUPPORT_noexcept __has_extension(cxx_noexcept)
#undef  CXXO_COMPILER_SUPPORT_nsdmi
#define CXXO_COMPILER_SUPPORT_nsdmi __has_feature(cxx_nonstatic_member_init)
#undef  CXXO_COMPILER_SUPPORT_nullptr
#define CXXO_COMPILER_SUPPORT_nullptr __has_extension(cxx_nullptr)
#undef  CXXO_COMPILER_SUPPORT_rvref
#define CXXO_COMPILER_SUPPORT_rvref (__has_extension(cxx_rvalue_references) * __has_extension(cxx_implicit_moves))
#undef  CXXO_COMPILER_SUPPORT_std_hash
#define CXXO_COMPILER_SUPPORT_std_hash (CXXOMFORT_CXX_STD>= 1997 )
#undef  CXXO_COMPILER_SUPPORT_std_is_trivially
#define CXXO_COMPILER_SUPPORT_std_is_trivially (CXXOMFORT_CXX_STD >= 2011)
#undef  CXXO_COMPILER_SUPPORT_static_assert
#define CXXO_COMPILER_SUPPORT_static_assert __has_extension(cxx_static_assert)
#undef  CXXO_COMPILER_SUPPORT_trailing_returns
#define CXXO_COMPILER_SUPPORT_trailing_returns __has_extension(cxx_trailing_return)
#undef  CXXO_COMPILER_SUPPORT_unique_ptr
#define CXXO_COMPILER_SUPPORT_unique_ptr (CXXOMFORT_CXX_STD >= 2011)
#undef  CXXO_COMPILER_SUPPORT_variadic
#define CXXO_COMPILER_SUPPORT_variadic (__has_feature(cxx_variadic_templates))
#undef  CXXO_COMPILER_SUPPORT_va_args
#define CXXO_COMPILER_SUPPORT_va_args 1

#undef  CXXO_COMPILER_SUPPORT_udl_lexer_nospace
#if (defined(__cpp_user_defined_literals) && __has_feature(user_defined_literals))
	#define CXXO_COMPILER_SUPPORT_udl_lexer_nospace 1
#else
	#define CXXO_COMPILER_SUPPORT_udl_lexer_nospace 0
#endif

// c++14

#undef  CXXO_COMPILER_SUPPORT_decltypeauto
#define CXXO_COMPILER_SUPPORT_decltypeauto __has_extension(cxx_decltype_auto)
#undef  CXXO_COMPILER_SUPPORT_constexpr_relaxed
#define CXXO_COMPILER_SUPPORT_constexpr_relaxed __has_feature(cxx_relaxed_constexpr)

// c++17

// see https://stackoverflow.com/questions/56843413/stdbyte-is-not-member-of-std re: std::byte in Clang
#if (defined(_LIBCPP_VERSION) && CXXOMFORT_CXX_STD<2017)
#elif (defined(__GLIBCXX__) && CXXOMFORT_CXX_STD==2017 && CXXOMFORT_COMPILER_VERSION<700)
    #undef  CXXO_COMPILER_SUPPORT_std_byte
    #define CXXO_COMPILER_SUPPORT_std_byte 0
#endif

#if (CXXOMFORT_CXX_STD==2017 && defined(__GLIBCXX__))
	#undef  CXXO_CXXSTDHEADER_string_view
	#define CXXO_CXXSTDHEADER_string_view 0
	#undef  CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view
	#define CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view 0
#elif (CXXOMFORT_CXX_STD>=2011 && defined(_LIBCPP_VERSION) && _LIBCPP_VERSION>=8000)
    #undef  CXXO_CXXSTDHEADER_string_view
    #define CXXO_CXXSTDHEADER_string_view 1
	#undef  CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view
	#define CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view (CXXOMFORT_CXX_STD>=2014)
#endif

#if (CXXO_CXXSTDHEADER_any==0 && defined(__has_include) && __has_include(<any>))
    #undef  CXXO_CXXSTDHEADER_any
    #define CXXO_CXXSTDHEADER_any (1)
#endif

#if (CXXO_CXXSTDHEADER_optional==0 && defined(__has_include) && __has_include(<optional>))
    #undef  CXXO_CXXSTDHEADER_optional
    #define CXXO_CXXSTDHEADER_optional (1)
#endif

#if (CXXO_CXXSTDHEADER_variant==0 && defined(__has_include) && __has_include(<variant>))
    #undef  CXXO_CXXSTDHEADER_variant
    #define CXXO_CXXSTDHEADER_variant (1)
#endif

// c++20

#undef  CXXO_COMPILER_SUPPORT_destroying_delete
#define CXXO_COMPILER_SUPPORT_destroying_delete (__has_feature(destroying_delete))
#undef  CXXO_COMPILER_SUPPORT_explicit_condition
#define CXXO_COMPILER_SUPPORT_explicit_condition (__has_feature(explicit_conditional))

// c++23

// -- NEW TODO -- get my claws on clang >= 8.0 (2022-03)

#if (CXXOMFORT_COMPILER_VERSION > 300 && CXXOMFORT_COMPILER_VERSION < 400)
    #if (CXXO_CXXSTDHEADER_cuchar)
        #undef  CXXO_CXXSTDHEADER_cuchar
        #define CXXO_CXXSTDHEADER_cuchar 0
    #endif
#endif

// treats ::max_align_t differently depending on stdlib or libc++
#if (CXXOMFORT_CXX_STD<2011)
    #if (defined(_LIBCPP))
    #else
    typedef long double max_align_t;
    #endif
#endif

// treats uchar differently depending on stdlib or libc++

#if (CXXOMFORT_COMPILER_VERSION >= 306)
    #undef  CXXO_COMPILER_SUPPORT__Pragma
    #define CXXO_COMPILER_SUPPORT__Pragma 1
#endif

#if (CXXOMFORT_COMPILER_VERSION >= 400)
    #undef  CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF
    #define CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF 1
#elif (!defined(_LIBCPP))
    #undef  CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF
    #define CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF 1
#endif

// we can have here both decltype and typeof

/* 
 * BUGFIX: Clang 3.0-3.4 + libstdc++ triggers an error in the GCC 
 * headers that use the name of type_info without declaring it before.
 * https://bugzilla.redhat.com/show_bug.cgi?id=1129899
 * */

#if (CXXOMFORT_COMPILER_VERSION>=300 && CXXOMFORT_COMPILER_VERSION<306)
    #if (CXXOMFORT_CXX_STD==2011 && !defined(_LIBCPP))
namespace std {
    class type_info;
}
    #endif
#endif

// Clang 2.8, 3.0 + libstdc++ wrongly announces support for nullptr_t; 
// it still has nullptr, but not nullptr_t
// example related: https://github.com/catchorg/Catch2/issues/487
#if (CXXOMFORT_COMPILER_VERSION>=208 && CXXOMFORT_COMPILER_VERSION<302)
    #if (CXXOMFORT_CXX_STD==2011 && !defined(_LIBCPP))
namespace std {
    typedef decltype(nullptr) nullptr_t;
    //using ::nullptr_t;
}
    #endif
#endif

// libcpp is missing cuchar in some versions
#if (defined(_LIBCPP))
	#undef  CXXO_CXXSTDHEADER_cuchar
	#define CXXO_CXXSTDHEADER_cuchar 0
#endif

// enable the three exclusive __cpp macros
#if (!defined(__cpp_decltype) && CXXO_COMPILER_SUPPORT_decltype>0)
	#define __cpp_decltype (CXXOMFORT_CXX_STD*100)
#endif
#if (!defined(__cpp_rvalue_references) && CXXO_COMPILER_SUPPORT_rvref>0)
	#define __cpp_rvalue_references (CXXOMFORT_CXX_STD*100)
#endif
#if (!defined(__cpp_user_defined_literals) && CXXO_COMPILER_SUPPORT_user_defined_literals>0)
	#define __cpp_user_defined_literals (CXXOMFORT_CXX_STD*100)
#endif


#endif
