#ifndef CXXOMFORT_CONFIG_HPP
    #error "This file must not be included directly (see cxxomfort/config.hpp)"
#endif
#ifndef CXXOMFORT_CONFIG_DMC_HPP
#define CXXOMFORT_CONFIG_DMC_HPP

// from https://sourceforge.net/p/predef/wiki/Compilers/
#define CXXOMFORT_COMPILER_ID CXXO_COMPILERID_DMC
#define CXXOMFORT_COMPILER_VERSION (__DMC__)
#define CXXOMFORT_COMPILER_VERSION_FULL (__DMC__)

/*
 * See: https://digitalmars.com/ctg/CPP-Language-Implementation.html#cpp_language
 * Digital Mars uses the __cplusplus value of 199711L, but they indicate the point where they are not C++98 conformant. Since those points include key stuff like alignment, templates and certain keywords like 'new' and 'inline', for the purposes of Cxxomfort they should be considered a compiler with less-than C++98 support.
 * */
#if (__DMC__<0x900)
    #undef  CXXOMFORT_CXX_STD
    #define CXXOMFORT_CXX_STD 1996
    #undef  CXXOMFORT_CXX_EMULATION
    #define CXXOMFORT_CXX_EMULATION 1997
#endif

#define CXXO_PRAGMA(x) _Pragma (#x)

#undef  CXXO_NOTICE_IMPL_ 
#define CXXO_NOTICE_IMPL_(msg) msg
#undef  CXXO_NOTICE
#define CXXO_NOTICE(msg) CXXO_NOTICE_IMPL_( "cxxomfort notice: " msg)


#if defined(CXXOMFORT_NOTICES)
    #pragma message CXXO_NOTICE("Found the Digital Mars Compiler [" CXXO_STRINGIZE(__DMC__) "]" )
#endif

#if (CXXOMFORT_COMPILER_VERSION >= 0x848)
    #undef  CXXO_COMPILER_SUPPORT_typeof
    #define CXXO_COMPILER_SUPPORT_typeof 1
    #undef  CXXO_TYPEOF
    #define CXXO_TYPEOF(...) typeof(__VA_ARGS__)
    
    #undef  CXXO_COMPILER_SUPPORT_std_hash
    #define CXXO_COMPILER_SUPPORT_std_hash 1
#endif

// 
// versions 8.5x onward define nullptr
// 

#if (CXXOMFORT_COMPILER_VERSION >= 0x852)
	#pragma CXXO_NOTICE("nullptr")
    #undef  CXXO_COMPILER_SUPPORT_nullptr
    #define CXXO_COMPILER_SUPPORT_nullptr 1
    #include <cstddef>
namespace std {
    using ::nullptr_t;
}

    #undef  CXXO_COMPILER_SUPPORT_static_assert
    #define CXXO_COMPILER_SUPPORT_static_assert 1
    #undef  CXXO_COMPILER_SUPPORT_raw_strings
    #define CXXO_COMPILER_SUPPORT_raw_strings 1
    #undef  CXXO_COMPILER_SUPPORT_unicode_string_literals
    #define CXXO_COMPILER_SUPPORT_unicode_string_literals 1

#endif

#if (CXXOMFORT_COMPILER_VERSION >= 0x857)
    #undef  CXXO_COMPILER_SUPPORT_charn_t
    #define CXXO_COMPILER_SUPPORT_charn_t 1
    #define CXXO_COMPILER_SUPPORT_charn_t_IS_TYPEDEF 0
    #undef  CXXO_COMPILER_SUPPORT_decltype
    #define CXXO_COMPILER_SUPPORT_decltype 1
#endif

// setting the three exclusive __cpp macros
#if (!defined(__cpp_decltype) && CXXO_COMPILER_SUPPORT_decltype > 0)
	#define __cpp_decltype (CXXOMFORT_CXX_STD*100)
#endif
#if (!defined(__cpp_rvalue_references) && CXXO_COMPILER_SUPPORT_rvref > 0)
	#define __cpp_rvalue_references (CXXOMFORT_CXX_STD*100)
#endif

// 
// DMC < 8.5 has a bug where including both wchar.h and stdint.h cause an error about the WCHAR_MIN,MAX macros.
// 

#endif
