#ifndef CXXOMFORT_CONFIG_HPP
    #error "This file must not be included directly (see cxxomfort/config.hpp)"
#endif
#ifndef CXXOMFORT_CONFIG_OW_HPP
#define CXXOMFORT_CONFIG_OW_HPP

/*
 * OpenWatcom C++ starting from version 1.9, which is the one 
 * released to public, and then 2.0x.
 * 
 * Support status for OpenWatcom is at
 * 
 * https://github.com/open-watcom/open-watcom-v2/wiki/C---Language
 * 
 * OpenWatcom has severely limited template support and as such many 
 * things are not going to work, among others nullptr. 
 * Support for in-declaration initializers is missing as well.
 * Because of this, here we set up some poor-mans-approaches 
 * that we use instead of the ones in base/.
 * 
 * The compiler also lacks intrinsics, in particular the 
 * __is_enum,... family of; partly because of that, the 
 * included implementation of <type_traits> doesn't work because 
 * the trait classes, while they are declared, are not defined.
 * 
 * For this compiler, nothing past the base features 
 * is officially supported. cxxomfort declares 
 * CXXOMFORT_CXX_STD < 1997 ( == 0 or 1 ).
*/

#define CXXOMFORT_COMPILER_ID CXXO_COMPILERID_OW
#define CXXOMFORT_COMPILER_VERSION (__WATCOMC__)
#define CXXOMFORT_COMPILER_VERSION_FULL (__WATCOMC__)
/* Versions - __WATCOMC__ 
 * Release History: https://en.wikipedia.org/wiki/Watcom_C/C%2B%2B
 * 
 * * ???? --> Open Watcom 1.8 (2009-02)
 * * 1290 --> Open Watcom 1.9 (2010-06)
 * * 1300 --> Open Watcom 2.0 beta from the travis-ci (2015-04)
 * 
 * */


#define CXXO_PRAGMA(x) _Pragma (#x)
#undef  CXXO_NOTICE
#define CXXO_NOTICE(msg) ( msg )

#if defined(CXXOMFORT_NOTICES)
	#pragma message ("Found Watcom/OpenWatcom [" CXXO_STRINGIZE1(__WATCOMC__) "]" )
#endif

#if (CXXOMFORT_COMPILER_VERSION_FULL < 1400)
    #define CXXOFLAG_NO_INITIALIZERS 1
#endif

// wait for OWC++ implementors to lower this bound
// for when we don't need fixes
#if (__WATCOMC__ < 9999) 

// static_assert seems to work OK?

// move uses move-00

// integer types are available in native cstdint

// explicit cast... do templates work?

#endif // 9999

#if (__WATCOMC__ >= 1200) // Watcom 1.9 or higher
    #undef  CXXO_CXXSTDHEADER_cstdint
    #define CXXO_CXXSTDHEADER_cstdint 1
    #undef  CXXO_COMPILER_SUPPORT_ssize_t
    #define CXXO_COMPILER_SUPPORT_ssize_t 1 // found in <cstddef>

#endif

#if (__WATCOMC__ >= 1300) // Watcom 2.0 / OpenWatcom or higher
    #undef  CXXO_COMPILER_SUPPORT_std_hash
    #define CXXO_COMPILER_SUPPORT_std_hash 1
#endif

#endif
