#ifndef CXXOMFORT_CONFIG_SD_HPP
#define CXXOMFORT_CONFIG_SD_HPP
/**
 * @file
 * @brief Testing and backport of compiler support (SD-n) macros.
 * 
 * This header complements the tasks in config/_support.hpp and makes available some 
 * macro names defined in SD6 that provide feature-testing for C++.
 * 
 * Interfaces defined in this header:
 * 
 * * @c __cpp_unicode_characters
 * * @c __cpp_range_based_for
 * * @c __cpp_static_assert
 * * @c __cpp_declype
 * * @c __cpp_variadic_templates
 * * @c __cpp_inheriting_constructors
 * 
 * 
 * @see https://isocpp.org/std/standing-documents/sd-6-sg10-feature-test-recommendations
 * 
 */

#include "../config.hpp"
#include "../config/_support.hpp"

// C++ Unicode Characters (C++11 n2249)
#ifdef  __cpp_unicode_characters
	#if __cpp_unicode_characters >= 200704
		#undef  CXXO_COMPILER_SUPPORT_charn_t
		#define CXXO_COMPILER_SUPPORT_charn_t __cpp_unicode_characters
	#endif
#else
	#define __cpp_unicode_characters CXXO_COMPILER_SUPPORT_charn_t
#endif

// C++ Range-based for
#ifdef  __cpp_range_based_for
	#if __cpp_range_based_for >= 200907
		#undef  CXXO_COMPILER_SUPPORT_foreach
		#define CXXO_COMPILER_SUPPORT_foreach __cpp_range_based_for
	#endif
#else
	#define __cpp_range_based_for CXXO_COMPILER_SUPPORT_foreach
#endif

// C++ Static Assert n1720
#ifdef  __cpp_static_assert
	#if __cpp_static_assert >= 200410
		#undef  CXXO_COMPILER_SUPPORT_static_assert
		#define CXXO_COMPILER_SUPPORT_static_assert __cpp_static_assert
	#endif
#else
	#define __cpp_static_assert CXXO_COMPILER_SUPPORT_static_assert
#endif

// C++ Decltype n2343
#ifdef  __cpp_decltype
	#if __cpp_decltype >= 200707
		#undef  CXXO_COMPILER_SUPPORT_decltype
		#define CXXO_COMPILER_SUPPORT_decltype __cpp_decltype
	#endif
#else
	#define __cpp_decltype CXXO_COMPILER_SUPPORT_decltype
#endif

// C++ Variadic Templates n2242
#ifdef  __cpp_variadic_templates
	#if __cpp_variadic_templates >= 200704
		#undef  CXXO_COMPILER_SUPPORT_variadic
		#define CXXO_COMPILER_SUPPORT_variadic __cpp_variadic_templates
	#endif
#else
	#define __cpp_variadic_templates CXXO_COMPILER_SUPPORT_variadic
#endif


// C++ Inheriting Constructors n2540
#ifdef  __cpp_inheriting_constructors
	#if __cpp_inheriting_constructors >= 200704
		#undef  CXXO_COMPILER_SUPPORT_inheriting_constructors
		#define CXXO_COMPILER_SUPPORT_inheriting_constructors __cpp_inheriting_constructors
	#endif
#else
	#define __cpp_inheriting_constructors 0
#endif

// C++ Constexpr
#ifdef  __cpp_constexpr
	#if __cpp_constexpr >= 200704
		#undef  CXXO_COMPILER_SUPPORT_constexpr
		#define CXXO_COMPILER_SUPPORT_constexpr __cpp_constexpr
	#endif
	#if __cpp_constexpr >= 201304 // relaxed
		#undef  CXXO_COMPILER_SUPPORT_constexpr_relaxed
		#define CXXO_COMPILER_SUPPORT_constexpr_relaxed __cpp_constexpr
	#endif
#else
	#define __cpp_constexpr CXXO_COMPILER_SUPPORT_constexpr
#endif

// C++14

// C++ transparent functors (n3421)
#ifdef  __cpp_lib_transparent_operators
	#if __cpp_lib_transparent_operators >= 201210
		#undef  CXXO_COMPILER_SUPPORT_functional_transparent
		#define CXXO_COMPILER_SUPPORT_functional_transparent __cpp_lib_transparent_operators
	#endif
#else
	#define __cpp_lib_transparent_operators CXXO_COMPILER_SUPPORT_functional_transparent
#endif

// C++ integral_constant::operator() (n3545)
#ifdef  __cpp_lib_integral_constant_callable
	#if __cpp_lib_integral_constant_callable >= 201304
		#undef  CXXO_COMPILER_SUPPORT_integral_constant_callable
		#define CXXO_COMPILER_SUPPORT_integral_constant_callable __cpp_lib_integral_constant_callable
	#endif
#else
	#define __cpp_lib_integral_constant_callable CXXO_COMPILER_SUPPORT_integral_constant_callable
#endif

// C++ tuple get<type> (n3670)
#ifdef  __cpp_lib_tuples_by_type
	#if __cpp_lib_tuples_by_type >= 201304
		#undef  CXXO_COMPILER_SUPPORT_integral_constant_callable
		#define CXXO_COMPILER_SUPPORT_integral_constant_callable __cpp_lib_integral_constant_callable
	#endif
#else
	#define __cpp_lib_integral_constant_callable CXXO_COMPILER_SUPPORT_integral_constant_callable
#endif


#endif
