#ifndef CXXOMFORT_CSTDIO_HPP
#define CXXOMFORT_CSTDIO_HPP
/*
 * @file
 * @brief Implementations and additions tied to Standard header <tt>cstdio</tt>.
 * @ingroup std0cstdio
 * 
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/config.hpp>
#include <cstdio>

namespace cxxomfort {

#if defined(DOXYGEN_DOC)
/**
 * @brief Outputs information about cxxomfort configuration
 * @xrefitem cxxomfort-config "" "" Outputs info about cxxomfort configuration.
 * @ingroup std0cstdio
 * */
static inline FILE* 	output_info (std::FILE* const ff= stdout) {

#elif (CXXOMFORT_CXX_STD>=1997)
static inline FILE* 	output_info (std::FILE * const ff= stdout) {
#else
static FILE* 	output_info (FILE * const ff= stdout) {
#endif
    using namespace std;
    fprintf( ff, "(cxxomfort:%lu ", (unsigned long)_.date);
    fprintf( ff, "compiler:%u:%u:%u ", _.compilerid, _.compilerversion, _.compilerversionfull );
    fprintf( ff, "mode:%u:%u cpp:%lu)\n", _.cxx_std, _.cxx_emulation
    , static_cast<unsigned long>(__cplusplus));
    return ff;
}

} // cxxomfort

// 
// lift snprintf in older MSVC compilers 
// 

#if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1800))
#include <cstdarg> // va_start, va_end
namespace std {
	static inline 
	int vsnprintf (char* buffer, size_t buf_size, char const* fmt, va_list ap) {
		int count = -1;
		if (buf_size != 0) {
			count= _vsnprintf_s(buffer, buf_size, _TRUNCATE, fmt, ap);
		}
		if (count == -1) {
			count= _vscprintf(fmt,ap);
		}
		return count;
	}
}
#endif
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1900))
#include <cstdarg>
namespace std {
	static inline 
	int snprintf (char* buffer, size_t buf_size, char const* fmt, ...) {
		int count;
		va_list ap;
		va_start(ap, fmt);
		count = vsnprintf(buffer, buf_size, fmt, ap);
		va_end(ap);
		return count;
	}
}
#endif

#endif

/**
 * @page std0cstdio <cstdio>
 * @brief Features related to Standard header <tt><cstdio></tt>.
 * @author Luis Machuca Bezzaza <luis [dot] machuca [at] gulix [dot] cl>
 * 
 * @code
 * #include <cxxomfort/cstdio.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * @section Interfaces 
 * 
 * Features included in this section and mapped to global nsmespace or <tt>std::</tt>:
 * 
 * * None (at last edition of this documentation)
 * 
 * Non-backport features (<tt>cxxomfort::fix</tt>):
 * 
 * * None (at last edition of this documentation)
 * 
 * */
