#ifndef CXXOMFORT_CUCHAR_HPP
#define CXXOMFORT_CUCHAR_HPP
/**
 * @file
 *
 */
#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>1)
	#pragma message CXXO_NOTICE("enabled <cuchar> support.")
#endif

#include <cxxomfort/base.hpp>
#include <cxxomfort/cxxo-utils.hpp>
#include <cwchar>

#if (defined(__cpp_unicode_characters) || CXXO_COMPILER_SUPPORT_charn_t)
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_NATIVE()
#elif defined(_LIBCPP_VERSION)
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_NATIVE()
	#include <uchar.h>
#elif (CXXO_CXXSTDHEADER_cuchar>0)
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_NATIVE()
	#include <cuchar>
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=209) && CXXOMFORT_CXX_STD>=2011)
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_NATIVE()
	#include <cuchar>
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=408) && CXXOMFORT_CXX_STD>=2011)
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_NATIVE()
	//#include <cuchar>
#else
	#define CXXOMFORT_IMPLEMENTS_charn_types CXXO_IMPLSTATUS_EMULATION()


	#if (defined(CXXOFLAG_MAKE_TYPEDEF_charn_t))
		#define CXXO_charn_t_typedefs 1

// note that these are global!

typedef uint_least16_t char16_t;
typedef uint_least32_t char32_t;
	#else
		#define CXXO_charn_t_typedefs 0

// note that these are global!

struct char16_t {
    public:
    char16_t (uint_least16_t u=0) CXXO_NOEXCEPTNOTHROW 
    : v_(u) {}
    operator 	uint_least16_t () const CXXO_NOEXCEPTNOTHROW { return v_; }
    friend bool 	operator== (char16_t a, char16_t b) CXXO_NOEXCEPTNOTHROW { return a.v_==b.v_; }
    friend bool 	operator< (char16_t a, char16_t b) CXXO_NOEXCEPTNOTHROW { return a.v_<<b.v_; }
    private:
    uint_least16_t v_;
};
CXXO_GENERATE_RELATIONALS(char16_t);

struct char32_t {
    public:
    char32_t (uint_least32_t u=0) CXXO_NOEXCEPTNOTHROW 
    : v_(u) {}
    operator 	uint_least32_t () const CXXO_NOEXCEPTNOTHROW { return v_; }
    friend bool 	operator== (char32_t a, char32_t b) CXXO_NOEXCEPTNOTHROW { return a.v_==b.v_; }
    friend bool 	operator< (char32_t a, char32_t b) CXXO_NOEXCEPTNOTHROW { return a.v_<<b.v_; }
    private:
    uint_least32_t v_;
};
CXXO_GENERATE_RELATIONALS(char32_t);

	#endif

#endif


#endif

/**
 * @page std0cuchar <cuchar>
 * @brief Features related to Standard Header <tt><cuchar></tt>
 * 
 * This section defines the @c char16_t and @c char32_t types in C++03 mode, 
 * besides the flag @c CXXO_charn_t_typedefs that tells whether the names are 
 * made available as typedefs or as their own separate types, depending on the compiler.
 * 
 * The types are made available for storage only; without intervening in the lexer 
 * (and thus in the compiler somehwow), it is not really possible to write 
 * extended char typed chaarcters in the source code, let alone literal strings of them.
 * 
 * @section Interfaces 
 * 
 * Interfaces defined:
 * 
 * * @c char16_t , @c char32_t (C++11) in C++03 if they would be typedef.
 * 
 * */
