#ifndef CXXOMFORT_HPP
#define CXXOMFORT_HPP
/**
 * @file cxxomfort.hpp
 * @brief Backports library for C++.
 * @author Luis Machuca Bezzaza <luis [dot] machuca [at] gulix [dot] cl>
 * @copyright MIT License (see LICENSE.txt)
 * 
 * @mainpage Cxxomfort Backport Tools for C++.
 *
 * Cxxomfort is a small, header-only library that provides backports of various C++ features and goodies to previous Standard versions. 
 * 
 * This is a precompiled help built using 
 * <a href="http://www.doxygen.nl/" target="_blank">Doxygen</a> and covers only 
 * the published interfaces (names, functions, etc) and their usage; for 
 * information on installation and configuration, check the repository's page:
 *
 * https://lambs.gulix.cl/fossil.cgi/cxxomfort
 *
 * 
 * <table><tr>
 *  <th>Library</th>
 *  <td align="center"> @ref cxxomfort-macros  <br/>
 * @ref cm-info — @ref cm-behaviour - @ref cxxomfort-codegen  
 *  </td>
 * </tr><tr>
 *  <th>Basic features</th>
 *  <td align="center"> @ref cxxo-base "Base Features (automatically included)" — <tt>\#include <cxxomfort/base.hpp></tt><br/>
 * <br/> @ref cxxomfort-utils "Independent Utilities" — <tt>\#include <cxxomfort/cxxo-utils.hpp></tt>
 * <br/> @c array_ref , @c CXXO_FOREACH , @c function_caller , @c None , array_initialization, etc...
 *  </td>
 * </tr><tr>
 *  <th rowspan="3">Backports & Standard&nbsp;C++</th>
 *  <td align="center"> <strong>Backports</strong> 
 *  <tt>\#include <cxxomfort/backports.hpp></tt><br/>
 *  @ref cxx03-backports "C++03/TR1" -- @ref cxx11-backports "C++11" --  @ref cxx14-backports "C++14" -- @ref cxx17-backports "C++17" --  @ref cxx20-backports "C++20" -- @ref cxx23-backports "C++23"
 *  </td>
 *  </tr><tr><td align="center">
 *  @ref std0cerrno "<cerrno>" — @ref std0cstddef "<cstddef>" — @ref std0cstdint "<cstdint>" 
— @ref std0cstdio "<cstdio>" — @ref std0cstring "<cstring>" — @ref std0ctime "<ctime>" — @ref std0cuchar "<cuchar>" 
 <br/> @ref std0algorithm "<algorithm>" — @ref std0array "<array>" — @ref std0bit "<bit>" — @ref std0complex "<complex>" — @ref std0expected "<expected>" — @ref std0forward_list "<forward_list>" — @ref std0functional "<functional>" — @ref std0iomanip "<iomanip>" — @ref std0ios "<ios>" — @ref std0iterator "<iterator>" — @ref std0limits "<limits>" — @ref std0memory "<memory>" — @ref std0numeric "<numeric>" — @ref std0ostream "<ostream>" – @ref std0random "<random>" – @ref std0ratio "<ratio>" — @ref std0string "<string>" — @ref std0string_view "<string_view>"  – @ref std0tuple "<tuple>" – @ref std0typeindex "<typeindex>" – @ref std0utility "<utility>" 
 * <br/> @ref std0system_error "<system_error>" — @ref std0type_traits "<type_traits>" 
 * <br/> @ref std0any "<any>" — @ref std0variant "<variant>" — @ref std0optional "<optional>"
 * <br/><br/>  @ref cxxomfort-fixes "Fixes to implementation issues" 
 * </td>
 *  </tr>
 * <tr>
 *  <td align="center"> <strong>Extras</strong> 
 *  <hr/>
 * <a href="https://chiselapp.com/user/lmachucab/repository/cxxo-vocabulary17">C++17 Vocabulary Types</a> - @ref std0any "@c any " — @ref std0optional "@c optional " — @ref std0variant "@c variant "  <br/>
 * <a href="https://chiselapp.com/user/lmachucab/repository/cxxo-minrange">Minimal Range Proposal</a> - <tt>range</tt> <br/>
 * <a href="">Minimal expected<T,E></a> - <tt><experimental/expected></tt>
 *  </td> </tr><tr>
 *  <th rowspan="2">Supplements</th>
 * <td align="center">
 *  @subpage cxxo-sup "Library Supplements" -- @cxxowiki{Supplements,repo}</a> 
 *  <br/> <tt>\#include <cxxomfort/library.hpp></tt>
 * <br/>
 * @ref cxxomfort::supplemental::array_ref " @c array_ref<T> " — @ref  cxxomfort::supplemental::fixed_vector " @c fixed_vector<T> " — @ref cxxomfort::supplemental::osprintf "iostream printf"  
 * <br/>
 * Supplementals for Standard C++ Headers
 * </td></tr>
 * </table>
 * 
 * <h2>Licensing Terms</h2>
 * 
 * This library is distributed under the terms of the
 * MIT License. Check @ref LICENSE.txt for more information.
 * 
 * SPDX-License-Identifier: MIT
 * 
 */

//
// Setup
#include "base.hpp"
#include "config.hpp"

// Backports
#include "backports.hpp"

// Supplements
//#include "library.hpp" // cxxomfort library supplements


#endif
