#ifndef CXXOMFORT_FUNCTIONAL_HPP
#define CXXOMFORT_FUNCTIONAL_HPP

/**
 * @file
 * @brief Implementations and backports tied to <tt>functional</tt>.
 * @xrefitem std0functional "<functional>" "Header File"
 * @ingroup std0functional
 * 
 * */


#include <cxxomfort/base.hpp>
#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
    #pragma message CXXO_NOTICE("enabled <functional> support.")
#endif

#ifdef DOXYGEN_DOC
#else

// 
// invoking header
// 

#include <cxxomfort/cxxo-utils.hpp>
#include <functional>
#if (CXXOMFORT_CXX_STD >= 2011)
#elif (CXXOMFORT_CXX_STD >= 1997 && CXXOFLAG_tr1>0)
    // GCC has TR1 files under tr1/ and symbols under std::tr1::
    #if (CXXOFLAG_directory_tr1)
        #include <functional>
        #include <tr1/functional>
        //#pragma message ("tr1/functional")
    #else
        #include <functional>
    #endif

    #if (CXXOFLAG_namespace_tr1 > 0)

namespace std {
    namespace placeholders { using namespace ::std::tr1::placeholders; };
    using tr1::reference_wrapper;
    using tr1::ref;
    using tr1::cref;
    using tr1::mem_fn;
    using tr1::bind;
    using tr1::function;
    using tr1::bad_function_call;
}

    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1600) )
    // MSC 2010 and ahead don't have a namespace tr1

    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1600) && CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1500) )
    // MSC 2008 has tr1 under includepath, symbols under tr1::
namespace std {
    using tr1::reference_wrapper;
    using tr1::ref;
    using tr1::cref;
    using tr1::mem_fn;
    using tr1::bind;
    using tr1::function;
    using tr1::bad_function_call;
    
}
    #else
    // MSC 2005, 2008 pre SP1 doesn't have a TR1
        #if defined(CXXOFLAG_USE_INTERNAL_TR1)
        #endif

    #endif // compiler

    #undef  CXXOMFORT_IMPLEMENTS_mem_fn
    #define CXXOMFORT_IMPLEMENTS_mem_fn CXXO_IMPLSTATUS_NATIVE()

#else
    // before support for TR1
    #if defined(CXXOFLAG_USE_INTERNAL_TR1)
        #if (CXXOMFORT_NOTICES > 1)
            #pragma message CXXO_NOTICE("internal TR1: <functional>")
        #endif
        #include "./impl/07-reference_wrapper.hpp"
        #include "./impl/07-fn.hpp"
    #endif
#endif // c++ pre-11, tr1

// 
// std::hash
// 

#if (CXXOMFORT_CXX_STD >= 2011)
#elif (CXXO_COMPILER_SUPPORT_std_hash)
    // the following compilers have std::tr1::hash
    #if ( \
    (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1500) && CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1600)) \
    || (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=404) && CXXOMFORT_CXX_EMULATION<2011) \
    || (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>0) && (CXXOMFORT_CXX_EMULATION<2011) ) \
    )
        //#pragma message ("functional hash -> tr1::hash")
namespace std {
    // instead of using tr1::hash, we create a forward, so that the forward can be specialized normally in namespace std
    //using tr1::hash;
    template <typename T>
    struct hash 
    : public std::tr1::hash<T> 
    { };
} // std

    #endif
#elif (CXXOFLAG_tr1)
    #pragma message ("functional CXXOFLAG_tr1")
#elif (defined(CXXOFLAG_USE_INTERNAL_TR1) && CXXOFLAG_USE_INTERNAL_TR1>0)
	#include "./impl/07-hash.hpp"
#endif


// 
// unwrap_reference 
// this goes before the C++11 backports because the concept is used by eg.: make_tuple, make_from_tuple.
// 

#if defined(DOXYGEN_DOC)
namespace cxxomfort { namespace cxxostd {
	/**
	 * @brief  @c unwrap_reference type trait.
	 * */
	template <typename T> struct unwrap_reference {};
} }

#elif (CXXOMFORT_CXX_STD>=2020)

#elif (CXXOMFORT_CXX_STD>=1997 || CXXOMFORT_CXX_EMULATION>=2011)
    #include "impl/11-decay.hpp"
namespace cxxomfort { namespace cxxostd {

    template <typename T> 
    struct unwrap_reference { 
        typedef T type;
    };
    template <typename T> 
    struct unwrap_reference< std::reference_wrapper<T> >{
        typedef T& type;
    };

    template< typename T >
    struct unwrap_ref_decay 
    : unwrap_reference<typename std::decay<T>::type> {};

} } // cxxomfort::cxxostd

namespace std {
    using ::cxxomfort::cxxostd::unwrap_reference;
    using ::cxxomfort::cxxostd::unwrap_ref_decay;
}
#endif

// 
// std::hash<std::nullptr_t>
// 

#if (CXXOMFORT_CXX_STD >= 2014)
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=600) && CXXOMFORT_CXX_STD>=2011)
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1916))
#elif (CXXOMFORT_IMPLEMENTS_nullptr >= CXXO_IMPLSTATUS_BACKPORT())
namespace std {
    template <> 
    struct hash<nullptr_t> {
        typedef nullptr_t argument_type;
        typedef size_t result_type;
        
        result_type operator() (argument_type const) const CXXO_NOEXCEPTNOTHROW {
			return hash<bool*>()(0);
        }
    };
}
#endif

// 
// functoids
// The bit_and and sibling functors from <functional> are missing in some compilers
// 

#include "./impl/03-functional_bit.hpp"

// 
// transparent operator functoids<void> 
// 

#include "./impl/14-functional-transparent.hpp"

// 
// hash<> for unique_ptr (C++14)
//

#if (CXXOMFORT_CXX_STD>=2014)
#elif (CXXOMFORT_CXX_STD>=2011)
	#include <memory>
namespace std {

template <typename T>
struct hash< unique_ptr<T> > {
	typedef size_t result_type;
	typedef unique_ptr<T> argument_type;
	result_type operator() (argument_type const& a) const CXXO_NOEXCEPTNOTHROW {
		hash<T*> H;
		return H(a.get());
	}
	
};

}
#elif (CXXOMFORT_CXX_STD >= 1997)
	#include <cxxomfort/memory.hpp>
namespace std {

template <typename T>
struct hash< std::unique_ptr<T> > {
	typedef size_t result_type;
	typedef std::unique_ptr<T> argument_type;
	result_type operator() (argument_type const& a) const CXXO_NOEXCEPTNOTHROW {
		hash<T*> H;
		return H(a.get());
	}
};

} // std

#endif

// 
// std::invoke , std::invoke_r
//

#include "./impl/17-invoke.hpp"
#include "./impl/23-invoke_r.hpp"

// 
// not_fn
// 

#include "./impl/n4022-not_fn.hpp"

// 
// type_identity, from C++20
// 

#if defined(DOXYGEN_DOC)
	/**
	 * @ingroup std00featuremacros 
	 * @cppfeature{ @c type_identity }
	 * */
	#define __cpp_lib_type_identity YYYY00L
	
#elif (defined(__cpp_lib_type_identity))
#else
	#define __cpp_lib_type_identity (CXXOMFORT_CXX_STD*100)
namespace std {
	template <typename T>
	struct type_identity { typedef T type; };
	
}
#endif

// 
// bind_front, from C++20
// 

#if (defined(__cpp_lib_bind_front))
#elif (CXXOMFORT_CXX_STD>=1997)
    #include "./impl/20-bind_front.hpp"
#endif

#if (CXXOMFORT_CXX_STD >= 1997)
	// extra power for std::hash
	#include "./impl/hash.hpp"

#endif

// 
// fix missing "result", "result_type" in C++17 mem_fn
// 

	#if (0 && CXXOMFORT_CXX_STD >= 2017 && defined(__GLIBCXX__)) 
	
		#include <type_traits>
namespace cxxomfort { namespace fix { 

// wraps the result in a mem_fn adding extra typedef members
template <typename MFN>
struct memfn_res_t {
	MFN fn;
	explicit memfn_res_t (MFN&& e) noexcept 
	: fn(std::move(e))
	{ }
	
	template <typename... Args>
	struct result {
		typedef decltype( std::invoke(fn, std::declval<Args>()...) ) type;
	};

	template <typename... Args>
	auto operator() (Args&&... args) const 
	-> decltype( fn(std::forward<Args>(args)...) )
	 {
		return fn(std::forward<Args>(args)...);
	}
	
};

template< class M, class Member >
auto mem_fn(M Member::* pm) noexcept {
	auto mfn = std::mem_fn(pm);
	return memfn_res_t<decltype(mfn)>(std::move(mfn));
}



} }

	#elif ( CXXOMFORT_CXX_STD >= 2017 && defined(__GLIBCXX__) )
		#include "./impl/17-memfn_recover.hpp"
namespace cxxomfort { namespace fix {
	using cxxomfort::cxxostd::mem_fn;
} }

	#elif (CXXOMFORT_CXX_STD >= 2011) 
namespace cxxomfort { namespace fix {
	using std::mem_fn;
} }

	#elif (CXXOMFORT_CXX_STD >= 1997) 
namespace cxxomfort { namespace fix {
	using std::mem_fn;
} }

	#endif


#endif // doc/code

#endif


/**
 * @page std0functional <functional>
 * @brief Backports related to Standard header <tt><functional></tt> 
 * 
 * @code
 * #include <cxxomfort/functional.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * 
 * @section Interfaces 
 * 
 * Interfaces defined in this section:
 * 
 * * Bit functoids @c bit_and , @c bit_or , @c bit_xor (C++98, only for some older compilers).
 * * @link cxxomfort::cxxostd::reference_wrapper reference_wrapper @endlink (C++03/TR1) and its assisting functions.
 * * @ref @c mem_fn() wrapper (C++11) (only for some compilers)
 * * @link impl/14-functional-transparent.hpp Transparent / Heterogegenous Functors @endlink (C++14).
 * * @ref @c not_fn()  (C++17) - generalically negate any functor.
 * * @link backports_invoke @c invoke() / @c invoke_r() invocation of callables @endlink (C++17, C++20), for C++11-onward only.
 * * @c unwrap_reference (C++20).
 * * @c bind_front() (C++20) - for C++11-onward only.
 * 
 * The following interfaces are repointed to namespace @c std:: when found in namespace @c std::tr1:: :
 * 
 * * @c std::reference_wrapper , @c std::ref() , @c std::cref() - object wrapper and related utilities.
 * * @c std::bind() - function wrapper and related utilities.
 * * <tt>std::placeholders::_*</tt> from @c _1 to @c _9 .
 * * @c std::mem_fn() - member function adapter.
 * * @c std::function - function/callable wrapper
 * * @c std::hash - generic hashing
 * 
 * The following interfaces are provided by cxxomfort when @ref @c CXOMFORT_USE_INTERNAL_TR1 is defined:
 * 
 * * @c #cxxomfort::cxxostd::reference_wrapper , @c std::ref() , @c std::cref() .
 * * @c std::placeholders::_* from @c _1 to @c _9 .
 * * @c std::mem_fn() .
 * 
 * Non-backport interfaces (<tt>cxxomfort::fix</tt>):
 * 
 * * @link cxxomfort::fix::hash <tt>hash</tt> @endlink - Extra-power, specializable forwarder to @c std::hash .
 *
 * @section backports_invoke Callable Invocation with std::invoke 
 * 
 * Cxxomfort implements a backport of @c std::invoke() from C++17 and @c std::invoke_r() from C++20 for C++11 onwards, and for certain compilers in C++03 mode. The feature is implemented and functions differently depending on the compiler and Standard mode.
 * 
 * * For compilers supporting at least C++11, @c invoke() is implemented via trailing return type and should be capable of handing any callable type and any return type generated.
 * * For compilers supporting @c __typeof__ , @c invoke() is implemented via deduced return type, which could cause the callable to be invoked twice.
 * * For compilers without at least @c __typeof__ support, @c invoke() is not provided.
 * 
 * Additionally, cxxomfort sets the SD6 macros <tt>__cpp_lib_invoke</tt> and <tt>__cpp_lib_invoke_r</tt> when it backports (not emulates) these facilities.
 * 
 * @section Notes 
 * 
 * @note <b>Cxxomfort</b> expects an implementation of <tt>std::reference_wrapper</tt> and related utilities to exist and function in order, as part of the general TR1 requirement.
 * 
 * @note <b>Cxxomfort</b> expects an implementation of <tt>std::bind</tt> and related utilities to exist and function in order, as part of the general TR1 requirement.
 * 
 * @section future "Planned for the future"
 * 
 * * Recovered function adaptors @c bind1st() , @c bind2nd() and @c ptr_fun (C++17/C++20).
 * 
 * @section also See Also
 * 
 * @see @cppref{header/functional}
 * @see @cppref{utility/functional/reference_wrapper}
 * @see @cppref{utility/functional/not_fn}
 * @see @ref cxxo-sup-functional "cxxomfort <functional> supplements"
 * 
 */
