#ifndef CXXOMFORT_IMPL_STD_BIT_HPP
#define CXXOMFORT_IMPL_STD_BIT_HPP
/**
 * @file
 * @brief Adds the missing @c and_bit , @c or_bit , @c xor_bit in some versions of early C++.
 * 
 * Fixes lack of bit_and, bit_or, bit_xor in MSVC 2008.
 */

// fix missing bit_and, bit_or, bit_xor in MSVC 2008
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, < 1600) || CXXO_COMPILERREQ(CXXO_COMPILERID_DMC, < 0x900))
#include <functional>
    #if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 2))
    #pragma message CXXO_NOTICE("supplementing bit_and, etc in MSVC2008.")
    #endif

namespace std {


// T operator()( const T& lhs, const T& rhs ) const;

//! @ingroup cxx03-backports
template <typename T> 
struct bit_and {
    typedef T result_type;
    T operator() (T const& a, T const& b) const { return a&b; }
};

//! @ingroup cxx03-backports
template <typename T> 
struct bit_or {
    typedef T result_type;
    T operator() (T const& a, T const& b) const { return a|b; }
};

//! @ingroup cxx03-backports
template <typename T> 
struct bit_xor {
    typedef T result_type;
    T operator() (T const& a, T const& b) const { return a^b; }
};

} // namespace std

#endif // msvc

#endif
