#ifndef CXXOMFORT_IMPL_DISCARD_BLOCK_ENGINE_HPP
#define CXXOMFORT_IMPL_DISCARD_BLOCK_ENGINE_HPP
/**
 * @file
 */

#include "../config.hpp"
#include "../base.hpp"
#include <cxxomfort/impl/07-type_traits.hpp>
#include <cxxomfort/impl/11-conditional.hpp>

namespace cxxomfort { namespace cxxostd { 

template <typename Engine, size_t P, size_t R>
struct discard_block_engine {
	Engine eng;
	size_t n;
	
	static_assert0( P > 0 );
	static_assert0( R > 0 && R < P );
	
	typedef typename Engine::result_type result_type;
	CXXO_CONSTEXPR static size_t const block_size = P;
	CXXO_CONSTEXPR static size_t const used_block = R;
	
	CXXO_CONSTEXPR static size_t 	min () { return Engine::min(); }
	CXXO_CONSTEXPR static size_t 	max () { return Engine::max(); }
	
	explicit discard_block_engine (Engine& e)
	: eng(e), n(0) { }
	
	explicit discard_block_engine (result_type s)
	: eng(s), n(0) { }
	
	template <typename G>
	explicit discard_block_engine (G& g, typename std::enable_if< !::cxxomfort::traits::is_same<G, Engine>::value , bool>::type * = 0) 
	: eng(g), n(0) { }
	
	void 	seed ( result_type s) { eng.seed(s); n= 0; }
	void 	seed () { eng.seed(); n= 0; }
	
	void 	discard (size_t k) { for (; k; --k) operator()(); }
	
	result_type 	operator() () {
		result_type r= 0;
		if (n >= static_cast<size_t>(R)) {
			eng.discard(P-R);
			n= 0;
		}
		++n;
		return eng();
	}
	
	Engine const& 	base () const CXXO_NOEXCEPT { return eng; }

	friend bool 	operator== (discard_block_engine const& v1, discard_block_engine const& v2) CXXO_NOEXCEPTNOTHROW {
		return v1.n == v2.n && v1.eng == v2.eng;
	}
	friend bool 	operator!= (discard_block_engine const& v1, discard_block_engine const& v2) CXXO_NOEXCEPTNOTHROW {
		return !(v1==v2);
	}
};

} }

#endif // file
