#ifndef CXXOMFORT_IMPL_07_MEM_FN_HPP
#define CXXOMFORT_IMPL_07_MEM_FN_HPP
/**
 * @file
 * @brief Barebones implementation of ptr_fun, mem_fun, mem_fun_ptr and mem_fn
 * @ingroup std0functional
 * 
 * This is an implementation of the function wrappers in C++07/TR1
 * not directly used by cxxomfort but made available for TR1 compatibility layers.
 * 
 * This file should end up included only when no TR1 has been found and 
 * the user has requested CXXO_USE_INTERNAL_TR1.
 * 
 */
#include <cxxomfort/base.hpp>
#include <cxxomfort/type_traits.hpp> // for is_member_*_pointer
#include <cxxomfort/impl/relationals.hpp>
#include <cxxomfort/impl/07-type_traits.hpp> // is_member_pointer
#include <functional>

#define CXXOMFORT_IMPLEMENTS_mem_fn   0
#if (CXXOMFORT_CXX_STD >= 2011)
	#undef  CXXOMFORT_IMPLEMENTS_mem_fn 
	#define CXXOMFORT_IMPLEMENTS_mem_fn CXXO_IMPLSTATUS_NATIVE()
#elif (CXXOMFORT_CXX_EMULATION >= 2011)
	#undef  CXXOMFORT_IMPLEMENTS_mem_fn 
	#define CXXOMFORT_IMPLEMENTS_mem_fn CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD >= 1997)
	#if (CXXOFLAG_tr1 == 0)
		#undef  CXXOMFORT_IMPLEMENTS_mem_fn 
		#define CXXOMFORT_IMPLEMENTS_mem_fn CXXO_IMPLSTATUS_EMULATION()
	#endif
#endif

#if (CXXOMFORT_CXX_STD >= 1997)
namespace cxxomfort {
namespace cxxostd {

namespace impl {

// general case: function member pointer
template <typename Sig>
struct mem_fn_ft 
{ };

template <typename Class, typename R>
struct mem_fn_ft<R (Class::*)(void)> {
	private:
	typedef typename std::conditional<std::is_void<R>::value, cxxomfort::rvoid, R>::type DR;
	typedef R (Class::*pmember)(void) ;
	pmember pm;
	public:
	typedef R result_type;
	explicit mem_fn_ft (pmember pm_) CXXO_NOEXCEPTNOTHROW 
	: pm(pm_)
	{ }
	
	R operator() (Class const& c) const {
		return (c->*pm)();
	}
	R operator() (Class& c) {
		return (c.*pm)();
	}
};


template <typename Class, typename R, typename A1>
struct mem_fn_ft<R (Class::*)(A1)> {
	private:
	typedef typename std::conditional<std::is_void<R>::value, cxxomfort::rvoid, R>::type DR;
	typedef R (Class::*pmember)(A1) ;
	pmember pm;
	public:
	typedef R result_type;
	explicit mem_fn_ft (pmember pm_) CXXO_NOEXCEPTNOTHROW 
	: pm(pm_)
	{ }
	
	R operator() (Class const& c, A1 a1) const {
		return (c.*pm)(a1);
	}
	R operator() (Class& c, A1 a1) {
		return (c.*pm)(a1);
	}

};

template <typename Class, typename R, typename A1, typename A2>
struct mem_fn_ft<R (Class::*)(A1,A2)> {
	private:
	typedef typename std::conditional<std::is_void<R>::value, cxxomfort::rvoid, R>::type DR;
	typedef R (Class::*pmember)(A1,A2) ;
	pmember pm;
	public:
	mem_fn_ft (pmember pm_) CXXO_NOEXCEPTNOTHROW 
	: pm(pm_)
	{ }
	
	R operator() (Class const& c, A1 a1, A2 a2) const {
		return (c.*pm)(a1,a2);
	}
	R operator() (Class& c, A1 a1, A2 a2) {
		return (c.*pm)(a1,a2);
	}

};

// if object member pointer
template <typename Ty, typename Class>
struct mem_fn_ot {
	private:
	typedef Ty (Class::*omember);
	static_assert( std::is_member_object_pointer<omember>::value, "not a member object pointer");
	omember o;
	public:
	explicit mem_fn_ot (omember optr) CXXO_NOEXCEPTNOTHROW 
	: o(optr)
	{ }
	
	Ty const & operator() (Class const& c) const CXXO_NOEXCEPTNOTHROW {
		return c.*o;
	}
	Ty const & operator() (Class const* c) const CXXO_NOEXCEPTNOTHROW {
		return c->*o;
	}
};


} // impl

template <typename Ty, typename Class>
impl::mem_fn_ft< Ty (Class::*)(void) >
mem_fn (Ty (Class::*pm)(void)
) CXXO_NOEXCEPTNOTHROW {
	enum { Test = std::is_member_function_pointer< Ty (Class::*)() >::value };
	static_assert(Test, "member function pointer to member with 0 args");
	return impl::mem_fn_ft< Ty (Class::*)(void) >(pm);
}

// 1-arg

template <typename Ty, typename Class, typename A1>
impl::mem_fn_ft< Ty (Class::*)(A1) >
mem_fn (Ty (Class::*pm)(A1) ) CXXO_NOEXCEPTNOTHROW {
	return impl::mem_fn_ft< Ty (Class::*)(A1) >(pm);
}

template <typename Ty, typename Class, typename A1>
impl::mem_fn_ft< Ty (Class::*)(A1) const >
mem_fn (Ty (Class::*pm)(A1) const ) CXXO_NOEXCEPTNOTHROW {
	return impl::mem_fn_ft< Ty (Class::*)(A1) const >(pm);
}

template <typename Ty, typename Class, typename A1, typename A2>
impl::mem_fn_ft< Ty (Class::*)(A1,A2) >
mem_fn (Ty (Class::*pm)(A1,A2)
) CXXO_NOEXCEPTNOTHROW {
	return impl::mem_fn_ft< Ty (Class::*)(A1,A2) >(pm);
}

// mem_fn for member object pointer
template <typename Ty, typename Class>
typename impl::mem_fn_ot<Ty,Class>
mem_fn (Ty Class::*pm ) CXXO_NOEXCEPTNOTHROW {
	static_assert(std::is_member_object_pointer<Ty Class::* >::value, "member object pointer");
	return impl::mem_fn_ot<Ty,Class>(pm);
}



} } // cxxomfort::cxxostd

	#if (CXXOFLAG_tr1 == 0)
		#undef  CXXOMFORT_IMPLEMENTS_mem_fn 
		#define CXXOMFORT_IMPLEMENTS_mem_fn CXXO_IMPLSTATUS_EMULATION()

namespace std {
	using ::cxxomfort::cxxostd::mem_fn;
}
	#endif

#else // < 1997
	// without correct templates and overloading, nothing to be done

#endif // c++ std

#endif
