#ifndef CXXOMFORT_IMPL_07_HASH_HPP
#define CXXOMFORT_IMPL_07_HASH_HPP
/**
 * @file
 * @brief Barebones implementation of @c std::hash 
 * 
 * This is an implementation of std::hash 
 * not directly used by cxxomfort but made available for TR1 compatibility layers.
 * 
 * This file should end up included only when no TR1 has been found and 
 * the user has requested CXXO_USE_INTERNAL_TR1.
 * 
 */


#include <cxxomfort/config.hpp>

#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_COMPILER_SUPPORT_std_hash)
#elif (CXXOFLAG_tr1==0 || CXXOMFORT_CXX_STD < 2011)
/* 
 * Note: 
 * We declare hash in namespace std:: , since 
 * users have to be able to specialize it.
 * */
	#if (CXXOMFORT_NOTICES > 2)
		#pragma message CXXO_NOTICE("std::hash")
	#endif

namespace std {

template <typename t> struct hash {};

#define CXXO_HASH_NATIVE(T) \
template <> struct hash< T > { \
	typedef T argument_type; \
	typedef size_t result_type; \
	result_type operator() (T x) const CXXO_NOEXCEPTNOTHROW { \
		return static_cast<result_type>(x); \
	} \
 \
} \

CXXO_HASH_NATIVE(bool);
CXXO_HASH_NATIVE(char);
CXXO_HASH_NATIVE(signed char);
CXXO_HASH_NATIVE(unsigned char);
CXXO_HASH_NATIVE(signed int);
CXXO_HASH_NATIVE(unsigned int);
CXXO_HASH_NATIVE(signed short);
CXXO_HASH_NATIVE(unsigned short);
CXXO_HASH_NATIVE(signed long);
CXXO_HASH_NATIVE(unsigned long);
CXXO_HASH_NATIVE(signed long long);
CXXO_HASH_NATIVE(unsigned long long);
CXXO_HASH_NATIVE(float);
CXXO_HASH_NATIVE(double);
CXXO_HASH_NATIVE(wchar_t);


#undef CXXO_HASH_NATIVE


// hash for pointer types

template <typename T>
struct hash <T*> {
	typedef T* argument_type; 
	typedef size_t result_type; 
	result_type operator() (T* x) const CXXO_NOEXCEPTNOTHROW { 
		return (static_cast<const char*>(x) - static_cast<const char*>(0));
	} 
};


}

#endif // std

#endif
