#ifndef CXXOMFORT_IMPL_RANDOM_RENAMES_HPP
#define CXXOMFORT_IMPL_RANDOM_RENAMES_HPP
/**
 * @file
 * @brief Provides C++11-compliant renames of some utilities in C++03 TR1's "random" that changed names.
 */

#include "../base.hpp"


// fix renamed functors in <random> between C++03 TR1 and C++11
#if ((CXXOMFORT_CXX_STD < 2011) )
namespace cxxomfort { namespace cxxostd { namespace random_renames {
// These are only "good enough to work" fixes.


template<class _UIntType, _UIntType __a, _UIntType __c, _UIntType __m>
class linear_congruential_engine 
: public std::tr1::linear_congruential<_UIntType, __a, __c, __m> {
    private:
    typedef std::tr1::linear_congruential<_UIntType, __a, __c, __m> base_type;
    public:
    using typename base_type::result_type;
    explicit linear_congruential_engine (unsigned long sd=1)
    : base_type(sd) {}
    
    using base_type::operator();
    
    void discard (uintmax_t z) {
		while (z-->0) { this->operator() (); }
	}
};

typedef linear_congruential_engine<uint_fast32_t, 16807, 0, 2147483647> 
minstd_rand0;

typedef linear_congruential_engine<uint_fast32_t, 48271, 0, 2147483647> 
minstd_rand;



#if (1)
//! @cxx1b 
template <typename II, II _w, II _s, II _r>
class subtract_with_carry_engine 
/* : protected subtract_with_carry<II,_w,_s,_r>  */
{
    private:
    typedef std::tr1::subtract_with_carry<II,_w,_s,_r> base_type;
    public:
    typedef typename base_type::result_type result_type;
    explicit subtract_with_carry_engine (result_type sd=0) 
    // : base_type(sd) {}
    : _P(sd) {}

    void seed ( result_type value = base_type::default_seed ) { _P.seed(value); }
    result_type operator() () { return _P(); }
    void discard (unsigned int z) { _P.discard(z); } 
    result_type min () const { return _P.min(); }
    result_type max () const { return _P.max(); }

    private:
    base_type _P;
};
#elif (0)
template <typename II, II _w, II _s, II _r>
class subtract_with_carry_engine 
: private subtract_with_carry<II,_w,_s,_r> 
{
    private:
    typedef subtract_with_carry<II,_w,_s,_r> base_type;
    public:
    using base_type::result_type;
    explicit subtract_with_carry_engine (result_type sd=0) 
    : base_type(sd) {}

    template <typename G>
    explicit subtract_with_carry_engine (G& g) 
    : base_type(g) {}

    using base_type::min;
    using base_type::max;
    using base_type::seed;
    using base_type::operator();
};
#endif


typedef subtract_with_carry_engine<uint_fast32_t, 24, 10, 24> ranlux24_base;
typedef subtract_with_carry_engine<uint_fast64_t, 48,  5, 12> ranlux48_base;


//! @cxx11b Updates the name of @c uniform_int in GCC
template <typename II>
class uniform_int_distribution
: public std::tr1::uniform_int<II> {
	typedef std::tr1::uniform_int<II> base_type;
    public:
    uniform_int_distribution(II a, II b)
    : std::tr1::uniform_int<II>(a,b) {}
    
    using base_type::operator();
};

//! @cxx11b Updates the name of @c uniform_real in GCC
template <typename Real>
class uniform_real_distribution
: public std::tr1::uniform_real<Real> {
	typedef std::tr1::uniform_real<Real> base_type;
    public:
    uniform_real_distribution(Real a, Real b)
    : std::tr1::uniform_real<Real>(a,b) {}
    
    using base_type::operator();
};

} // random
} }

#endif // fix
#endif
