#ifndef CXXOMFORT_IMPL_07_REFERENCE_WRAPPER_HPP
#define CXXOMFORT_IMPL_07_REFERENCE_WRAPPER_HPP
/*
 * @file
 */

#include "../config.hpp"

#define CXXOMFORT_IMPLEMENTS_n1436 CXXO_IMPLSTATUS_BACKPORT()
#define CXXOMFORT_IMPLEMENTS_n1453 CXXO_IMPLSTATUS_BACKPORT()
#undef  CXXOMFORT_IMPLEMENTS_reference_wrapper
#define CXXOMFORT_IMPLEMENTS_reference_wrapper CXXOMFORT_IMPLEMENTS_n1453

#if defined(DOXYGEN_DOC)
namespace std {

/**
 * @brief std::reference_wrapper<T> implementation
 * @cxx03b Wraps an object for lightweight passing.
 * @xrefitem std0functional "" "" Wraps an object for lightweight passing.
 * @ingroup std0functional
 * 
 * A @c reference_wrapper hosts a reference to an object in the same way a normal reference does, but separates assignment mechanics so that assigning to the wrapper changes @e which@e object is hosted, instead of changing the object itself. It basically behaves as a pointer-to-T.
 * 
 * @see @cppref{utility/functional/reference_wrapper}
 * 
 */
template <typename T>
struct reference_wrapper { 

	//! copy-ctor
	reference_wrapper (reference_wrapper const& R) CXXO_NOEXCEPTNOTHROW
	: :v(R.get()){} 
	//! Assignment operator - assigns to the wrapped object.
    reference_wrapper& operator= (reference_wrapper const& R) CXXO_NOEXCEPTNOTHROW { }

    //! access the referenced object
    T& get () const CXXO_NOEXCEPTNOTHROW { }
    //! access the referenced object
    operator T& () const CXXO_NOEXCEPTNOTHROW { }

};

/**
 * @brief Wraps an object in a @c reference_wrapper .
 * @returns <tt>reference_wrapper<T></tt> wrapping the named object @a t .
 * @cxx03b Wraps a named object @a t into a <tt>reference_wrapper<T></tt>.
 * @xrefitem std0functional "" "" Wraps a named object @a t into a <tt>reference_wrapper<T></tt>.
 * 
 * @see @ref @c reference_wrapper
 * 
 * */
template <typename T>
reference_wrapper<T> ref (T& t) CXXO_NOEXCEPTNOTHROW { }

/**
 * @brief Wraps an object in a @c reference_wrapper that provides a const view.
 * @returns <tt>reference_wrapper<T const></tt> wrapping the named object @a t .
 * @cxx03b Wraps a named object @a t into a <tt>reference_wrapper<T const></tt>.
 * @xrefitem std0functional "" "" Wraps a named object @a t into a <tt>reference_wrapper<T const></tt>.
 * 
 * @see @ref @c reference_wrapper
 * 
 * */
template <typename T>
reference_wrapper<T const> cref (T const& t) CXXO_NOEXCEPTNOTHROW { }

}
#endif


namespace cxxomfort { namespace cxxostd {

//! @internal
namespace detail_ref {
	//! @internal
	template <typename T> T& declval();
	//! @internañ
		template <typename T> inline
	T* addressof (T& arg) {
		return reinterpret_cast<T*>(
			&const_cast<char&>( reinterpret_cast<const volatile char&>( arg) ) 
		);
	}
}

template <typename T>
struct reference_wrapper {
    public:
    typedef T value_type;
    explicit reference_wrapper(T& t) CXXO_NOEXCEPT 
    : v(detail_ref::addressof(t)) {}

	//! copy-ctor
	reference_wrapper (reference_wrapper const& R) CXXO_NOEXCEPTNOTHROW
	: :v(R.get()){} 
	//! Assignment operator
    reference_wrapper& operator= (reference_wrapper const& R) CXXO_NOEXCEPTNOTHROW {
        v= R.v;
        return *this;
    }

	//! access the referenced object
    T& get () const CXXO_NOEXCEPTNOTHROW { return v; }
    //! access the referenced object
    operator T& () const CXXO_NOEXCEPTNOTHROW { return v; }

	#if defined(DOXYGEN_DOC)
	//! Invoke using the referenced object, if capable
	template <typename... Args>
	undefined_type_t 
	operator() (Args... args) const {}

	#elif (CXXO_COMPILER_SUPPORT_typeof)
    __typeof__( detail_ref::declval<T>()() ) 
    operator() () const {
        return get()();
    }
    template <typename T1>
    __typeof__( detail_ref::declval<T>()(detail_ref::declval<T1>()) )
    operator() (T1 const& a1) const {
        return get()(a1);
    }
    template <typename T1, typename T2>
    __typeof__( detail_ref::declval<T>()(detail_ref::declval<T1>(), detail_ref::declval<T2>() ) )
    operator() (T1 const& a1, T2 const& a2) const {
        return get()(a1,a2);
    }
    #endif
    
    private:
    T* v;
};

template <typename T>
reference_wrapper<T> ref (T& t) CXXO_NOEXCEPTNOTHROW {
    return reference_wrapper<T>(t);
}

template <typename T>
reference_wrapper<T const> cref (T const& t) CXXO_NOEXCEPTNOTHROW {
    return reference_wrapper<T const>(t);
}

} }

#endif // file
