#ifndef CXXOMFORT_IMPL_07_TUPLE_HPP
#define CXXOMFORT_IMPL_07_TUPLE_HPP
/*
 * @file
 * @brief Barebones implementation of @c std::tuple 
 * 
 * This is an implementation of std::hash 
 * not directly used by cxxomfort but made available for TR1 compatibility layers.
 * 
 * This file should end up included only when no TR1 has been found and 
 * the user has requested CXXO_USE_INTERNAL_TR1.
 * 
 */


#include <cxxomfort/config.hpp>
#include <cxxomfort/impl/07-type_traits.hpp>

namespace cxxomfort { namespace cxxostd {

struct _TNo {};

template <size_t I, typename T=_TNo >
struct _THold {
	enum { index = I };
	T t;
	typedef typename cxxomfort::traits::add_reference<T>::type RT;
	typedef typename cxxomfort::traits::add_reference<T const>::type RCT;
	explicit _THold () {}
	explicit _THold (RT t)
	: t(t)
	{ }
	RCT get () const CXXO_NOEXCEPTNOTHROW { return static_cast<RCT>(*this); }
	RT get () CXXO_NOEXCEPTNOTHROW { return static_cast<RT>(*this); }
};

template <typename T0, typename T1=_TNo, typename T2=_TNo
	, typename T3=_TNo, typename T4=_TNo >
class tuple ;

template <typename T>
struct tuple_size {};

template <size_t I, typename Tu>
struct tuple_element { };

template <size_t I, typename Tu>
typename tuple_element<I,Tu>::type const& 
get (Tu const& tu);

template <size_t I, typename Tu>
typename cxxomfort::traits::add_reference< typename tuple_element<I,Tu>::type >::type 
get (Tu & tu);


struct ignore_t {
	template <typename T>
	ignore_t const & operator=(const T&) const { return *this; }
};
const ignore_t ignore;

// 
// tuple 
// 

/**
 * @brief std::tuple<T...> implementation
 * @ingroup std0tuple
 * @cxx03b Implementation of a @e tuple concept
 * 
 * This implementation is limited to 5 elements and is in use only in pre-C++03 compilers when @c @ref CXXOFLAG_USE_INTERNAL_TR1 is defined, for example for compilers that don't themselves provide <tt><tuple></tt>.
 * */
template <
	typename T0, typename T1, typename T2
	, typename T3, typename T4 >
class tuple 
: public _THold<0,T0> , public _THold<1,T1> , public _THold<2,T2> , public _THold<3,T3> , public _THold<4,T4>
{
	public:
	enum { size = cxxomfort::traits::is_same<T1,_TNo>::value ? 1 
		: cxxomfort::traits::is_same<T2,_TNo>::value ? 2
		: cxxomfort::traits::is_same<T3,_TNo>::value ? 3
		: cxxomfort::traits::is_same<T4,_TNo>::value ? 4 : 5
		};   

	tuple (T0 t0, T1 t1, T2 t2, T3 t3, T4 t4)
	: _THold<0,T0>(t0), _THold<1,T1>(t1), _THold<2,T2>(t2)
	, _THold<3,T3>(t3), _THold<4,T4>(t4)
	{ }

	tuple (T0 t0, T1 t1, T2 t2, T3 t3)
	: _THold<0,T0>(t0), _THold<1,T1>(t1), _THold<2,T2>(t2)
	, _THold<3,T3>(t3)
	{ }

	tuple (T0 t0, T1 t1, T2 t2)
	: _THold<0,T0>(t0), _THold<1,T1>(t1), _THold<2,T2>(t2)
	{ }

	tuple (T0 t0, T1 t1)
	: _THold<0,T0>(t0), _THold<1,T1>(t1)
	{ }

	tuple (T0 t0)
	: _THold<0,T0>(t0)
	{ }

	tuple ()
	{ }

	tuple& operator= (tuple const& tu) {
		assign(tu);
		return *this;
	}
	
	template <typename U0, typename U1, typename U2, typename U3, typename U4>
	tuple& operator= (tuple<U0,U1,U2,U3,U4> const& tu) {
		static_cast<_THold<0,T0>&>(*this).t= static_cast<_THold<0,U0> const&>(tu).t;
		static_cast<_THold<1,T1>&>(*this).t= static_cast<_THold<1,U1> const&>(tu).t;
		static_cast<_THold<2,T2>&>(*this).t= static_cast<_THold<2,U2> const&>(tu).t;
		static_cast<_THold<3,T3>&>(*this).t= static_cast<_THold<3,U3> const&>(tu).t;
		static_cast<_THold<4,T4>&>(*this).t= static_cast<_THold<4,U4> const&>(tu).t;
		return *this;
	}
	
	private:
	
	tuple&     assign (tuple const& tu) {
		static_cast<_THold<0,T0>&>(*this)= static_cast<_THold<0,T0> const&>(tu);
		static_cast<_THold<1,T1>&>(*this)= static_cast<_THold<1,T1> const&>(tu);
		static_cast<_THold<2,T2>&>(*this)= static_cast<_THold<2,T2> const&>(tu);
		static_cast<_THold<3,T3>&>(*this)= static_cast<_THold<3,T3> const&>(tu);
		static_cast<_THold<4,T4>&>(*this)= static_cast<_THold<4,T4> const&>(tu);
		return *this;
	}
	
	void       swap (tuple& tu) {
	}
	

}; // tuple

// 
// tuple_size , tuple_element
// 

template <typename T0, typename T1, typename T2, typename T3, typename T4>
struct tuple_size< tuple<T0,T1,T2,T3,T4> > 
: public cxxomfort::traits::integral_constant<size_t, tuple<T0,T1,T2,T3,T4>::size >
{ };

template <size_t I, typename T0, typename T1, typename T2, typename T3, typename T4>
struct tuple_element< I, tuple<T0,T1,T2,T3,T4> > {
	private:
	struct _NT;
	public:
	typedef 
	typename cxxomfort::traits::conditional< I==0 , T0, 
		typename cxxomfort::traits::conditional< I==1, T1, 
			typename cxxomfort::traits::conditional< I==2, T2, 
				typename cxxomfort::traits::conditional< I==3, T3, 
					typename cxxomfort::traits::conditional< I==4, T4, _NT>::type
				>::type 
			>::type
		>::type
	>::type type;
};

// 
// make_tuple 
// 

template <typename T0>
inline tuple<T0> make_tuple (T0 t0) {
	return tuple<T0>(t0);
}

template <typename T0, typename T1>
inline tuple<T0,T1> make_tuple (T0 t0, T1 t1) {
	return tuple<T0,T1>(t0,t1);
}

template <typename T0, typename T1, typename T2>
inline tuple<T0,T1,T2> make_tuple (T0 t0, T1 t1, T2 t2) {
	return tuple<T0,T1,T2>(t0,t1,t2);
}


/*
std::tuple<unwrap_decay_t<Types>...> make_tuple(Types&&... args)
{
    return std::tuple<unwrap_decay_t<Types>...>(std::forward<Types>(args)...);
}
*/

// 
// get<N>
// 

template <size_t I, typename Tu>
inline 
typename tuple_element<I,Tu>::type const& 
get (Tu const& tu) {
	typedef typename tuple_element<I,Tu>::type Ty;
	_THold<I,Ty> const& tr(tu);
	return tr.t;
}

template <size_t I, typename Tu>
inline 
typename cxxomfort::traits::add_reference< typename tuple_element<I,Tu>::type >::type 
get (Tu & tu) {
	typedef typename tuple_element<I,Tu>::type Ty;
	_THold<I,Ty> & tr(tu);
	return tr.t;
}

// 
// tie 
// 

template <typename T0> inline
tuple<T0&> tie (T0& t0) {
	return tuple<T0&>(t0);
}

template <typename T0, typename T1> inline
tuple<T0&,T1&> tie (T0& t0, T1& t1) {
	return tuple<T0&,T1&>(t0,t1);
}

template <typename T0, typename T1, typename T2> inline
tuple<T0&,T1&,T2&> tie (T0& t0, T1& t1, T2& t2) {
	return tuple<T0&,T1&,T2&>(t0,t1,t2);
}

template <typename T0, typename T1, typename T2, typename T3> inline
tuple<T0&,T1&,T2&> tie (T0& t0, T1& t1, T2& t2, T3& t3) {
	return tuple<T0&,T1&,T2&,T3&>(t0,t1,t2,t3);
}

} } // cxxomfort::cxxostd


#endif
