#ifndef CXXOMFORT_IMPL_BASIC_STRING_CHARn_HPP
#define CXXOMFORT_IMPL_BASIC_STRING_CHARn_HPP
/**
 * @file
 * @brief Implementation of basic_string for char16_t, char32_t in C++03
 *
 */


#if (CXXOMFORT_CXX_STD>=1997)

#if (defined( CXXOFLAG_NO_STD_USING ) && CXXOFLAG_NO_STD_USING>0)
#else

#include <cxxomfort/base.hpp>
#include <cxxomfort/cstddef.hpp> 
#include <cxxomfort/cuchar.hpp> // char16_t
#include <cxxomfort/impl/07-type_traits.hpp> // is_arithmetic w/o C++11 dependency
#include <string>

#if (CXXOMFORT_CXX_STD < 2011 && CXXOMFORT_CXX_EMULATION < 2011)
namespace cxxomfort { namespace cxxostd {
} }

#if (CXXO_charn_t_typedefs>0)
namespace std {
    typedef basic_string<char16_t> u16string;
    typedef basic_string<char32_t> u32string;
}

#else
// this is a specialization, so it goes in std::
namespace std {

template<>
class basic_string<char16_t> 
: public basic_string<uint16_t> {
    // c++03 has no inheriting constructors
    typedef basic_string<uint16_t> base_t;
    public:
    basic_string () : base_t() {}
    basic_string (basic_string const& s) : base_t(s) {}
    basic_string (size_t n, char16_t c) : base_t(n,static_cast<uint16_t>(c)) {}
    explicit basic_string (char16_t const * p) : base_t(reinterpret_cast<uint16_t const*>(p)) {}
};

template<>
class basic_string<char32_t> : public basic_string<uint32_t> {
    // c++03 has no inheriting constructors
    typedef basic_string<uint32_t> base_t;
    public:
    basic_string () : base_t() {}
    basic_string (basic_string const& s) : base_t(s) {}
    basic_string (size_t n, char32_t c) : base_t(n,static_cast<uint32_t>(c)) {}
    explicit basic_string (char32_t const * p) : base_t(reinterpret_cast<uint32_t const*>(p)) {}
};

typedef basic_string<char16_t> u16string;
typedef basic_string<char32_t> u32string;

}
#endif


#endif // c++<2011

#endif // no_std_using

#endif // CXX_STD
#endif
