#ifndef CXXOMFORT_IMPL_CONDITIONAL_HPP
#define CXXOMFORT_IMPL_CONDITIONAL_HPP
/**
 * @file cxxomfort/type_traits.hpp
 * @brief Implementation of std::enable_if , std::conditional
 *
 */


#include "../config.hpp"

#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_COMPILER_SUPPORT_conditional_enableif)
	#define CXXOMFORT_IMPLEMENTS_n2240 CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD >= 1997 && CXXOFLAG_USE_INTERNAL_TR1==0)
	#define CXXOMFORT_IMPLEMENTS_n2240 CXXO_IMPLSTATUS_BACKPORT()

namespace cxxomfort { namespace cxxostd {

template <bool B, typename T=void>
struct enable_if{};

template <typename T>
struct enable_if<true, T> {
    typedef T type;
};

template <bool B, typename T, typename U>
struct conditional {};

template <typename T, typename U>
struct conditional<false,T,U> { typedef U type; };

template <typename T, typename U>
struct conditional<true,T,U> { typedef T type; };

} } // cxxomfort::cxxostd

namespace std {
	using ::cxxomfort::cxxostd::enable_if;
	using ::cxxomfort::cxxostd::conditional;
}

#elif (CXXOFLAG_USE_INTERNAL_TR1)
	#define CXXOMFORT_IMPLEMENTS_n2240 CXXO_IMPLSTATUS_BACKPORT()
	#include "07-type_traits.hpp"

namespace cxxomfort { namespace cxxostd {
	using cxxomfort::traits::enable_if;
	using cxxomfort::traits::conditional;
} }

namespace std {
	using ::cxxomfort::cxxostd::enable_if;
	using ::cxxomfort::cxxostd::conditional;
}

#else 

#endif



#endif // file

#ifdef DOXYGEN_DOC
namespace std {

/**
 * @brief Metaprogramming helper that defines a @c type only if a condition holds.
 * @ingroup std0typetraits
 * @sa n2240
 * @anchor enable_if
 */
template <bool B, typename T=void>
struct enable_if{
    typedef T type;
};

/**
 * @brief Metaprogramming helper that defines a @c type to be either @a T or @a U.
 * @ingroup std0typetraits
 * @sa n2240
 * @anchor conditional
 */
template <bool B, typename T, typename U>
struct conditional {
    typedef Either_T_Or_U type;
};

} // std

#endif
