#ifndef CXXOMFORT_IMPL_DECAY_HPP
#define CXXOMFORT_IMPL_DECAY_HPP

/**
 * @file 
 * @brief Implementation of std::decay
 */

#include <cxxomfort/config.hpp>
#include "./using_tr1_type_traits.hpp"

// obtained directly from cppreference.

#if defined(DOXYGEN_DOC)
namespace std {
/**
 * @brief Given a type, applies the same transformations as the ones 
 * used in the language for implicit conversion at call site: 
 * array-to-pointer, function-to-pointer and loss of reference.
 * @ingroup std0typetraits
 * @cxx11b Performs implicit decay conversions on a type.
 * 
 * @see https://en.cppreference.com/w/cpp/types/decay
 */
	template <typename T> struct decay {};
}
#elif (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#elif (CXXOMFORT_CXX_STD>=2011)
	//#pragma message CXXO_NOTICE("std::decay native c++11")
	#undef  CXXOMFORT_IMPLEMENTS_n2069
	#define CXXOMFORT_IMPLEMENTS_n2069 CXXO_IMPLSTATUS_NATIVE()
	#include <type_traits>
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >= 406) && CXXOMFORT_CXX_EMULATION==2011)
	//#pragma message CXXO_NOTICE("std::decay native gcc")
	#undef  CXXOMFORT_IMPLEMENTS_n2069
	#define CXXOMFORT_IMPLEMENTS_n2069 CXXO_IMPLSTATUS_NATIVE()

#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, >= 1600))
	//#pragma message CXXO_NOTICE("std::decay native msvc")
	#undef  CXXOMFORT_IMPLEMENTS_n2069
	#define CXXOMFORT_IMPLEMENTS_n2069 CXXO_IMPLSTATUS_NATIVE()

#else
	#include "11-conditional.hpp"
	//#pragma message CXXO_NOTICE("backporting std::decay")
	#undef  CXXOMFORT_IMPLEMENTS_n2069
	#define CXXOMFORT_IMPLEMENTS_n2069 CXXO_IMPLSTATUS_BACKPORT()

namespace std {

template< typename T >
struct decay {
    private:
    typedef typename ::std::remove_reference<T>::type U;
    public:
    typedef typename ::std::conditional< 
        std::is_array<U>::value,
        typename ::std::remove_extent<U>::type*,
        typename ::std::conditional< 
            ::std::is_function<U>::value,
            typename ::std::add_pointer<U>::type,
            typename ::std::remove_cv<U>::type
        >::type
    >::type type;
};

}

	//#pragma message CXXO_NOTICE("std::decay backport")
#endif

#undef  CXXOMFORT_IMPLEMENTS_decay
#define CXXOMFORT_IMPLEMENTS_decay CXXOMFORT_IMPLEMENTS_n2069


#endif
