#ifndef CXXOMFORT_IMPL_IS_CONSTRUCTIBLE_HPP
#define CXXOMFORT_IMPL_IS_CONSTRUCTIBLE_HPP

// This solution mostly extracted from 
// https://stackoverflow.com/questions/38181357/how-is-stdis-constructiblet-args-implemented?noredirect=1&lq=1

#include <cxxomfort/impl/07-type_traits.hpp>
#include "17-void_t.hpp"


#if defined(DOXYGEN_DOC)
namespace cxxomfort { namespace cxxostd {

/**
 * @brief Trait that determines if a type is constructible with given arguments.
 * @cxx11b Determine if @c T is constructible with <tt>T(args...)</tt>.
 * @xrefitem std0type_traits "" ""Determine if a type is constructible with arguments
 * */
template <typename T, typename... Args>
struct is_constructible {};

}}

#endif
// In theory all STD C++14 compilers support is_constructible, nothing to do here


#if (CXXOMFORT_CXX_STD >= 2014)
	// nothing to do

// list of C++11 compilers missing is_constructible
#if (CXXOMFORT_CXX_STD >= 2011 && (\
1 \
) )

namespace cxxomfort { namespace cxxostd {

template <class, class T, class... Args>
struct is_constructible_ : std::false_type {};

template <class T, class... Args>
struct is_constructible_<
    std::void_t<decltype(T(std::declval<Args>()...))>,
T, Args...> : std::true_type {};

template <class T, class... Args>
struct is_constructible 
: is_constructible_<std::void_t<>, T, Args...> 
{};

} }
// end of c++11 mode
#else 

#if (CXXOMFORT_CXX_EMULATION == 2011 && CXXO_COMPILER_SUPPORT_decltype > 0)
#elif (CXXO_COMPILER_SUPPORT_typeof>0)
#else
#endif

namespace cxxomfort { namespace cxxostd {

template <class, class T, typename A0=void, typename A1=void, typename A2=void, typename A3=void, typename A4=void>
struct is_constructible_ : std::false_type {};

template <class T, typename A0, typename A1, typename A2, typename A3, typename A4>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T(std::declval<A0>(), declval<A1>(), declval<A2>(), declval<A3>(), declval<A4>() ))>::type,
T, A0, A1, A2, A3, A4> : std::true_type {};

template <class T, typename A0, typename A1, typename A2, typename A3>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T(std::declval<A0>(), declval<A1>(), declval<A2>(), declval<A3>() ))>::type,
T, A0, A1, A2, A3> : std::true_type {};

template <class T, typename A0, typename A1, typename A2>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T(std::declval<A0>(), declval<A1>(), declval<A2>() ))>::type,
T, A0, A1, A2> : std::true_type {};

template <class T, typename A0, typename A1>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T(std::declval<A0>(), declval<A1>() ))>::type,
T, A0, A1> : std::true_type {};

template <class T, typename A0>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T(std::declval<A0>() ))>::type,
T, A0> : std::true_type {};

template <class T>
struct is_constructible_<
    typename make_void<CXXO_TYPEOF(T())>::type,
T> : std::true_type {};


template <class T, typename A0=void, typename A1=void, typename A2=void, typename A3=void, typename A4=void>
struct is_constructible 
: is_constructible_<typename make_void<>::type, T, A0, A1, A2, A3, A4> 
{};

} }

#endif // non-c++11 mode




#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#else
#if (CXXOMFORT_CXX_STD < 2011)
namespace std {
    using ::cxxomfort::cxxostd::is_constructible;
}
#endif
#endif // no using in std

#endif
