#ifndef CXXOMFORT_IMPL_IS_TN_XBLE_HPP
#define CXXOMFORT_IMPL_IS_TN_XBLE_HPP
/**
 * @file 
 * @brief Family of "is_trivially_", "is_nothrow_" traits.
 */

#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>

#if (CXXOMFORT_CXX_STD < 2011)
    #if (CXXOMFORT_CXX_EMULATION < 2011)
        #define CXXO_is_tn_xble 1
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, < 408) )
        #define CXXO_is_tn_xble 1
    #endif // c++11 emulation
    #if (CXXOMFORT_CXX_EMULATION>=2011)
        #if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, < 408))
            #define CXXO_is_tn_xble 1
        #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, < 1700))
            #define CXXO_is_tn_xble 1
        #endif
    #endif
#endif // c++11

// 
// These go under namespace std
// 
#ifdef CXXO_is_tn_xble

// is_nothrow_ traits
namespace std {

    template <typename T>
    struct is_nothrow_default_constructible
    : cxxomfort::traits::integral_constant<bool, __has_nothrow_constructor(T)> {};

    template <typename T>
    struct is_nothrow_copy_constructible
    : cxxomfort::traits::integral_constant<bool, __has_nothrow_copy(T)> {};

    template <typename T>
    struct is_nothrow_copy_assignable 
    : cxxomfort::traits::integral_constant<bool, __has_nothrow_assign(T)> {};

    template <typename T>
    struct is_nothrow_destructible
    : cxxomfort::traits::integral_constant<bool, __has_trivial_destructor(T)> {};



}

#endif // is_tn_xble


// compiler missing out on some of the constructible/destructible traits:
// GCC 407, 408 in c++0x mode
// clang 3.0~3.3 in c++0x mode
#if ( CXXOMFORT_CXX_STD==2011 && ( \
 CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, <= 408) || CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,<304) \
) )
	//#pragma message "missing is_trivially..."

namespace std {

    template <typename T>
    struct is_trivially_default_constructible
    : std::integral_constant<bool, __has_trivial_constructor(T)> {};

    template <typename T>
    struct is_trivially_copy_constructible 
    : std::integral_constant<bool, __has_trivial_copy(T)> {};

    template <typename T>
    struct is_trivially_copy_assignable 
    : std::integral_constant<bool, __has_trivial_assign(T)> {};

}

#endif 

#if (CXXOMFORT_CXX_STD<2011 && CXXOMFORT_CXX_EMULATION<2011)
    #define CXXO_is_constructible 1
#elif (CXXOMFORT_CXX_EMULATION==2011)
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, < 407))
        #define CXXO_is_constructible 1
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, < 1700))
        #define CXXO_is_constructible 1
    #endif
#endif

#if (defined(CXXO_is_constructible))
namespace std {
    template <typename T>
    struct is_default_constructible
    : std::integral_constant<bool, is_trivially_default_constructible<T>::value || is_nothrow_default_constructible<T>::value>
    {};

} // std::

#endif


//
// is_copy_constructible (for c++03)
// is_copy_assignable (for c++03)
//
#if (CXXOMFORT_CXX_STD < 2011)
    #define CXXO_needs_copy_constructible
#endif
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >= 500) && 2011==CXXOMFORT_CXX_EMULATION)
    #undef CXXO_needs_copy_constructible
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, >= 1700))
    #undef CXXO_needs_copy_constructible
#endif

#ifdef CXXO_needs_copy_constructible
namespace std {
    // pending: use is_convertible as an assist
    template <typename T>
    struct is_copy_constructible
    : std::integral_constant<bool, is_trivially_copy_constructible<T>::value || is_nothrow_copy_constructible<T>::value > 
    {};

    // pending: use is_convertible as an assist
    template <typename T>
    struct is_copy_assignable
    : cxxomfort::traits::integral_constant<bool, is_trivially_copy_assignable<T>::value || is_nothrow_copy_assignable<T>::value > 
    {};

} // std::
#endif

#endif
