#ifndef CXXOMFORT_IMPL_IS_OF_HPP
#define CXXOMFORT_IMPL_IS_OF_HPP
/**
 * @file cxxomfort/algorithm.hpp
 */

#include <cxxomfort/config.hpp>
#include <algorithm>

#if defined(DOXYGEN_DOC)
namespace cxxomfort { namespace cxxostd {

/**
 * @{
 * @ingroup cxx11-backports
 * */

/**
 * @brief Checks if @e all elements of a sequence fit a given predicate.
 * @returns @c true if for all elements @e e in <tt>[ini,fin)</tt>, <tt>f(e) == true</tt>.
 * */
template <typename Iter, typename Predicate>
bool all_of (Iter ini, Iter fin, Predicate f) {}

/**
 * @brief Checks if @e any one element of a sequence fit a given predicate.
 * @returns @c true if for at least one elements @e e in <tt>[ini,fin)</tt>, <tt>f(e) == true</tt>.
 * */
template <typename Iter, typename Predicate>
bool any_of (Iter ini, Iter fin, Predicate f) {}

/**
 * @brief Checks if @e no elements of a sequence fit a given predicate.
 * @returns @c true if for no elements @e e in <tt>[ini,fin)</tt>, <tt>f(e) == true</tt>.
 * */
template <typename Iter, typename Predicate>
bool none_of (Iter ini, Iter fin, Predicate f) {}

/**
 * @}
 * */
} } 

#elif (CXXOMFORT_CXX_STD >= 2011 || \
	(CXXOMFORT_CXX_STD >= 1997 && CXXOMFORT_CXX_EMULATION == 2011))

	#define CXXOMFORT_IMPLEMENTS_n2569 CXXO_IMPLSTATUS_NATIVE()


#else

	#define CXXOMFORT_IMPLEMENTS_n2569 CXXO_IMPLSTATUS_BACKPORT()

namespace cxxomfort {
namespace cxxostd {


/**
 * @ingroup algorithm
 * @ingroup cxx11-backports
 * @see @wg21{n2569}
 * @{
 */

/**
 * @brief Returns @c true  if all elements in sequence <var>[ini,fin)</var> comply with predicate @p p .
 * @param p A predicate (a function that returns @c true or @c false ).
 * @xrefitem std0algorithm "<algorithm> backports" ""
 * 
 */
template< class Iterator, class Predicate >
bool all_of(Iterator ini, Iterator fin, Predicate p) {
    for (; ini != fin; ++ini) {
        if (!p(*ini)) return false;
    }
    return true;
}

/**
 * @brief Returns @c true  if at least one element in sequence <var>[ini,fin)</var> comply with predicate @p p .
 * @param p A predicate (a function that returns @c true or @c false ).
 * @xrefitem std0algorithm "<algorithm> backports" ""
 */
template< class Iterator, class Predicate >
bool any_of(Iterator ini, Iterator fin, Predicate p) {
    for (; ini != fin; ++ini) {
        if (p(*ini)) return true;
    }
    return false;
}

/**
 * @brief Returns @c true  if no elements in sequence <var>[ini,fin)</var> comply with predicate @p p .
 * @param p A predicate (a function that returns @c true or @c false ).
 * @xrefitem std0algorithm "<algorithm> backports" ""
 */
template< class Iterator, class Predicate >
bool none_of(Iterator ini, Iterator fin, Predicate p) {
    for (; ini != fin; ++ini) {
        if (p(*ini)) return false;
    }
    return true;
}

/**
 * @}
 */

} }  // cxxomfort::cxxostd

	#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=404) && CXXOMFORT_COMPILER_VERSION <= 406 && CXXOMFORT_CXX_EMULATION==2011 )
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1900))
	#else
namespace std {
	using ::cxxomfort::cxxostd::all_of;
	using ::cxxomfort::cxxostd::any_of;
	using ::cxxomfort::cxxostd::none_of;
}
	#endif

#endif
#undef  CXXOMFORT_IMPLEMENTS_algorithm_allanynoneof
#define CXXOMFORT_IMPLEMENTS_algorithm_allanynoneof CXXOMFORT_IMPLEMENTS_n2569

#endif
