#ifndef CXXOMFORT_IMPL_11_MOVE_TRAITS_HPP
#define CXXOMFORT_IMPL_11_MOVE_TRAITS_HPP
/**
 * @file
 */
#include <cxxomfort/base.hpp>
#include <cxxomfort/type_traits.hpp>

// is_trivially_move_.... for C++03
// is_nothrow_move_... for C++03
// These just return false for everything but scalar types

/*
 * is_move_constructible for C++03, we just look at scalarness
 */

namespace cxxomfort { namespace cxxostd {

template <typename T> 
struct is_trivially_move_constructible 
: std::is_scalar<T> {
};

template <typename T>
struct is_nothrow_move_constructible 
: std::is_scalar<T> {
};

template <typename T>
struct is_move_constructible
: std::integral_constant<bool,
    std::is_scalar<T>::value && !(std::is_same<T,void>::value)
> {};

} } // cxxomfort::cxxostd

#if (CXXOMFORT_CXX_STD < 2011 )
    #if (CXXOMFORT_CXX_EMULATION < 2011)
        #define CXXO_needs_move_xxxible_traits
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, < 408))
        #define CXXO_needs_move_xxxible_traits
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, < 1700))
        #define CXXO_needs_move_xxxible_traits
    #endif
#endif


#ifdef CXXO_needs_move_xxxible_traits
namespace std {
    using ::cxxomfort::cxxostd::is_trivially_move_constructible;
    using ::cxxomfort::cxxostd::is_nothrow_move_constructible;
    using ::cxxomfort::cxxostd::is_move_constructible;
}
#endif

// GCC 4.7~4.8 announce C++11 mode, but they still lack trivially_move_constructible for some reason
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >= 407)) \
    && (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, < 409)) \
    && (CXXOMFORT_CXX_STD == 2011)
namespace std {
    using ::cxxomfort::cxxostd::is_trivially_move_constructible;
}

#endif


#endif


