#ifndef CXXOMFORT_IMPL_11_TYPE_TRAITS_HPP
#define CXXOMFORT_IMPL_11_TYPE_TRAITS_HPP

/** 
 * @file
 * @brief Implements some of the type_traits added in C++11 that are *not* part of the TR1 specification
 * */

#include <cxxomfort/base.hpp>

// 
// pre-traits checks
// 

// at this point, <???/type_traits> should be included so we can access the types 

// we define the support level as follows:
// 0 - the compiler does not implement the trait is_foo<T> and does not implement the intrinsic __is_foo(T)
// 1 - the compiler implements only the intrinsic __is_foo(T)
// 2.- the compiler implements the intrinsic and the trait


#define CXXO_enable_is_trivial  (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_trivially_constructible (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_trivially_copyable (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_trivially_destructible (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_trivially_assignable (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_literal_type (2 * (CXXOMFORT_CXX_STD >= 2011))

//#pragma message "11-type_traits.hpp"

#if (CXXOMFORT_CXX_STD == 2011)
    // for whatever reason, GCC 4.8 c++0x announces mode as c++11, but still misses the triviality traits
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >=408))
        #undef  CXXO_enable_is_trivially_constructible
        #define CXXO_enable_is_trivially_constructible 1
        #undef  CXXO_enable_is_trivially_copyable
        #define CXXO_enable_is_trivially_copyable 1
        #undef  CXXO_enable_is_trivially_assignable
        #define CXXO_enable_is_trivially_assignable 1
    #endif
#elif (CXXOMFORT_CXX_STD < 2011)

    #if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_CLANG)
    // clang defines the intrinsics
        #undef  CXXO_enable_is_trivial
        #define CXXO_enable_is_trivial (1 + (CXXOMFORT_CXX_EMULATION >= 2011))
        #undef  CXXO_enable_is_trivially_constructible
        #define CXXO_enable_is_trivially_constructible (1 + (CXXOMFORT_CXX_EMULATION >= 2011))
        #undef  CXXO_enable_is_trivially_copyable
        #define CXXO_enable_is_trivially_copyable (1 + (CXXOMFORT_CXX_EMULATION >= 2011))

        #undef  CXXO_enable_is_literal_type
        #define CXXO_enable_is_literal_type (1 + (CXXOMFORT_CXX_EMULATION >= 2011))

    #elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_GCC)
        #if (CXXOMFORT_CXX_EMULATION == 2011) // in c++0x mode
            // we can find is_trivial, is_literal in namespace tr1
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 2
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 2
        #else
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 1
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 1
        #endif
        
        #if (CXXOMFORT_COMPILER_VERSION<406 && CXXOMFORT_CXX_EMULATION<2011)
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 0
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 0
        #elif (CXXOMFORT_CXX_EMULATION<2011)
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 1
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 1
        #endif

    #elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_MSC)
        #if (CXXOMFORT_COMPILER_VERSION>=1600)
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 2 // already available
        #elif (CXXOMFORT_COMPILER_VERSION>=1400)
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 1 // but check intrinsic
        #else // MSVC 2003?
            #undef  CXXO_enable_is_trivial
            #define CXXO_enable_is_trivial 0
        #endif
        #if (CXXOMFORT_COMPILER_VERSION>1600)
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 2
        #else
            #undef  CXXO_enable_is_literal_type
            #define CXXO_enable_is_literal_type 1
        #endif
        
        #if (CXXOMFORT_COMPILER_VERSION>=1700)
            #undef  CXXO_enable_is_trivially_constructible
            #define CXXO_enable_is_trivially_constructible 2
            #undef  CXXO_enable_is_trivially_copyable
            #define CXXO_enable_is_trivially_copyable 2
            #undef  CXXO_enable_is_trivially_assignable
            #define CXXO_enable_is_trivially_assignable 2
            #undef  CXXO_enable_is_trivially_destructible
            #define CXXO_enable_is_trivially_destructible 2
        #endif
    #endif

#endif


#define CXXO_enable_is_nothrow_constructible (2 * (CXXOMFORT_CXX_STD >= 2011))
#define CXXO_enable_is_nothrow_copyable (2 * (CXXOMFORT_CXX_STD >= 2011))

#if (CXXOMFORT_CXX_STD < 2011)

#endif // std


#if defined(DOXYGEN_DOC)

#elif (CXXOMFORT_CXX_STD < 1997)

    #include "07-type_traits.hpp"
    #include "11-decay.hpp"
    #include "11-conditional.hpp"
    #undef  CXXOMFORT_IMPLEMENTS_integral_constant
    #define CXXOMFORT_IMPLEMENTS_integral_constant CXXO_IMPLSTATUS_EMULATION()

#elif (CXXOMFORT_CXX_STD < 2011)

    #include "07-type_traits.hpp"
    #include "11-decay.hpp"
    #include "11-conditional.hpp"
    #include "11-common_type.hpp"
    //#include "11-is_constructible.hpp"
    #include "11-underlying_type.hpp"
    #undef  CXXOMFORT_IMPLEMENTS_integral_constant
    #define CXXOMFORT_IMPLEMENTS_integral_constant CXXO_IMPLSTATUS_BACKPORT()
    

#else
    #include <type_traits>
    #undef  CXXOMFORT_IMPLEMENTS_integral_constant
    #define CXXOMFORT_IMPLEMENTS_integral_constant CXXO_IMPLSTATUS_NATIVE()
#endif // std

// 
// if to enable is_trivially... traits
// (they have different names per compiler when using pre-C++11 intrinsics)
// 

#if (CXXOMFORT_CXX_STD >= 2014 || CXXOMFORT_CXX_EMULATION >= 2014)
#elif (CXXOMFORT_CXX_STD >= 2011 && CXXOMFORT_CXX_EMULATION > 0)
#elif (1 || CXXOMFORT_CXX_STD < 2011)
namespace std {

    #if (CXXO_enable_is_trivially_constructible==2)
        #if (CXXOMFORT_CXX_EMULATION >= 2011 && CXXOFLAG_namespace_tr1 && CXXOMFORT_COMPILER_ID!=CXXO_COMPILERID_MSC)
            //#pragma message "tr1"
            using tr1::is_trivially_default_constructible;
        #endif
    #elif (CXXO_enable_is_trivially_constructible==1)
        //#pragma message "__has"
    template <typename T>
    struct is_trivially_default_constructible
    : public integral_constant<bool, __has_trivial_constructor(T)>
    { };
    #elif (CXXO_enable_is_trivially_constructible==0)
        //#pragma message "emulation"
    template <typename T>
    struct is_trivially_default_constructible
    : public integral_constant<bool, is_fundamental<T>::value && !is_void<T>::value>
    { };
    #endif

    #if (CXXO_enable_is_trivially_copyable==1)
    template <typename T>
    struct is_trivially_copy_constructible
    : public integral_constant<bool, __has_trivial_copy(T)>
    { };
    #elif (CXXO_enable_is_trivially_copyable==0)
    template <typename T>
    struct is_trivially_copy_constructible
    : public is_fundamental<T> 
    { };
    #endif

    #if (CXXO_enable_is_trivially_assignable==1)
    template <typename T>
    struct is_trivially_copy_assignable
    : public integral_constant<bool, __has_trivial_assign(T)>
    { };
    #elif (CXXO_enable_is_trivially_assignable==0)
    template <typename T>
    struct is_trivially_copy_assignable
    : public is_fundamental<T> 
    { };
    #endif

    #if (CXXO_enable_is_trivially_destructible==1)
    template <typename T>
    struct is_trivially_destructible
    : public integral_constant<bool, /*__has_trivial_destructor(T)*/ is_fundamental<T>::value && !is_void<T>::value>
    { };
    #elif (CXXO_enable_is_trivially_destructible==0)
    template <typename T>
    struct is_trivially_destructible
    : public integral_constant<bool, is_fundamental<T>::value && !is_void<T>::value>
    { };
    #endif


} // std

// 
// is_standard_layout 
// 

#define CXXO_enable_standard_layout (2 * (CXXOMFORT_CXX_STD >= 2011))

#if (CXXOMFORT_CXX_STD < 2011) 
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, >= 1600))
        #undef  CXXO_enable_standard_layout
        #define CXXO_enable_standard_layout 2
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >=406))
        #undef  CXXO_enable_standard_layout
        #define CXXO_enable_standard_layout (1 + (CXXOMFORT_CXX_EMULATION==2011))
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG, >= 300))
        #undef  CXXO_enable_standard_layout
        #define CXXO_enable_standard_layout (1 + (CXXOMFORT_CXX_EMULATION==2011))
    #endif
#endif

#if defined(DOXYGEN_DOC)
namespace std {
	/**
	 * @brief Implementation of @c is_standard_layout .
	 * @ingroup std0typetraits
	 * 
	 * Provides the member @c value with the following conditions:
	 * 
	 * * equals <tt>__is_standard_layout(T)</tt> if the intrinsic is available
	 * * equals <tt>is_scalar<T>::value</tt> otherwise.
	 * 
	 * */
}
#elif (CXXO_enable_standard_layout==2)
#elif (CXXO_enable_standard_layout==1)
namespace std {
    template <typename T>
    struct is_standard_layout 
    : integral_constant <bool, __is_standard_layout(T) >
    { };
}
#elif (CXXO_enable_standard_layout==0)
namespace std {
    template <typename T>
    struct is_standard_layout 
    : integral_constant <bool, is_scalar<T>::value >
    { };
}
#else
namespace std {

    template <typename T>
    struct is_standard_layout
    : integral_constant<bool,
    #if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_MSC)
        #if (CXXOMFORT_COMPILER_VERSION>=1600)
        __is_standard_layout(T)
        #else
        false
        #endif
    #else
        is_scalar<T>::value
    #endif

    > 
    { };

} // namespace std
    #undef CXXO_needs_standard_layout
#endif


// 
// is_trivial, is_literal_type 
// 


    #if (CXXO_enable_is_literal_type==1)
namespace std {
        #if (defined(__is_literal_type) || CXXO_has_literal_intrinsic_==1)
    template <typename T>
    struct is_literal_type
    : integral_constant<bool, __is_literal_type(T)> {};
        #else
    // a C++03 literal type is a scalar or a reference
    template <typename T>
    struct is_literal_type
    : integral_constant<bool, cxxomfort::traits::is_scalar<T>::value || cxxomfort::traits::is_reference<T>::value > {};
    
        #endif
} // std
    #endif // enable_is_literal

    #if (CXXO_enable_is_trivial==1)
namespace std {
        #if (defined(__is_trivial) || CXXO_has_trivial_intrinsic==1)
    template <typename T>
    struct is_trivial
    : integral_constant<bool, __is_trivial(T)> {};

        #else
    // a C++03 trivial type is a scalar or a class whose construction and destruction behaves like that of a scalar
    template <typename T>
    struct is_trivial
    : integral_constant<bool
        , cxxomfort::traits::is_scalar<T>::value || 
        ( cxxomfort::traits::is_class<T>::value && is_trivially_default_constructible<T>::value 
        && is_trivially_copy_constructible<T>::value && is_trivially_destructible<T>::value 
        )
    > {};

        #endif
} // std
    #endif // enable_is_trivial

    #undef CXXO_enable_is_trivial
    #undef CXXO_enable_is_literal


#endif // std < 2011

// 
// has_virtual_destructor
// provided by GCC TR1, MSVC 2008 SP1, Clang 3.2(?)
// 

#if defined(DOXYGEN_DOC)
namespace std {
	/**
	 * @brief Implementation of @c is_standard_layout .
	 * @ingroup std0typetraits
	 * 
	 * If and only if the intrinsic <tt>__has_virtual_destructor()</tt> exists, cxxomfort provides this type traits that exposes the member @c value with the value <tt>__has_trivial_destructor(T)</tt>.
	 * 
	 * The macro @c CXXOMFORT_IMPLEMENTS_std_has_virtual_destructor informs if this type trait is provided.
	 * 
	 * */
	 template <typename T>
	 struct has_virtual_destructor {};
}

#elif (CXXOMFORT_CXX_STD >= 2011)
	#define CXXOMFORT_IMPLEMENTS_std_has_virtual_destructor CXXO_IMPLSTATUS_NATIVE()
#elif (CXXOMFORT_CXX_EMULATION >= 2011)
	#define CXXOMFORT_IMPLEMENTS_std_has_virtual_destructor CXXO_IMPLSTATUS_NATIVE()
#elif (( CXXOMFORT_CXX_STD >= 1997 && CXXOMFORT_CXX_EMULATION < 2011 ))
	#define CXXOMFORT_IMPLEMENTS_std_has_virtual_destructor CXXO_IMPLSTATUS_BACKPORT()
namespace std {
    template <typename T>
    struct has_virtual_destructor 
    : public integral_constant <bool, __has_virtual_destructor(T)>
    { };
}

#else
	#define CXXOMFORT_IMPLEMENTS_std_has_virtual_destructor 0
#endif // has_virtual_destructor


#if defined(DOXYGEN_DOC)
namespace std {
	//! @cxx11b Type traits.
	//! @see @cppref{utility/type_traits}
	template <typename T>
	struct is_trivial {};

	//! cxx11b Type traits,
	//! @see @cppref{utility/type_traits}
	template <typename T>
	struct is_literal {};

	//! cxx11b Type traits,
	//! @see @cppref{utility/type_traits}
	template <typename T>
	struct has_virtual_destructor {};
	
}

#endif

#endif // guard
