#ifndef CXXOMFORT_IMPL_TYPEINDEX_TYPE_INDEX_HPP
#define CXXOMFORT_IMPL_TYPEINDEX_TYPE_INDEX_HPP
/**
 * @file
 */

/*
 * 
 * 
 */

#include <cxxomfort/base.hpp>
#include <typeinfo>
#include <cxxomfort/functional.hpp> // std::hash
#include <cxxomfort/impl/relationals.hpp>
//#include <string> // for debug of hash_code

namespace cxxomfort { namespace cxxostd { 

//! @internal
static inline size_t ty_id () {
    static size_t i= 0;
    return i++;
}

template <typename T>
struct impl_hash_code {
    static const size_t value;
};

template<typename T> 
const size_t impl_hash_code<T>::value = ty_id();

/**
 * An orderable, comparable view of a <code>std::type_info</code>.
 * 
 * */
class type_index {
    public:
    type_index (std::type_info const& t)
    : TR(&t) {}
    
    type_index& operator= (std::type_info const& r) CXXO_NOEXCEPTNOTHROW {
        TR= &r;
        return *this;
    }
    
    char const*  name () const CXXO_NOEXCEPT { return TR->name(); }
    size_t       hash_code () const CXXO_NOEXCEPT { 
#if (CXXOMFORT_CXX_STD >= 2011)
        return TR->hash_code(); 
#else
        // in C++03 there's no type_info::hash_code
        // we go to hash<const char*>
        using namespace std;
        //hash<char const*> const H ();
        return hash<char const*>()( TR->name() );
#endif
    }
    
    friend bool     operator== (type_index const& l, type_index const& r) CXXO_NOEXCEPTNOTHROW {
        return l.TR==r.TR;
    }
    friend bool     operator<  (type_index const& l, type_index const& r) CXXO_NOEXCEPTNOTHROW {
#ifdef _MSC_VER
        return ((l.TR)->before(*r.TR) != 0);
#else
        return static_cast<bool>((l.TR)->before(*r.TR));
#endif
    }

    private:
    std::type_info const* TR;
    
};

CXXO_GENERATE_RELATIONALS(type_index);

} }

#endif // 


