#ifndef CXXOMFORT_IMPL_UNDERLYING_TYPE_HPP
#define CXXOMFORT_IMPL_UNDERLYING_TYPE_HPP

/**
 * @file
 */
#include <cxxomfort/config.hpp>
#include <cxxomfort/cxxo-utils.hpp>
#include <cxxomfort/impl/to_signed.hpp> // make_signed; integral_of_size
#include <cxxomfort/impl/11-conditional.hpp> // conditional

#ifdef DOXYGEN_DOC
namespace std {
/**
* @brief Given an enumeration type, provides the integral type serving as its storage.
* @ingroup cxx11-backports
* @xrefitem std0type_traits "<type_traits>" ""
* @anchor underlying_type
* 
* @pre <tt>std::is_enum< EnumeratedType >::value == true </tt>
* 
* @note For pre-C++11 compilers this is backported taking advantage of also backporting @c make_signed and @c make_unsigned ; however take note that this might lead to incorrect results.
* 
* */
template <typename EnumeratedType> struct underlying_type { 
    typedef SomeIntegralType type; 
};


#endif // doc


#define CXXOMFORT_IMPLEMENTS_n2984_1055 CXXO_IMPLSTATUS_EMULATION()
#undef  CXXOMFORT_IMPLEMENTS_underlying_type
#define CXXOMFORT_IMPLEMENTS_underlying_type CXXOMFORT_IMPLEMENTS_n2984_1055

#if 0
#elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_GCC)
    // Add underlying_type approximation for GCC < 4.7 in any mode
    // see http://stackoverflow.com/questions/9343329/how-to-know-underlying-type-of-class-enum/10956467#10956467
    #if (CXXOMFORT_COMPILER_VERSION < 407)
        #define CXXOMFORT_USING_underlying_type 1
    #elif (CXXOMFORT_COMPILER_VERSION >= 407 && CXXOMFORT_CXX_STD < 2011 )
        #define CXXOMFORT_USING_underlying_type 1
    #endif
#elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_CLANG)
    #if (CXXOMFORT_CXX_STD >= 2011)
    #else
        #define CXXOMFORT_USING_underlying_type 1
    #endif
#elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_MSC)
    #if (CXXOMFORT_CXX_STD<2011)
        #if (CXXOMFORT_COMPILER_VERSION<1700)
            #define CXXOMFORT_USING_underlying_type 1
        #endif
    #endif
#elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_OW)
	#define CXXOMFORT_USING_underlying_type 1
#else

#endif // support for underlying_type

#if (defined(CXXOMFORT_USING_underlying_type))
	#undef  CXXOMFORT_IMPLEMENTS_underlying_type
	#define CXXOMFORT_IMPLEMENTS_underlying_type CXXO_IMPLSTATUS_BACKPORT()

    #if defined (CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1)
        #pragma message CXXO_WARNING( " cxxomfort library - enabled underlying_type helper" )
    #endif

    #if (CXXOMFORT_CXX_STD < 2011)
namespace {
    using cxxomfort::traits::make_signed;
    using cxxomfort::traits::make_unsigned;
}
    #endif

namespace cxxomfort { namespace cxxostd {

    template< typename TpEnum >
    struct underlying_type_impl {
        typedef typename cxxomfort::traits::integral_of_size<sizeof(TpEnum)>::type tsize_t;
        private:
        typedef typename make_signed< tsize_t >::type S;
        typedef typename make_unsigned< tsize_t >::type U;
        public:
        typedef typename std::conditional<
            (TpEnum( -1 ) < TpEnum( 0 )), S, U
            >::type type;
    };
} }

namespace std {
    template <typename T>
    struct underlying_type 
    : std::conditional< is_enum<T>::value , typename ::cxxomfort::cxxostd::underlying_type_impl<T>, ::cxxomfort::EmptyT<T> 
    > {};


}

#else // At this point there should be full C++11 support for underlying_type
	#undef  CXXOMFORT_IMPLEMENTS_underlying_type
	#define CXXOMFORT_IMPLEMENTS_underlying_type CXXO_IMPLSTATUS_NATIVE()

#endif // underlying_type


#endif

