#ifndef CXXOMFORT_IMPL_FUNCTIONAL_14FUNCTORS_HPP
#define CXXOMFORT_IMPL_FUNCTIONAL_14FUNCTORS_HPP
/**
 * @file
 * @brief Implementation of C++14's transparent functors.
 * @ingroup std0functional
 *
 * This header is part of cxxomfort and is included automagically when #including <tt><cxxomfort/functional.hpp></tt>.  It should not be #included explicitly.
 * 
 * 
 * 
 */


#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>
#include <functional>

#if defined(DOXYGEN_DOC)

namespace std {

    /**
    * @ingroup std0functional
    * @{
    * */

    /**
     * @brief Transparent @em equal_to functor.
     * @cxx14b Type-transparent comparator.
     * */
    template<> struct equal_to<void>  {};


    /**
     * @brief Transparent @em not_equal_to functor.
     * @cxx14b Type-transparent comparator.
     * */
    template<> struct not_equal_to<void>  {};

    /**
     * @brief Transparent @em less functor.
     * @cxx14b Type-transparent comparator.
     * */
    template<> struct less<void>  {};

    /**
     * @brief Transparent @em greater functor.
     * @cxx14b Type-transparent comparator.
     * */
    template<> struct greater<void>  {};



    /**
     * @brief Transparent @em plus operator.
     * @cxx14b Type-transparent operator.
     * */
    template<> struct plus<void>  {};
    
    /**
     * @}
     * */

}

    /**
     * @ingroup std00featuremacros
     * @cppfeature{Transparent Operators}
     * */
    #define __cpp_lib_transparent_operators YYYY00L
    #define CXXOMFORT_IMPLEMENTS_n3421 CXXO_IMPLSTATUS_NATIVE()

#elif (defined(__cpp_lib_transparent_operators))
    #undef  CXXO_COMPILER_SUPPORT_transparent_operators_functional
    #define CXXO_COMPILER_SUPPORT_transparent_operators_functional 1
    #define CXXOMFORT_IMPLEMENTS_n3421 CXXO_IMPLSTATUS_NATIVE()

#elif (CXXO_COMPILER_SUPPORT_transparent_operators_functional>0)
    #define __cpp_lib_transparent_operators (CXXOMFORT_CXX_STD*100)
    #define CXXOMFORT_IMPLEMENTS_n3421 CXXO_IMPLSTATUS_NATIVE()

#else

#endif

// 
// transparent comparators 
// 


#if (defined(CXXOMFORT_IMPLEMENTS_n3421))
    
#elif (CXXOMFORT_CXX_STD>=1)
    #define CXXOMFORT_IMPLEMENTS_n3421 CXXO_IMPLSTATUS_BACKPORT()
#else
    #define CXXOMFORT_IMPLEMENTS_n3421 0 
#endif // cxx-std

#if (CXXO_COMPILER_SUPPORT_transparent_operators_functional >0)

#else
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
		#pragma message CXXO_NOTICE("enabled <functional> transparent functors")
	#endif
	#define __cpp_lib_transparent_operators (CXXOMFORT_CXX_STD*100)

	#include <cxxomfort/utility.hpp> // declval
// 
// comparators 
// 

	#define CXXO_TRANSPARENT_COMPARATOR(Op,Sym) \
	typedef bool return_type; \
	struct is_transparent {};  \
	template <typename T1, typename T2> \
	CXXO_CONSTEXPR inline \
	bool operator() (T1 const& t1, T2 const& t2) const { return t1 Sym t2; }  \
	
	
namespace std {

    template<> struct equal_to<void>  {
		CXXO_TRANSPARENT_COMPARATOR(equal_to , == );
	};
    template<> struct not_equal_to<void> {
		CXXO_TRANSPARENT_COMPARATOR(not_equal_to , != );
	};
    template<> struct less<void> {
		CXXO_TRANSPARENT_COMPARATOR(less , < );
	};
    template<> struct greater<void> {
		CXXO_TRANSPARENT_COMPARATOR(greater, > );
	};
    template<> struct less_equal<void> {
		CXXO_TRANSPARENT_COMPARATOR(less_equal, <= );
	};
    template<> struct greater_equal<void> {
		CXXO_TRANSPARENT_COMPARATOR(greater_equal, >= );
	};
}

	#if (CXXO_COMPILER_SUPPORT_auto && CXXO_COMPILER_SUPPORT_rvref)
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("transparent functors with auto")
		#endif
	
		#define CXXO_TRANSPARENT_OPERATOR(Op,Sym) \
	struct is_transparent {};  \
	template <typename T1, typename T2> \
    auto operator() (T1&& v1, T2&& v2) const \
    -> decltype( std::forward<T1>(v1) Sym std::forward<T2>(v2) ) {  \
        return std::forward<T1>(v1) Sym std::forward<T2>(v2); \
    }  \
    template <typename T> struct result { typedef void type; };  \

	#elif (0 && CXXO_COMPILER_SUPPORT_typeof)
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("transparent functors with typeof")
		#endif
		#define CXXO_TRANSPARENT_OPERATOR(Op,Sym) \
	struct is_transparent {};  \
	template <typename T1, typename T2> \
    CXXO_TYPEOF(declval<T1>() Sym declval<T2>()) \
    operator() (T1 const& v1, T2 const& v2) const {  \
        return v1 Sym v2; \
    }  \
    template <typename T> struct result { typedef void type; };  \
	
	#else
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("transparent functors resolve to LHS type")
		#endif
		#define CXXO_TRANSPARENT_OPERATOR(Op,Sym) \
	struct is_transparent {};  \
	template <typename T1, typename T2> \
    T1 operator() (T1 const& v1, T2 const& v2) const  {  \
        return v1 Sym v2; \
    } \
    template <typename T> struct result { typedef void type; };  \

	#endif

// 
// bitwise 
// 

namespace std {
	template<> struct bit_and<void> {
		CXXO_TRANSPARENT_OPERATOR(bit_and, & );
	};
	template<> struct bit_or<void> {
		CXXO_TRANSPARENT_OPERATOR(bit_or, | );
	};
	template<> struct bit_xor<void> {
		CXXO_TRANSPARENT_OPERATOR(bit_xor, ^ );
	};
}

// 
// arithmetic 
// 

namespace std {
	template<> struct plus<void> {
		CXXO_TRANSPARENT_OPERATOR(plus, + );
	};
	template<> struct minus<void> {
		CXXO_TRANSPARENT_OPERATOR(minus, - );
	};
	template<> struct multiplies<void> {
		CXXO_TRANSPARENT_OPERATOR(multiplies, * );
	};
}

#endif // transparent operators


#include <cxxomfort/type_traits.hpp> // common_type
#include "../type_traits.hpp"
#include "../impl/11-common_type.hpp"

namespace cxxomfort {
namespace cxxostd {

/**
 * @ingroup cxx14-backports
 * @{
 */
template <typename T> struct less
: std::less<T> {};
template <typename T> struct greater
: std::greater<T> {};
template <typename T> struct less_equal
: std::less_equal<T> {};
template <typename T> struct greater_equal
: std::greater_equal<T> {};
template <typename T> struct equal_to
: std::equal_to<T> {};
template <typename T> struct not_equal_to
: std::not_equal_to<T> {};

/**
 * @}
 */
} }

// this was missing from the original STL
template <typename T = void> struct bit_not {
    typedef T result_type;
    typedef T argument_type;
    inline T operator() (T const& t) const { return ~t; }
};


#if (CXXOMFORT_CXX_STD < 2011)

// Under C++03, one can not trust that std::common_type is available
// (because no decltype), so we assume that heterogeneous functors
// return homogeneously to the left (aka: they return the type of 1st operand).

    #define LPP_SPEC_FUNCTIONAL_VOID(Op,Sym) \
template<> struct Op <void>    \
{    \
    template <typename T1, typename T2>     \
    inline typename \
    std::remove_reference<T1>::type operator() (T1 const& a, T2 const& b) const { return a Sym b; } \
    template <typename T> struct result { typedef T type; }; \
    static const bool is_transparent = true;  \
}; \
template <typename T1, typename T2> struct Op<void>:: result< Op<void>(T1,T2) > { \
    typedef T1 type; \
}; \


#else

    #define LPP_SPEC_FUNCTIONAL_VOID(Op,Sym) \
template<> struct Op <void> { \
    struct is_transparent {}; \
    \
    template <typename T1, typename T2>  \
    auto \
    operator() (T1&& v1, T2&& v2) const \
    -> decltype( std::forward<T1>(v1) Sym std::forward<T2>(v2) ) \
    {  \
        return std::forward<T1>(v1) Sym std::forward<T2>(v2); \
    }  \
    template <typename T> struct result { typedef void type; };  \
}; \
template <typename T1, typename T2> struct Op<void>:: result< Op<void>(T1,T2) > { \
    typedef typename std::common_type<T1,T2>::type type; \
}; \

#endif

#undef LPP_SPEC_FUNCTIONAL_VOID


// for the facilities to function correctly they are rebuilt in
// namespace std; but we give users the option not to.
#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#else
/*
 * Transparent functors were established in C++14
 */
	#if (CXXOMFORT_IMPLEMENTS_n3421 == CXXO_IMPLSTATUS_BACKPORT() )
namespace std {


}
	#endif // c++14

#endif // no std

#undef  CXXOMFORT_IMPLEMENTS_heterogeneous_operator_functors
#define CXXOMFORT_IMPLEMENTS_heterogeneous_operator_functors CXXOMFORT_IMPLEMENTS_n3421

#endif // CXXOMFORT_EXTRAS_14FUNCTORS_HPP
