#ifndef CXXOMFORT_IMPL_MAKE_UNIQUE_HPP
#define CXXOMFORT_IMPL_MAKE_UNIQUE_HPP
/**
 * @file
 */
#include "../base.hpp"
#include <memory>

#if defined(DOXYGEN_DOC)
namespace cxxomfort { namespace cxxostd {

/**
 * @brief Creates a unique_ptr holding a constructed object
 * @ingroup std0memory
 * @cxx14b Create a @c unique_ptr holding a @a T constructed by the given arguments
 * 
 * @note For compilers in pre-C++11 mode, the function can copy (forward) up to 6 arguments to the @a T constructor.
 * 
 */
template <typename T, typename ...Args> inline
std::unique_ptr<T> make_unique (Args&&... args) {
    return std::unique_ptr<T>( new T(std::forward<Args>(args)... ) ) 
}

} }

#elif (defined(__cpp_lib_make_unique))
	#define CXXOMFORT_IMPLEMENTS_n3588 CXXO_IMPLSTATUS_NATIVE()
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1800))
	#define CXXOMFORT_IMPLEMENTS_n3588 CXXO_IMPLSTATUS_NATIVE()
	#define __cpp_lib_make_unique (CXXOMFORT_CXX_STD*100)
#elif (CXXOMFORT_IMPLEMENTS_unique_ptr>0)
	#define CXXOMFORT_IMPLEMENTS_n3588 CXXO_IMPLSTATUS_BACKPORT()
#else 
	#define CXXOMFORT_IMPLEMENTS_n3588 0 // can't be implemented
#endif


#if (CXXOMFORT_IMPLEMENTS_n3588==CXXO_IMPLSTATUS_BACKPORT())
	#define __cpp_lib_make_unique (CXXOMFORT_CXX_STD*100)

namespace cxxomfort { namespace cxxostd { 

	#if (CXXOMFORT_CXX_STD >= 2011)

template <typename T, typename ...Args> inline
std::unique_ptr<T> make_unique (Args&&... args) {
    return std::unique_ptr<T>( new T(std::forward<Args>(args)... ) );
}

	#else

template <typename T> inline
std::unique_ptr<T> make_unique () {
    return std::unique_ptr<T> (new T());
}
template <typename T, typename A1> inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1) {
    return std::unique_ptr<T> (new T( std::forward<A1>(a1) ));
}
template <typename T, typename A1, typename A2> inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2) {
    return std::unique_ptr<T> (new T( std::forward<A1>(a1), std::forward<A2>(a2) ));
}
template <typename T, typename A1, typename A2, typename A3> inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2, CXXO_FWD_REF(A3) a3) {
    return std::unique_ptr<T> (new T( 
       std::forward<A1>(a1), std::forward<A2>(a2), std::forward<A3>(a3) ));
}
template <typename T, typename A1, typename A2, typename A3, typename A4> inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2, CXXO_FWD_REF(A3) a3, CXXO_FWD_REF(A4) a4) {
    return std::unique_ptr<T> (new T(
       std::forward<A1>(a1), std::forward<A2>(a2), std::forward<A3>(a3)
     , std::forward<A4>(a4) ));
}
template <typename T, typename A1, typename A2, typename A3, typename A4, typename A5> inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2, CXXO_FWD_REF(A3) a3, CXXO_FWD_REF(A4) a4, CXXO_FWD_REF(A5) a5) {
    return std::unique_ptr<T> (new T(
      std::forward<A1>(a1), std::forward<A2>(a2), std::forward<A3>(a3), 
      std::forward<A4>(a4), std::forward<A5>(a5) ));
}
template <typename T
, typename A1, typename A2, typename A3, typename A4, typename A5
, typename A6 > inline
std::unique_ptr<T> make_unique (CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2, CXXO_FWD_REF(A3) a3, CXXO_FWD_REF(A4) a4, CXXO_FWD_REF(A5) a5, CXXO_FWD_REF(A6) a6) {
    return std::unique_ptr<T> (new T(
      std::forward<A1>(a1), std::forward<A2>(a2), std::forward<A3>(a3), 
      std::forward<A4>(a4), std::forward<A5>(a5), std::forward<A6>(a6) 
      ));
}
template <typename T
, typename A1, typename A2, typename A3, typename A4, typename A5
, typename A6, typename A7 > inline
std::unique_ptr<T> make_unique (
  CXXO_FWD_REF(A1) a1, CXXO_FWD_REF(A2) a2, CXXO_FWD_REF(A3) a3, CXXO_FWD_REF(A4) a4, CXXO_FWD_REF(A5) a5
, CXXO_FWD_REF(A6) a6, CXXO_FWD_REF(A7) a7) {
    return std::unique_ptr<T> (new T(
      std::forward<A1>(a1), std::forward<A2>(a2), std::forward<A3>(a3), 
      std::forward<A4>(a4), std::forward<A5>(a5), std::forward<A6>(a6),
      std::forward<A7>(a7)
      ));
}

	#endif

} }  // cxxomfort::cxxostd

namespace std {
	using ::cxxomfort::cxxostd::make_unique;
}


#endif

#undef  CXXOMFORT_IMPLEMENTS_make_unique
#define CXXOMFORT_IMPLEMENTS_make_unique CXXOMFORT_IMPLEMENTS_n3588

// this is included after unique_ptr

// Add make_unique
// equivalent to return std::std::unique_ptr<T>(new T(std::forward<Args>(args)...));


#endif

