#ifndef CXXOMFORT_IMPL_14_TYPE_TRAITS_HPP
#define CXXOMFORT_IMPL_14_TYPE_TRAITS_HPP

/*
 * Implements some of the type_traits added in C++11 that are *not* part of the TR1 specification
 * */

#include <cxxomfort/base.hpp>

// 
// is_null_pointer (also provided by cxxomfort)
// 

#if (defined(__cpp_lib_is_null_pointer))
// these implementations provide is_null_pointer
#elif (CXXOMFORT_CXX_STD == 2011 && CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_CLANG)
#elif (CXXOMFORT_CXX_STD == 2011 && CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >=500 ))
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, >= 1900))
#else
namespace std {
    using ::cxxomfort::traits::is_null_pointer;
}
	#define __cpp_lib_is_null_pointer 201103
#endif


// 
// is_final
// 

#if defined(DOXYGEN_DOC)
namespace std {
	//! @xrefitem std0type_traits Type traits.
	//! @cxx14b Type traits.
	template <typename T> struct is_final {};
}
#elif (defined(__cpp_lib_is_final))
	// nothing to do
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1900))
	// nothing to do
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG, >=500) && CXXOMFORT_CXX_EMULATION >= 2014)
	// nothing to do
#else

namespace std {
	template <typename T>
	struct is_final
	: integral_constant<bool,
	#if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, > 1800))
		__is_final(T)
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >= 600))
		__is_final(T)
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG, >= 300))
		__is_final(T)
	#else
		false
	#endif
	> 
	{ };
} // namespace
	#define __cpp_lib_is_final 201103
#endif

#endif
