#ifndef CXXOMFORT_IMPL_AS_CONST_HPP
#define CXXOMFORT_IMPL_AS_CONST_HPP
/**
 * @file
 * @brief Implements std::add_const by David Alan Martin
 * @author Luis Machuca Bezzaza <luis [dot] machuca [at] gulix [dot] cl>
 *
 */


#define CXXOMFORT_IMPLEMENTS_n4380 CXXO_IMPLSTATUS_BACKPORT()
#define CXXOMFORT_IMPLEMENTS_as_const CXXOMFORT_IMPLEMENTS_n4380

#include "../config.hpp"
#include <utility>
#include "../impl/07-type_traits.hpp" // add_const


#if defined(DOXYGEN_DOC)
namespace std {

/**
 * @brief returns a @c const  view of object @p t . A @ref cxx17-backports .
 * @return A reference to its argument with @c const qualifier added.
 * @ingroup cxx17-backports
 * @see @wg21{n4380}
 * @cxx17b @c const view of an object
 * @xrefitem std:utility "<utility>" "" @c const view of an object
 */
template< typename T >
inline constexpr typename std::add_const< T >::type & 
as_const( T& t ) noexcept {
}

/**
 * @brief returns a @c const  view of a pointer @p t . An extension to @c as_const() .
 * @return A pointer to a const view of the argument.
 * @cxx17b @c const view of an object
 * @xrefitem std0utility "<utility>" "" @c const view of an object
 * @see @wg21{p1011r0} , @ref as_const()
 */
template< typename T >
inline constexpr typename std::add_const< T >::type *
as_ptr_const( T* const t ) noexcept {
}

}

	/**
	 * @ingroup std00featuremacros
	 * @cppfeature{@ref @c as_const() }
	 * */
	#define __cpp_lib_as_const YYYY00L

#elif defined(__cpp_lib_as_const)

#else
	// extend with p1011r0 - as_ptr_const
	#define CXXOMFORT_IMPLEMENTS_p1011r0 CXXO_IMPLSTATUS_BACKPORT()
	#define CXXOMFORT_IMPLEMENTS_as_ptr_const CXXOMFORT_IMPLEMENTS_p1011r0

namespace std {

template< typename T >
inline CXXO_CONSTEXPR typename cxxomfort::traits::add_const< T >::type & 
as_const( T& t ) CXXO_NOEXCEPT {
    return t;
}


/**
 * @brief returns a @c const  view of a pointer @p t . An extension to @c as_const() .
 * @return A pointer to a const view of the argument.
 * @cxx17b "Type Utilities" as_const
 * @xrefitem std0utility "<utility>" ""
 * @see @wg21{p1011r0} , @ref as_const()
 */
template< typename T >
inline CXXO_CONSTEXPR14 typename cxxomfort::traits::add_const< T >::type *
as_ptr_const( T* const t ) CXXO_NOEXCEPTNOTHROW {
    return t;
}
    
} // std

	#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
	#else
		#define __cpp_lib_as_const (CXXOMFORT_CXX_STD*100)
	#endif

#endif


#endif // file
