#ifndef CXXOMFORT_IMPL_MAKE_FROM_TUPLE_HPP
#define CXXOMFORT_IMPL_MAKE_FROM_TUPLE_HPP
/**
 * @file
 * @brief Implementation of std::make_from_tuple
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/tuple.hpp>

#ifdef DOXYGEN_DOC
namespace cxxomfort { namespace cxxostd {
    /**
     * @brief Create an object from a tuple expression
     * @ingroup std0tuple
     * @cxx17b Constructs an object of type @a T using the tuple's elements for the constructor.
     * @return An object of type @a T , constructed via a constructor call using a tuple as arguments.
     * @see  @cppref{utility/make_from_tuple}
     * 
     * The call <code>make_from_tuple<T>( footuple )</code> is 
     * equivalent to the call <code>T (get<0>(footuple), get<1>(footuple), get<2>(footuple), ...)</code>.
     */
    template <typename T, typename... TupleArgs>
    T make_from_tuple (std::tuple<TupleArgs...>&& Tuple) {}
    
} }



#elif defined(__cpp_lib_make_from_tuple)
	#define CXXOMFORT_IMPLEMENTS_p0209 CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD >= 2011)
	#define CXXOMFORT_IMPLEMENTS_p0209 (CXXO_IMPLSTATUS_BACKPORT()+(CXXO_COMPILER_SUPPORT_constexpr_relaxed>0?14:11))
	#include <cxxomfort/utility.hpp> // index_sequence
	#include <type_traits>
	

namespace cxxomfort { namespace cxxostd {

namespace detail {
template <class T, class Tuple, std::size_t... I>
CXXO_CONSTEXPR14 T make_from_tuple_impl( Tuple&& t, std::index_sequence<I...> )
{
  return T(std::get<I>(std::forward<Tuple>(t))...);
}

} // namespace detail
 
template <class T, class Tuple>
CXXO_CONSTEXPR14 T make_from_tuple( Tuple&& t ) {
	typedef typename std::remove_reference<Tuple>::type UnrefTuple;
    return detail::make_from_tuple_impl<T>(std::forward<Tuple>(t),
        std::make_index_sequence<std::tuple_size <UnrefTuple>::value>{}
        );
}


} }

#elif (CXXOMFORT_IMPLEMENTS_tuple > 0)

	#define CXXOMFORT_IMPLEMENTS_p0209 CXXO_IMPLSTATUS_EMULATION()

namespace cxxomfort { namespace cxxostd {

namespace detail {
template <typename R> 
struct maker_from_tuple_t {
    template <typename T0>
    R operator() (std::tuple<T0> const& tt) const {
        using std::get;
        return R( get<0>(tt) );
    }

    template <typename T0, typename T1>
    R operator() (std::tuple<T0,T1> const& tt) const {
        using std::get;
        return R( get<0>(tt), get<1>(tt) );
    }

    template <typename T0, typename T1, typename T2>
    R operator() (std::tuple<T0,T1,T2> const& tt) const {
        using std::get;
        return R( get<0>(tt), get<1>(tt), get<2>(tt) );
    }

    template <typename T0, typename T1, typename T2, typename T3>
    R operator() (std::tuple<T0,T1,T2,T3> const& tt) const {
        using std::get;
        return R( get<0>(tt), get<1>(tt), get<2>(tt), get<3>(tt) );
    }

    template <typename T0, typename T1, typename T2, typename T3, typename T4>
    R operator() (std::tuple<T0,T1,T2,T3,T4> const& tt) const {
        using std::get;
        return R( get<0>(tt), get<1>(tt), get<2>(tt), get<3>(tt), get<4>(tt) );
    }
    
};
    

} // detail

template <typename T, typename Tuple>
T make_from_tuple (Tuple const& t) {
    detail::maker_from_tuple_t<T> M;
    return M(t);
}

} }


#endif



#undef  CXXOMFORT_IMPLEMENTS_make_from_tuple
#define CXXOMFORT_IMPLEMENTS_make_from_tuple CXXOMFORT_IMPLEMENTS_p0209 

#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#elif (CXXOMFORT_IMPLEMENTS_make_from_tuple >0 && CXXOMFORT_IMPLEMENTS_make_from_tuple<CXXO_IMPLSTATUS_NATIVE())
	#if (CXXOMFORT_IMPLEMENTS_make_from_tuple>=CXXO_IMPLSTATUS_BACKPORT())
		#define __cpp_lib_make_from_tuple 201103
	#endif

    namespace std {
        using cxxomfort::cxxostd::make_from_tuple;
    }
#endif // using std


#endif
