#ifndef CXXOMFORT_IMPL_07_MEM_FN_HPP
#define CXXOMFORT_IMPL_07_MEM_FN_HPP
/**
 * @file
 * @brief Barebones implementation of ptr_fun, mem_fun, mem_fun_ptr and mem_fn
 * 
 * This is an implementation of the function wrappers in C++07/TR1
 * not directly used by cxxomfort but made available for TR1 compatibility layers.
 * 
 * This file should end up included only when no TR1 has been found and 
 * the user has requested CXXO_USE_INTERNAL_TR1.
 * 
 */
#include <cxxomfort/base.hpp>
#include <cxxomfort/functional.hpp> // for is_member_*_pointer
#include <cxxomfort/type_traits.hpp>

#if (CXXOMFORT_CXX_STD >= 2017)

namespace cxxomfort { namespace cxxostd {

template <typename R, typename Class>
struct memfn_cover_t {
	typedef R (Class::*PM);
	PM fn;

	explicit memfn_cover_t(PM pm)
	: fn(pm) 
	{ }
	
	template <typename... Args>
	auto operator() (Class&& c, Args&&... args) const 
	-> decltype((std::forward<Class>(c).*fn)(std::forward<Args>(args)...) )
	{
		return (std::forward<Class>(c).*fn)(std::forward<Args>(args)...);
	}

};


template <typename R, typename Class>
memfn_cover_t< R, Class> mem_fn (R (Class::*pm) ) 
{
	return memfn_cover_t< R, Class > (pm);
}



} } // cxxomfort

#endif // std

#endif
