#ifndef CXXOMFORT_IMPL_17_TYPE_TRAITS_HPP
#define CXXOMFORT_IMPL_17_TYPE_TRAITS_HPP

/*
 * */

#include <cxxomfort/base.hpp>

//#include "17-bool_constant.hpp" // deprecated
#include "17-void_t.hpp"

// 
// bool_constant 
// 
#if (defined(DOXYGEN_DOC))

namespace std {

/**
 * @class bool_constant 
 * @brief A <code>bool</code> compile time equivalent of @c integral_constant , as per C++17.
 * @ingroup std0typetraits
 * @cxx17b Implementation of @c bool_constant (bool specialization of @c integral_constant ).
 * @anchor std_bool_constant
 * 
 * <tt>bool_constant<false></tt> is semantically equialent to <tt>false_type</tt> and 
 * <tt>bool_constant<true></tt> is semantically equialent to <tt>true_type</tt>.
 * 
 * */
template <bool B>
struct bool_constant : public std::integral_constant<bool,B> 
{ };

}
#elif (defined(__cpp_lib_bool_constant))
    #undef  CXXOMFORT_IMPLEMENTS_n4334
    #define CXXOMFORT_IMPLEMENTS_n4334 CXXO_IMPLSTATUS_NATIVE()

#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1900))
    #undef  CXXOMFORT_IMPLEMENTS_n4334
    #define CXXOMFORT_IMPLEMENTS_n4334 CXXO_IMPLSTATUS_NATIVE()
    #define __cpp_lib_bool_constant (CXXOMFORT_CXX_STD*100)

#elif (CXXOMFORT_CXX_STD >= 2011)

namespace std {
    template <bool B>
    using bool_constant = integral_constant<bool,B>;
}

    #undef  CXXOMFORT_IMPLEMENTS_n4334
    #define CXXOMFORT_IMPLEMENTS_n4334 CXXO_IMPLSTATUS_BACKPORT()
    #define __cpp_lib_bool_constant (CXXOMFORT_CXX_STD*100)

#elif (CXXOMFORT_CXX_STD >= 1997)

namespace std {
    using cxxomfort::traits::bool_constant;
}

    #undef  CXXOMFORT_IMPLEMENTS_n4334
    #define CXXOMFORT_IMPLEMENTS_n4334 CXXO_IMPLSTATUS_EMULATION()
    #define __cpp_lib_bool_constant (CXXOMFORT_CXX_STD*100)

#else
    // nothing to do
    #define CXXOMFORT_IMPLEMENTS_n4334 0 
#endif

#undef  CXXOMFORT_IMPLEMENTS_bool_constant
#define CXXOMFORT_IMPLEMENTS_bool_constant CXXOMFORT_IMPLEMENTS_n4334

#if (defined(CXXOMFORT_NOTICES))
    #if (CXXOMFORT_NOTICES > 2)
    #pragma message CXXO_NOTICE("enabled boolean_constant implementation")
    #endif
#endif


// 
// negation, conjunction, disjunction 
// 

#if defined(DOXYGEN_DOC)
namespace std {

/**
 * @brief Negation metatemplate
 * @cxx17b Negation metatemplate (logical negation of a type trait).
 * @xrefitem std0type_traits "" "" Negation metatemplate (logical negation of a type trait).
 * */
template<class B>
struct negation : std::bool_constant<!bool(B::value)> { };

/**
 * @brief Conjunction metatemplate
 * @cxx17b Conjunction metatemplate (logical AND of a list of traits).
 * @xrefitem std0type_traits "" "" onjunction metatemplate (logical AND of a list of traits).
 * 
 * Inherits from the first @c Ty whose @c ::value evaluates to @c false , if any; else from the last @c Ty .
 * 
 * @note @c conjunction is provided only for C++11 onwards.
 * 
 * @see @cppref{types/conjunction}
 * */
template<class... Ty>
struct conjunction : std::bool_constant<bool(Ty...::value)> { };

/**
 * @brief Disjunction metatemplate
 * @cxx17b Disjunction metatemplate (logical OR of a list of traits).
 * @xrefitem std0type_traits "" "" Disjunction metatemplate (logical OR of a list of traits).
 * 
 * Inherits from the first @c Ty whose @c ::value evaluates to @c true , if any; else from the last @c Ty .
 * 
 * @note @c disjunction is provided only for C++11 onwards.
 * 
 * @see @cppref{types/disjunction}
 * */
template<class... Ty>
struct disjunction : std::bool_constant<bool(Ty...::value)> { };

}

#elif defined(__cpp_lib_logical_traits)
#elif (CXXOMFORT_CXX_STD >= 2011)
	#define __cpp_lib_logical_traits 201103

namespace cxxomfort { namespace cxxostd {

template<class B>
struct negation : std::bool_constant<!bool(B::value)> { };

template<class...> struct conjunction : std::true_type { };
template<class B1> struct conjunction<B1> : B1 { };
template<class B1, class... Bn>
struct conjunction<B1, Bn...> 
    : std::conditional<bool(B1::value), conjunction<Bn...>, B1>::type {};

template<class...> struct disjunction : std::false_type { };
template<class B1> struct disjunction<B1> : B1 { };
template<class B1, class... Bn>
struct disjunction<B1, Bn...> 
    : std::conditional<bool(B1::value), B1, disjunction<Bn...> >::type  { };
} }

namespace std {
	using cxxomfort::cxxostd::negation;
	using cxxomfort::cxxostd::conjunction;
	using cxxomfort::cxxostd::disjunction;
}


#else

namespace cxxomfort { namespace cxxostd {

template<class B>
struct negation 
: std::bool_constant<!bool(B::value)> { };


} }

#endif


    
// 
// has_unique_object_representations
// (only when it uses an intrinsic)
// 

#if (CXXOMFORT_CXX_STD>=2017 || CXXOMFORT_CXX_EMULATION>=2017)

#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=500) && CXXOMFORT_CXX_STD>=2011)

namespace std {
	template <typename T>
	struct has_unique_object_representations 
	: integral_constant<bool, __has_unique_object_representations(T)> 
	{ };

}

#endif

#endif
