#ifndef CXXOMFORT_IMPL_VOID_T_HPP
#define CXXOMFORT_IMPL_VOID_T_HPP
/**
 * @file 
 * @brief Implementation of std::make_void , std::void_t 
 * 
 * @sa http://en.cppreference.com/w/cpp/types/void_t
 * @sa n3911
 * 
 */

#include <cxxomfort/config.hpp>

#if defined(DOXYGEN_DOC)

namespace cxxomfort { namespace cxxostd {

/**
 * @struct make_void
 * @brief Metaprogramming helper that exposes @c void  as a type, for transformations.
 * @ingroup std0typetraits
 * @cxx17b Constructs @c void from an instantiation.
 */
template <typename... Arglist>
struct make_void { 
    typedef void type;  //< exports @c void as @a type .
};


/**
 * @struct void_t
 * @brief Generalized void type.
 * @ingroup std0typetraits
 * @cxx17b void type 
 * */
template <typename... Arglist>
using void_t = void;

} }

#elif defined(__cpp_lib_void_t)
	#define CXXOMFORT_IMPLEMENTS_n3911 CXXO_IMPLSTATUS_NATIVE()

#elif (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )

#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1900))
	#define CXXOMFORT_IMPLEMENTS_n3911 CXXO_IMPLSTATUS_NATIVE()
	#define __cpp_lib_void_t (CXXOMFORT_CXX_STD*100)

#elif (CXXOMFORT_CXX_STD >= 2011  ||  (CXXO_COMPILER_SUPPORT_variadic>0) )
	#define CXXOMFORT_IMPLEMENTS_n3911 CXXO_IMPLSTATUS_BACKPORT()
	#define __cpp_lib_void_t (CXXOMFORT_CXX_STD*100)

namespace std {

template <typename... Arglist>
struct make_void { 
    typedef void type;  //< exports @c void as @a type .
};

template <typename... Arglist>
using void_t = void;

}
#else
	#define CXXOMFORT_IMPLEMENTS_n3911 CXXO_IMPLSTATUS_EMULATION()
	#define __cpp_lib_void_t 1

namespace std {

template <typename T0=void, typename T1=void, typename T2=void, typename T3=void, typename T4=void, typename T5=void, typename T6=void, typename T7=void, typename T8=void, typename T9=void>
struct make_void {
	typedef void type;
};

}

#endif

#define CXXOMFORT_IMPLEMENTS_make_void CXXOMFORT_IMPLEMENTS_n3911

#endif
