#ifndef CXXOMFORT_IMPL_20_SHIFT_ALGORITHMS_HPP
#define CXXOMFORT_IMPL_20_SHIFT_ALGORITHMS_HPP
/**
 * @file
 * @brief Implementation of shift_left, shift_right
 */
#include <cxxomfort/config.hpp>
#include <cxxomfort/base/iterator.hpp>
#include <cxxomfort/algorithm.hpp>


#if (defined(__cpp_lib_shift))
	#define CXXOMFORT_IMPLEMENTS_p0769 CXXO_IMPLSTATUS_NATIVE()
#else
	#if (CXXO_COMPILER_SUPPORT_rvref)
		#define CXXOMFORT_IMPLEMENTS_p0769 CXXO_IMPLSTATUS_BACKPORT()
	#else
		#define CXXOMFORT_IMPLEMENTS_p0769 CXXO_IMPLSTATUS_EMULATION()
	#endif

namespace cxxomfort { namespace cxxostd {

namespace detail {

template< class FwIt >
FwIt shift_left ( FwIt ini, FwIt fin
, typename std::iterator_traits<FwIt>::difference_type n
, std::forward_iterator_tag ) {
	using namespace std;
	typedef typename std::iterator_traits<FwIt>::difference_type dt;
	dt dist = distance(ini, fin);
	if (n < dt(0)) return fin;
	if (n >= dist) return ini;
	FwIt xini = next(ini, n);
	for (; xini != fin; ++ini, ++xini) {
#if (CXXO_COMPILER_SUPPORT_rvref)
		*ini= std::move(*xini);
#else
		*ini= *xini;
#endif
	}
	return ini;
}

template< class FwIt >
FwIt shift_right ( FwIt ini, FwIt fin
, typename std::iterator_traits<FwIt>::difference_type n
, std::forward_iterator_tag ) {
	using namespace std;
	typedef typename std::iterator_traits<FwIt>::difference_type dt;
	dt dist = distance(ini, fin);
	if (n < dt(0)) return ini;
	if (n >= dist) return fin;

	dt k = 0;
	FwIt walk= ini;
	// iterativamente caminamos la secuencia hasta que la distancia al fin sea n + k + 1
	do {
		walk= ini;
		while (distance(walk, fin) > n + k + 1) { ++walk; }
#if (CXXO_COMPILER_SUPPORT_rvref)
		*next(walk,n)= std::move(*walk);
#else
		*next(walk,n)= *walk;
#endif
		++k;
	} while (walk != ini);
	// a este punto sólo necesitamos mover *ini
	return ini;
}

template< class BIt >
BIt shift_right ( BIt ini, BIt fin
, typename std::iterator_traits<BIt>::difference_type n
, std::bidirectional_iterator_tag ) {
	using namespace std;
	typedef typename std::iterator_traits<BIt>::difference_type dt;
	dt dist = distance(ini, fin);
	if (n < dt(0)) return ini;
	if (n >= dist) return fin;
	BIt xfin = prev(fin, n);
	for (; xfin != ini; --xfin, --fin) {
#if ((CXXO_COMPILER_SUPPORT_rvref))
		*prev(fin)= std::move(*prev(xfin));
#else
		*prev(fin)= *prev(xfin);
#endif
	}
	return ini;
}


} // detail

template< class FwIt >
FwIt shift_left ( FwIt ini, FwIt fin
, typename std::iterator_traits<FwIt>::difference_type n ) {
	typedef typename std::iterator_traits<FwIt>::iterator_category C;
	return detail::shift_left(ini,fin,n,C() );
}

template< class FwIt >
FwIt shift_right ( FwIt ini, FwIt fin
, typename std::iterator_traits<FwIt>::difference_type n ) {
	typedef typename std::iterator_traits<FwIt>::iterator_category C;
	return detail::shift_right(ini,fin,n,C() );
}
                                
} } 

#endif

#undef  CXXOMFORT_IMPLEMENTS_algorithm_shift
#define CXXOMFORT_IMPLEMENTS_algorithm_shift CXXOMFORT_IMPLEMENTS_p0769

#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#elif (CXXOMFORT_IMPLEMENTS_algorithm_shift>= CXXO_IMPLSTATUS_EMULATION() && CXXOMFORT_IMPLEMENTS_algorithm_shift<CXXO_IMPLSTATUS_NATIVE() )
	#define __cpp_lib_shift (CXXOMFORT_CXX_STD*100)
namespace std {
	using ::cxxomfort::cxxostd::shift_left;
	using ::cxxomfort::cxxostd::shift_right;
}
#endif


#endif

