#ifndef CXXOMFORT_IMPL_23_CONTAINS_HPP
#define CXXOMFORT_IMPL_23_CONTAINS_HPP
/**
 * @file
 * @brief Implementation of contains from C++23.
 */
#include <cxxomfort/base.hpp>
#include <cxxomfort/type_traits.hpp>
#include <cxxomfort/string_view.hpp>

namespace cxxomfort { namespace text {

#if (CXXOMFORT_CXX_STD >= 2020)
	#undef  CXXO_COMPILER_SUPPORT_p1679
	#define CXXO_COMPILER_SUPPORT_p1679 (CXXOMFORT_CXX_STD >= 2020)

template <typename Ch, typename CT>
constexpr bool 
contains (std::basic_string_view<Ch,CT> const str, std::basic_string_view<Ch,CT> const a) noexcept {
	return str.contains(a);
}

template <typename Ch, typename CT>
constexpr bool 
contains (std::basic_string_view<Ch,CT> const str, Ch const a) noexcept {
	return str.contains(a);
}

template <typename Ch, typename CT>
constexpr bool 
contains (std::basic_string_view<Ch,CT> const str, Ch const* a) noexcept {
	return str.contains(a);
}


#elif (CXXOMFORT_CXX_STD >= 2011)

template <typename Ch, typename CT>
constexpr bool 
contains (std::basic_string_view<Ch,CT> const str, std::basic_string_view<Ch,CT> const a) noexcept {
	return str.find(a) != std::basic_string_view<Ch,CT>::npos;
}

template <typename Ch, typename CT>
constexpr bool 
contains (std::basic_string_view<Ch,CT> const str, Ch const a) noexcept {
	return str.find(a) != std::basic_string_view<Ch,CT>::npos;
}

#else

template <typename Ch, typename CT>
bool 
contains (std::basic_string_view<Ch,CT> const str, std::basic_string_view<Ch,CT> const a) {
	return str.find(a) != std::basic_string_view<Ch,CT>::npos;
}

template <typename Ch, typename CT>
bool 
contains (std::basic_string_view<Ch,CT> const str, Ch const a) {
	return str.find(a) != std::basic_string_view<Ch,CT>::npos;
}

#endif

} } // cxxomfort::text

#endif
