#ifndef CXXOMFORT_IMPL_ERRC_HPP
#define CXXOMFORT_IMPL_ERRC_HPP
//! @file 
#include <cxxomfort/config.hpp>
#include <cxxomfort/cerrno.hpp>

#define CXXOMFORT_IMPLEMENTS_posix2008_errc 0

#if defined(DOXYGEN_DOC)
namespace std {
	//! Enumeration that holds the system error codes, derived from POSIX 2008.
	//! @see @cppref{error/errc}
	//! @see @ref cxxomfort::siesta::errc 
	enum class errc : int {
		//! @include "../impl/errc-list.hpp"
		#include "../impl/errc-list.hpp"
	};
}
#elif (CXXOMFORT_CXX_STD>=1997)

	//#include <cxxomfort/utils/typesafe_enum.hpp>

	namespace std {

	//
	// errc and helpers
	//
	CXXO_OPEN_TYPESAFE_ENUM(errc, int){
	#include "../impl/errc-list.hpp"
	} CXXO_CLOSE_TYPESAFE_ENUM(errc, int);
	
	static_assert(sizeof(errc) == sizeof(int), "error in definition of errc");


#if defined(_POSIX_VERSION) && _POSIX_VERSION>=200800UL
	#undef CXXOMFORT_IMPLEMENTS_posix2008_errc
	#define CXXOMFORT_IMPLEMENTS_posix2008_errc CXXO_IMPLSTATUS_NATIVE()	
#else
	//#define CXXOMFORT_IMPLEMENTS_posix2008_errc 0
#endif


	template <typename T> struct is_error_code_enum 
	: public std::false_type {};

	template<> struct is_error_code_enum <errc> : public std::true_type {};
	// Here we need to specialize for myerrc::type as well
	// because the constructors for error_condition and error_code below 
	// use SFINAE to match the exact type, before conversion.
	//template<> struct is_error_code_enum<myerrc::type> : public std::true_type {};


	} // namespace

#else
    // for systems that don't really advertise c++ compatibility
    namespace cxxomfort { namespace cxxostd {
        enum errc {};
    } }
#endif // CXX_STD

#endif
