#ifndef CXXOMFORT_IMPL_EXTERNAL_ALLOCATOR_HPP
#define CXXOMFORT_IMPL_EXTERNAL_ALLOCATOR_HPP
/**
 * @file 
 */

#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>
#include <cxxomfort/impl/functionmeta.hpp>

namespace cxxomfort { namespace impl {

template <typename T, (T*)(*Malloc)(size_t n), (void)(*Free)(T*)>
struct external_allocator {
	typedef T type;
	typedef T* pointer;
	static const cxxomfort::impl::function_caller< T*(size_t), Malloc> _Malloc;
	static const cxxomfort::impl::function_caller< void(T*), Free> _Free;
	static size_t& 	count () CXXO_NOEXCEPT {
		static size_t c= 0;
		return c;
	}
	static cxxomfort::siesta::generic_code& 	status () CXXO_NOEXCEPT {
		static cxxomfort::siesta::generic_code e= 0;
		return e;
	}
	// operations
	pointer 	allocate (size_t n) {
		pointer p= _Malloc(n);
		if (p) { count()++; status()= cxxomfort::message_codes::success; }
		else { p= nullptr; status()= cxxomfort::message_codes::allocator_error; }
		return p;
	}
	void 	deallocate (pointer p) {
		_Free(p); count()--;
	}
	// stateless
	CXXO_CONSTEXPR bool 	operator== (external_allocator const& rhs) const CXXO_NOEXCEPT { return true; }
	CXXO_CONSTEXPR bool 	operator!= (external_allocator const& rhs) const CXXO_NOEXCEPT { return false; }
};

}}
#endif

