// A stripped down version of FOREACH for
// illustration purposes. NOT FOR GENERAL USE.
// For a complete implementation, see BOOST_FOREACH at
// http://boost-sandbox.sourceforge.net/vault/index.php?directory=eric_niebler
//
// Copyright 2004 Eric Niebler.
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//#include <functional>
#include <cxxomfort/base.hpp>
#include <cxxomfort/base/iterator.hpp> // global begin, end
#include <valarray>
#include <cxxomfort/impl/lift_type_traits.hpp>

#if (CXXOMFORT_CXX_STD>=1997)

namespace cxxomfort {
/**
 * @internal
 * @{
 * */

namespace impl_ifa {

///////////////////////////////////////////////////////////////////////////////
// auto_any

struct auto_any_base {
    operator bool() const { return false; }
    //virtual ~auto_any_base() {}
};


template<typename T> struct auto_any : public auto_any_base {
    auto_any(T const& t) : item(t) {}
    mutable T item;
};

template<typename T> T& auto_any_cast(auto_any_base const& any) {
    return static_cast<auto_any<T> const&>(any).item;
}

///////////////////////////////////////////////////////////////////////////////
// type2type

// a simple type wrapper
template< typename T > struct type2type {};

// convert an expression of type T to an
// expression of type type2type<T>
template< class T > type2type< T > encode_type( T const & ) {
  return type2type< T >();
}

// convertible to type2type<T> for any T
struct any_type {
    template< typename T > 
    operator type2type< T > () const {
        return type2type< T >();
    }
};

// convert an expression of type T to an
// expression of type type2type<T> without
// evaluating the expression
#define ENCODED_TYPEOF( container ) \
  ( true ? ::cxxomfort::impl_ifa::any_type() : ::cxxomfort::impl_ifa::encode_type( container ) )


///////////////////////////////////////////////////////////////////////////////
// FOREACH helper function

template <typename T>
struct type2iterator {
    typedef T container_type;
    typedef type2iterator<T> type;
    typedef typename T::value_type value_type;
    typedef typename ::cxxomfort::traits::conditional< ::cxxomfort::traits::is_const<T>::value , typename T::const_iterator, typename T::iterator>::type iterator;
    typedef typename std::iterator_traits<iterator>::reference reference;
    typedef typename std::iterator_traits<iterator>::pointer pointer;
};

template <> struct type2iterator <char const*> {
    typedef char const* container_type;
    typedef type2iterator<char const*> type;
    typedef char value_type;
    typedef char const* iterator;
    typedef char const& reference;
    typedef char const* pointer;
};

template <typename T, size_t N> struct type2iterator <T [N]> {
    typedef T container_type[N];
    typedef type2iterator<T (&)[N]> type;
    typedef T value_type;
    typedef T * iterator;
    typedef T& reference;
    typedef T* pointer;

};

template <typename T> struct type2iterator <std::valarray<T> > {
    typedef std::valarray<T> container_type;
    typedef T value_type;
    typedef T* iterator;
    typedef T& reference;
    typedef T* pointer;
};


template<typename T>
auto_any<typename type2iterator<T>::iterator> begin(T& t) {
    return ::cxxomfort::iterator::begin(t);
}


template<typename T>
auto_any<typename type2iterator<T>::iterator> end(T& t) {
    return ::cxxomfort::iterator::end(t);
}

template<typename T>
bool done(auto_any_base const& cur, auto_any_base const& end, type2type<T>) {
    typedef typename type2iterator<T>::iterator iter_type;
    return auto_any_cast<iter_type>(cur) == auto_any_cast<iter_type>(end);
}

template<typename T>
void next(auto_any_base const& cur, type2type<T>) {
    typedef typename type2iterator<T>::iterator iter_type;
    ++auto_any_cast<iter_type>(cur);
}


template<typename T>
typename type2iterator<T>::reference deref(auto_any_base const& cur, type2type<T>) {
    typedef typename type2iterator<T>::iterator iter_type;
    return *auto_any_cast<iter_type>(cur);
}


} // impl_ifa
/**
 * @}
 * */
} // cxxomfort

///////////////////////////////////////////////////////////////////////////////
// FOREACH


#if defined(_MSC_VER)
// use a modified version with pragmas to suppress
// "assignment within conditional" warnings
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("foreach MSVC with suppressing pragmas")
	#endif
	#define CXXO_FOREACH(item, container)                  \
__pragma(warning(suppress:4706)) __pragma(warning(suppress:6001)) \
if(::cxxomfort::impl_ifa::auto_any_base const& bX = ::cxxomfort::impl_ifa::begin(container)) {} else   \
if(::cxxomfort::impl_ifa::auto_any_base const& eX = ::cxxomfort::impl_ifa::end(container)) {} else     \
for(bool more = true;                                   \
    more && !::cxxomfort::impl_ifa::done(bX,eX,ENCODED_TYPEOF(container));                       \
    more ? ::cxxomfort::impl_ifa::next(bX,ENCODED_TYPEOF(container)) : (void)0)                 \
    __pragma(warning(suppress:4706))                       \
    if ((more = false)!=0) {} else                           \
    for(item = ::cxxomfort::impl_ifa::deref(bX,ENCODED_TYPEOF(container)); !more; more = true)

#else
// all other compilers
	#define CXXO_FOREACH(item, container)                  \
if(::cxxomfort::impl_ifa::auto_any_base const& bX = ::cxxomfort::impl_ifa::begin(container)) {} else   \
if(::cxxomfort::impl_ifa::auto_any_base const& eX = ::cxxomfort::impl_ifa::end(container)) {} else     \
for(bool more = true;                                   \
    more && !::cxxomfort::impl_ifa::done(bX,eX,ENCODED_TYPEOF(container));                       \
    more ? ::cxxomfort::impl_ifa::next(bX,ENCODED_TYPEOF(container)) : (void)0)                 \
    if ((more = false)!=0) {} else                           \
    for(item = ::cxxomfort::impl_ifa::deref(bX,ENCODED_TYPEOF(container)); !more; more = true)

#endif // foreach macro definition


#define CXXO_FOREACH_AUTO(item, container)     \
void f_foreach_no_typeof_deduction (void* item) 

#undef Suppress_

//////////////////

#else // pre-c++97

	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("foreach emulation for pre-C++97")
	#endif

namespace cxxomfort { namespace impl_ifa {

// auto_any

struct auto_any_base {
    operator bool() const { return false; }
};

template<typename T>
struct auto_any : auto_any_base {
    auto_any(T const& t) : item(t) {}
    mutable T item;
};

template<typename T>
T& auto_any_cast (auto_any_base const& any) throw() {
    return static_cast<auto_any<T> const&>(any).item;
}

// FOREACH helper function

template<typename T>
auto_any<typename T::const_iterator> begin (T const& t) {
    return t.begin();
}

template<typename T>
auto_any<typename T::iterator> begin (T& t) {
    return t.begin();
}

template<typename T>
auto_any<typename T::const_iterator> end(T const& t) {
    return t.end();
}

template<typename T>
auto_any<typename T::iterator> end(T& t) {
    return t.end();
}

template<typename T>
bool done(auto_any_base const& cur, auto_any_base const& end, T const&) {
    typedef typename T::const_iterator iter_type;
    return auto_any_cast<iter_type>(cur) == auto_any_cast<iter_type>(end);
}

template<typename T>
void next(auto_any_base const& cur, T const&) {
    typedef typename T::const_iterator iter_type;
    ++auto_any_cast<iter_type>(cur);
}

template<typename T>
typename T::const_reference deref(auto_any_base const& cur, T const&) {
    typedef typename T::const_iterator iter_type;
    return *auto_any_cast<iter_type>(cur);
}

// FOREACH

#define CXXO_FOREACH(item, container)                            \
    if(::cxxomfort::impl_ifa::auto_any_base const& b = ::cxxomfort::impl_ifa::begin(container)) {} else   \
    if(::cxxomfort::impl_ifa::auto_any_base const& e = ::cxxomfort::impl_ifa::end(container)) {} else     \
    for(bool more = true;                                   \
        more && !::cxxomfort::impl_ifa::done(b,e,container);                       \
        more ? ::cxxomfort::impl_ifa::next(b,container) : (void)0)                 \
        if (more = false) {} else                           \
        for(item = ::cxxomfort::impl_ifa::deref(b,container); !more; more = true)

// namespace
} }

#endif

