#ifndef CXXOMFORT_IMPL_FUNCTIONAL_OVERLOAD_HPP
#define CXXOMFORT_IMPL_FUNCTIONAL_OVERLOAD_HPP
/**
 * @file
 * @brief Implementation of the "overload set" idiom 
 * 
 * @see http://martinecker.com/martincodes/lambda-expression-overloading/
 * 
 */

#if (CXXOMFORT_CXX_STD >= 2011)
	#include <functional>

namespace cxxomfort { namespace functional_impl {

template <class... Fs> struct overload_set;

template <class F1, class... Fs>
struct overload_set<F1, Fs...> : 
F1, overload_set<Fs...>::type
{
	typedef overload_set type;

	overload_set(F1 head, Fs... tail)
		: F1(head), overload_set<Fs...>::type(tail...)
	{ }

	using F1::operator();
	using overload_set<Fs...>::type::operator();
};

template <class F>
struct overload_set<F> 
: F
{
	typedef F type;
	using F::operator();
};

template <class... Fs>
typename overload_set<Fs...>::type overload(Fs... x) {
	return overload_set<Fs...>(x...);
}

} }

#elif (CXXOMFORT_CXX_STD>=1997)

namespace cxxomfort { namespace functional_impl {

struct nofunct_t {
	nofunct_t () {}
	void operator() (...) {}
};

template <typename F1, typename F2=nofunct_t, typename F3=nofunct_t, typename F4=nofunct_t, typename F5=nofunct_t> 
struct overload_set
: F1, overload_set<F2,F3,F4,F5>::type
{
	typedef overload_set type;
	
	overload_set (F1 f1, F2 f2, F3 f3, F4 f4, F5 f5 )
	: F1(f1), overload_set<F2,F3,F4,F5>::type(f2,f3,f4,f5) 
	{ }

	overload_set (F1 f1, F2 f2, F3 f3, F4 f4 )
	: F1(f1), overload_set<F2,F3,F4>::type(f2,f3,f4) 
	{ }

	overload_set (F1 f1, F2 f2, F3 f3 )
	: F1(f1), overload_set<F2,F3>::type(f2,f3) 
	{ }

	overload_set (F1 f1, F2 f2 )
	: F1(f1), overload_set<F2>::type(f2) 
	{ }
	
	using F1::operator();
	using overload_set<F2,F3,F4,F5>::type::operator();
};

template <typename F1>
struct overload_set<F1> 
: F1 {
	typedef F1 type;
	using F1::operator();
};


template <typename F1>
typename overload_set<F1>::type overload(F1 f1) {
	return overload_set<F1>(f1);
}

template <typename F1, typename F2>
typename overload_set<F1,F2>::type overload(F1 f1, F2 f2) {
	return overload_set<F1,F2>(f1,f2);
}

template <typename F1, typename F2, typename F3>
typename overload_set<F1,F2,F3>::type overload(F1 f1, F2 f2, F3 f3) {
	return overload_set<F1,F2,F3>(f1,f2,f3);
}


} }

#endif 

#endif
