#ifndef CXXOMFORT_IMPL_HASH_HPP
#define CXXOMFORT_IMPL_HASH_HPP

#include <cxxomfort/config.hpp>
#include <functional>
#include <cxxomfort/functional.hpp>
#include <cxxomfort/type_traits.hpp>
#include <cxxomfort/impl/07-type_traits.hpp> // is_enum
#include <cxxomfort/impl/11-underlying_type.hpp>

/**
 * @file
 * @brief cxxomfort "fix" helper for <tt>std::hash</tt>.
 * 
 * */

#if (CXXOMFORT_CXX_STD >= 1997)

namespace cxxomfort {  namespace fix {

/**
 * @struct hash
 * @brief Specialization-friendly alternatve to @c std::hash .
 * 
 * A version of @c std::hash that allows for better specialization. 
 * When not specialized, it just delegates to the already available 
 * specialization of @c std::hash for the given template parameters.
 * 
 * @ingroup cxx14-backports
 * @xrefitem std0functional "<functional>" "Non-backport Assists"
 * @see 
 * * Documentation for C++ @c std::hash .
 * 
 */
template <typename T, typename = void >
struct hash
: public std::hash<T> {
    enum { uses_std = true };
};

/**
 * @brief Specialization of @c std::hash for @c enum types, as per C++ 14.
 * @note Since pre-C++17 @c hash is not specializable generically, this 
 * is implemented through another facility ( cxxomfort::fix::hash ).
 * 
 * */
template <typename E>
struct hash<E , typename std::enable_if< std::is_enum<E>::value>::type >
: public hash< typename std::underlying_type<E>::type> {
    enum { uses_std = false };
};


} }

#endif

#endif

