#ifndef CXXOMFORT_IMPL_HEXDUMP_HPP
#define CXXOMFORT_IMPL_HEXDUMP_HPP

namespace cxxomfort { namespace library {

namespace utility {

/**
 * @brief Dumps hex string representation of an area of memory.
 * @ingroup cxxo-sup-utility
 * @overload hexdump
 * @param dest An <tt>array_ref<char></tt> pointing to a writable area of memory of the specified size.
 * @param buffer The data buffer to read, as an <tt>array_ref<byte></tt>.
 * 
 * @returns An integer with the following possible values: <br/>
 * * <tt>==length</tt> if all characters were dumped without issues. <br/>
 * * <tt>0</tt> if there is no data to read, or if <tt>dest</tt> does not hold enough room to write the representation. <br/>
 * * <tt><0</tt> if there was an error when writing the data to @a dest .
 * 
 * @throws Only throws if <tt>sprintf()</tt> were somehow to throw.
 * 
 * */
inline 
int hexdump (::cxxomfort::array_ref<char> dest, ::cxxomfort::array_ref<char const> buffer) {
	using namespace std;
	//error_code ec;
	if (buffer.size() < 1 || dest.size() * 2 < buffer.size()) {
		return 0;
	}
	char const * const buff = ( reinterpret_cast<char const * const>(buffer.data()) );
	size_t i= 0;
	for (i= 0; i < buffer.size(); ++i) {
		int sp = sprintf(dest.data()+2*i, "%hhx", buff[i]); 
		if (sp < 2) {
			return -static_cast<int>(i) -1;
		}
	}
	return i;
	//return ec;
}


#if 0
/**
 * @brief Dumps hex string representation of an area of memory.
 * @ingroup cxxo-sup-utility
 * @param dest An <tt>array_ref<unsigned char></tt> pointing to a writable area of memory of the specified size.
 * @param ptr A pointer to an area of memory to read.
 * @param length Length of the area of memory to read.
 * */
inline 
int hexdump (cxxomfort::array_ref<char> dest, void const * const ptr, size_t length) {
	unsigned char const* b = (unsigned char const*)(ptr);
	return hexdump(dest, cxxomfort::library::carrayref(b, length) );
}


inline
int hexload (bytes_view dest, std::string_view src) {
	int res= 0;
	(void)(src);
	(void)(dest);
	return res;
}
#endif

} 
} } // cxxomfort::library 
#endif  // guard
