#ifndef CXXOMFORT_IMPL_IC_TYPES_HPP
#define CXXOMFORT_IMPL_IC_TYPES_HPP

#if (CXXOMFORT_CXX_STD >= 1997)

/**
 * @file
 * @brief Implements shortcut types for @c std::integral_constant
 * @sa bool_constant
 *
 * Interfaces defined in this header:
 *
 * * iC - integral_constant of @c int type
 * * uC - integral_constant of @c unsigned @c int type
 * * huC - integral_constant of @c unsigned @c short type
 * * luC - integral_constant of @c unsigned @c long type
 * * juC - integral_constant of @c uintmax_t type
 * * zuC - integral_constant of @c size_t type
 * * integral_constant_cast
 *
 * The library makes available, as supplements to @c integral_constant ,
 * a series of shortcut names for specific instantiations, using
 * specific numeric types. These instantiations follow the naming scheme
 * of the @c printf family format flags (eg.: "hu" for short unsigned).
 *
 * @code

template <typename T> foo ();

namespace TT = cxxomfort::library::type_traits;

foo ( TT::juC<5>() ); // calls foo<std::integral_constant<uintmax_t,5> >

 * @endcode
 *
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/cstdint.hpp>
#include <cxxomfort/impl/11-type_traits.hpp> // std::integral_constant

namespace cxxomfort { namespace impl { namespace ic_types {

template <int I> CXXO_CONSTEXPR 
std::integral_constant<int,I> iC () CXXO_NOEXCEPT {
	return std::integral_constant<int,I>();
}
template <unsigned int I> CXXO_CONSTEXPR 
std::integral_constant<unsigned int,I> uC () CXXO_NOEXCEPT {
	return std::integral_constant<unsigned int,I>();
}
template <int16_t I> CXXO_CONSTEXPR 
std::integral_constant<int16_t,I> hiC () CXXO_NOEXCEPT {
	return std::integral_constant<int16_t,I>();
}
template <uint16_t I> CXXO_CONSTEXPR 
std::integral_constant<uint16_t,I> huC () CXXO_NOEXCEPT {
	return std::integral_constant<uint16_t,I>();
}
template <size_t I> CXXO_CONSTEXPR 
std::integral_constant<size_t,I> zuC () CXXO_NOEXCEPT {
	return std::integral_constant<size_t,I>();
}
template <intmax_t I> CXXO_CONSTEXPR 
std::integral_constant<intmax_t,I> jiC () CXXO_NOEXCEPT {
	return std::integral_constant<intmax_t,I>();
}
template <uintmax_t I> CXXO_CONSTEXPR 
std::integral_constant<uintmax_t,I> juC () CXXO_NOEXCEPT {
	return std::integral_constant<uintmax_t,I>();
}


/*
 * integral constant<unsigned short> ( integral_constant<int,8>() ) --> integral_constant<unsigned short,8>()
 * */
template <typename To, typename I, I X>
std::integral_constant<To,static_cast<To>(X)>
integral_constant_cast (std::integral_constant<I,X> const&) {
    return std::integral_constant<To,static_cast<To>(X)>();
}

} // ic_types
} }

#endif


#endif  // guard
