#ifndef CXXOMFORT_IMPL_LIFT_type_traits_HPP
#define CXXOMFORT_IMPL_LIFT_type_traits_HPP

/*
 * This header brings whichever version is available of the basic type_traits into namespace std if that header implements the set of the basic type_traits (the ones that deal with fundamental types):
 * 
 * * For C++11 onward, bring in <type_traits>
 * * For C++03 with TR1 support, bring in <type_traits> or <tr1/type_traits>
 * * For compilers for which the support macro is set, bring in <type_traits>
 */

#include <cxxomfort/base.hpp>

#undef  CXXOMFORT_LIFTED_type_traits

#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
	#pragma message CXXO_NOTICE("lifting type_traits from the compiler to std::")
#endif

#if (CXXOMFORT_CXX_STD >= 2011 && CXXO_CXXSTDHEADER_type_traits)
    #include <type_traits>
    // nothing else needs to be done
    #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_NATIVE()
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
		#pragma message CXXO_NOTICE("type_traits from C++11")
	#endif
#elif (CXXO_CXXSTDHEADER_type_traits)
    #include <type_traits>
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1600))
        #include <cxxomfort/impl/using_tr1_type_traits.hpp>
    #endif
    // nothing else needs to be done
    #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_NATIVE()
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
		#pragma message CXXO_NOTICE("native type_traits")
	#endif
#elif (CXXO_CXXSTDHEADER_tr1_type_traits)
    #include <tr1/type_traits>
    #include <cxxomfort/impl/using_tr1_type_traits.hpp>
    // nothing else needs to be done
    #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_NATIVE()
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
		#pragma message CXXO_NOTICE("native tr1/type_traits")
	#endif
#elif (CXXOMFORT_CXX_STD>=1997 && (CXXO_COMPILER_SUPPORT_tr1_type_traits|| CXXO_CXXSTDHEADER_type_traits) )

    #if (CXXOFLAG_directory_tr1)
        #include <tr1/type_traits>
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("tr1/type_traits from C++03/TR1 tr1/")
		#endif
    #else
        #include <type_traits>
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("type_traits from C++03/TR1")
		#endif
    #endif

    #if (CXXOFLAG_namespace_tr1)
        #include "using_tr1_type_traits.hpp"
    #endif

    #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_OW)
    // OpenWatcom brings some elements of type_traits but they are incomplete
    #include <type_traits>
    #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_NATIVE()

#else

    // at this point we assume the client has provided a <type_traits> somewhere
	#if defined(CXXOFLAG_USE_INTERNAL_TR1)
		#if (CXXOMFORT_NOTICES > 1)
			#pragma message CXXO_NOTICE("lifting from cxxomfort TR1")
		#endif
		#include "./07-type_traits.hpp" // at least the basic traits
		#include "./using_cxxo_type_traits.hpp" // lift cxxomfort::traits to std
        #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_BACKPORT()
		
    #else
		#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
			#pragma message CXXO_NOTICE("requesting external type_traits")
		#endif
        #include <type_traits>
        #define CXXOMFORT_LIFTED_type_traits CXXO_IMPLSTATUS_EMULATION()
    #endif

#endif


#endif
