#ifndef CXXOMFORT_IMPL_n3668_HPP
#define CXXOMFORT_IMPL_n3668_HPP

#include "../config.hpp"
#include "../base/move.hpp" // move emulation
#include <utility>

/**
 * @file 
 * @brief assign new value and return old
 * @see N3608, N3668
 * 
 */

//#define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_BACKPORT()
//#define CXXOMFORT_IMPLEMENTS_exchange CXXOMFORT_IMPLEMENTS_n3668


#if defined(DOXYGEN_DOC)

namespace std {

/**
 * @brief Replaces a variable's value, returns the old value.
 * @ingroup std0utility
 * @cxx14b Assign a new value to a variable and return the old value.

 * 
 * @see @wg21{n3668}
 * 
 * The macro @c CXXOMFORT_IMPLEMENTS_exchange informs about the implementation status. Additionally, the library also sets @ref @c __cpp_lib_exchange_function from SD-6.
 *  
 */
template <typename T, typename U = T>
T exchange (T& obj, CXXO_RV_REF(U) nval);

}

    /**
     * @brief Feature macro set for @c exchange() .
     * @ingroup std00featuremacros
     * @cppfeature{\ref \c exchange() }
     * */
    #define __cpp_lib_exchange_function YYYYMML

#elif (defined(__cpp_lib_exchange_function))
    #define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_NATIVE()
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1900))
    #define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_NATIVE()
    #define __cpp_lib_exchange_function __cplusplus
#elif (CXXOMFORT_CXX_STD>=2011 && CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=500))
    #define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_NATIVE()
    #define __cpp_lib_exchange_function __cplusplus
#elif (CXXO_COMPILER_SUPPORT_rvref > 0 && CXXO_COMPILER_SUPPORT_variadic)
    #define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_BACKPORT()
    #define __cpp_lib_exchange_function (CXXOMFORT_CXX_STD*100)

namespace std {

template <typename T, typename U=T >
T exchange (T& obj, U&& nval) {
    T oval = std::move(obj);
    obj= std::forward<U>(nval);
    return oval;
}

}

#else
    #define CXXOMFORT_IMPLEMENTS_n3668 CXXO_IMPLSTATUS_EMULATION()
    #define __cpp_lib_exchange_function 199703L

namespace std {

template <typename T, typename U >
T exchange (T& obj, U const& nval) {
    T oval = obj;
    obj= nval;
    return oval;
}

template <typename T>
T exchange (T& obj, T const& nval) {
    T oval = obj;
    obj= (nval);
    return oval;
}

}


#endif

#define CXXOMFORT_IMPLEMENTS_exchange CXXOMFORT_IMPLEMENTS_n3668

#if (defined(CXXOFLAG_NO_STD_USING) && CXXOFLAG_NO_STD_USING>0 )
#elif (CXXOMFORT_IMPLEMENTS_exchange >= CXXO_IMPLSTATUS_NATIVE())
#elif (CXXOMFORT_IMPLEMENTS_exchange >= CXXO_IMPLSTATUS_EMULATION())
#endif // std using


#endif
