#ifndef CXXOMFORT_IMPL_n4502_HPP
#define CXXOMFORT_IMPL_n4502_HPP

#include "../config.hpp"
#include "../base.hpp" // move emulation
#include <cxxomfort/impl/07-type_traits.hpp> // decay
#include <cxxomfort/type_traits.hpp> // integral_constant, make_void
#include <cxxomfort/utility.hpp> // declval



// 
// detection idiom from 4502 - for C++11 onwards
// 

#if 0
// example usage
template<class T>
using copy_assign_t = decltype(std::declval<T&>() = std::declval<const T&>());
 
struct Meow { };
struct Purr { void operator=(const Purr&) = delete; };
 
static_assert(std::experimental::is_detected<copy_assign_t, Meow>::value,
              "Meow should be copy assignable!");
static_assert(!std::experimental::is_detected_v<copy_assign_t, Purr>,
              "Purr should not be copy assignable!");
static_assert(std::experimental::is_detected_exact_v<Meow&, copy_assign_t, Meow>,
              "Copy assignment of Meow should return Meow&!");
              
#endif
#if (CXXOMFORT_CXX_STD>=2011)
namespace cxxomfort { namespace impl {

namespace detail {

template <class Default, class Void, template<class...> class Op, class... Args>
struct detector {
	typedef std::false_type value_t;
	typedef Default type;
};

template <class Default, template<class...> class Op, class... Args>
struct detector<Default, typename std::make_void<Op<Args...> >::type, Op, Args...> {
  typedef std::true_type value_t;
  using type = Op<Args...>;
}; 

/* from original test
  template <template <class...> class Trait, class Enabler, class... Args>
  struct is_detected : std::false_type{};

  template <template <class...> class Trait, class... Args>
  struct is_detected<Trait, typename std::make_void<Trait<Args...> >::type, Args...> : std::true_type{};

*/
} // detail

struct nonesuch {
    ~nonesuch() = delete;
    nonesuch(nonesuch const&) = delete;
    void operator=(nonesuch const&) = delete;
};

template <template<class...> class Op, class... Args>
using is_detected = typename detail::detector<nonesuch, void, Op, Args...>::value_t;
 
template <template<class...> class Op, class... Args>
using detected_t = typename detail::detector<nonesuch, void, Op, Args...>::type;
 
template <class Default, template<class...> class Op, class... Args>
using detected_or = detail::detector<Default, void, Op, Args...>;

/* from original test 
template <template <class...> class Trait, class... Args>
using is_detected = typename detail::is_detected<Trait, void, Args...>::type;
*/

} } // cxxomfort::impl

	#if (defined(CXXOFLAG_NO_STD_USING) && (CXXOFLAG_NO_STD_USING>0))
	#else
namespace std { namespace experimental {
	using ::cxxomfort::impl::nonesuch;
	using ::cxxomfort::impl::is_detected;
	using ::cxxomfort::impl::detected_t;
	using ::cxxomfort::impl::detected_or;
}}
	#endif

#endif // cxx-std

#endif

