#ifndef CXXOMFORT_IMPL_p1223_HPP
#define CXXOMFORT_IMPL_p1223_HPP

/**
 * @file
 * @brief Implements part of p1223, a proposal for an algorithm to find an element backwards.
 * */

namespace cxxomfort { namespace impl {

namespace detail_algorithm {

template <typename It, typename T>
It find_backward (It ini, It fin, T const& t, std::bidirectional_iterator_tag) {
	if (ini == fin) return fin;
	while (fin-- != ini) {
		if (*fin == t) return fin;
	}
	return fin;
}

template <typename It, typename Pred>
It find_if_backward (It ini, It fin, Pred if_, std::bidirectional_iterator_tag) {
	if (ini == fin) return fin;
	while (fin-- != ini) {
		if (if_(*fin)) return fin;
	}
	return fin;
}


} // detail_algorithm

template <typename It, typename T>
It find_backward (It ini, It fin, T const& t) {
	typename std::iterator_traits<It>::iterator_category cat_It;
	return detail_algorithm::find_backward(ini,fin,t,cat_it);
}

template <typename It, typename Pred>
It find_if_backward (It ini, It fin, Pred if_) {
	typename std::iterator_traits<It>::iterator_category cat_It;
	return detail_algorithm::find_if_backward(ini,fin,if_,cat_it);
}



} }


#endif
