#ifndef CXXOMFORT_IMPL_p2187_HPP
#define CXXOMFORT_IMPL_p2187_HPP
/**
 * @file
 * @brief Implementation of swap_if proposal
 * 
 */

#undef  CXXOMFORT_IMPLEMENTS_p2187

#if (defined(DOXYGEN_DOC))
namespace cxxomfort { namespace impl {

/**
 * @brief Conditionally swaps elements avoiding a branch if possible.
 * @see @wg21{p2187}
 * @return The condition @a c passed.
 * 
 * Given (movable) objects @a a and @a b , @c swap_if() will swap them if the condition @a c holds , avoiding a branch (@c if ) where possible.
 * 
 * 
 * */
template <typename T>
bool swap_if (bool c, T& a, T& b) {
	return c;
}

} }

#elif (CXXOMFORT_CXX_STD >= 2011)

	#include <cxxomfort/config.hpp>
	#include <type_traits>
	#include <algorithm>
	#include <utility> // std::move
	#define CXXOMFORT_IMPLEMENTS_p2187 CXXO_IMPLSTATUS_BACKPORT()

namespace cxxomfort { namespace impl {

template <typename T
	, typename std::enable_if< std::is_scalar<T>::value==true, bool >::type = true
>
bool swap_if (bool c, T& a, T& b) {
	// for scalar it's just easier to use normal assign
	T tmp[2] = { a, b};
	b= tmp[1-c], a= tmp[c];
	return c;
}

template <typename T
	, typename std::enable_if< std::is_scalar<T>::value==false , bool >::type = true
>
bool swap_if (bool c, T& a, T& b) {
	using std::move;
	T tmp[2] = { move(a), move(b) };
	b = move(tmp[1-c]), a = move(tmp[c]);
	return c;
}

} }

#else

	#include <cxxomfort/base.hpp> // move
	#include <cxxomfort/impl/07-type_traits.hpp>
	#include <algorithm>
	#define CXXOMFORT_IMPLEMENTS_p2187 CXXO_IMPLSTATUS_EMULATION()

namespace cxxomfort { namespace impl {

template <typename T>
bool swap_if (bool c, T& a, T& b) {
	using std::move;
	T tmp[2] = { move(a), move(b) };
	b = move(tmp[1-c]), a = move(tmp[c]);
	return c;
}

} }
#endif

#endif
