#ifndef CXXOMFORT_IMPL_SEQ_HPP
#define CXXOMFORT_IMPL_SEQ_HPP
/**
 * @file
 */

// implementation of seq_<T>(), ...

#include <iterator>
#include <cxxomfort/base/iterator.hpp>
#include <valarray>

namespace cxxomfort { namespace library {

namespace impl {

template <typename Ty>
struct generate_seq_t {
    private:
    struct p_notype;
    
    public:
    std::vector<Ty> elems;
    typedef Ty value_type;
    typedef typename std::vector<Ty>::difference_type difference_type;
    typedef typename std::vector<Ty>::const_iterator const_iterator;
    typedef typename std::vector<Ty>::const_iterator iterator;

    generate_seq_t () : elems() { }

    const_iterator begin () const CXXO_NOEXCEPT {
        return std::begin(elems);
    }
    const_iterator cbegin () const CXXO_NOEXCEPT {
        return std::begin(elems);
    }
    const_iterator end () const CXXO_NOEXCEPT {
        return std::end(elems);
    }
    const_iterator cend () const CXXO_NOEXCEPT {
        return std::end(elems);
    }

    generate_seq_t& operator& (Ty const& t) {
        elems.push_back(t);
        return *this;
    }

    generate_seq_t& operator, (Ty const& t) {
        elems.push_back(t);
        return *this;
    }
    
    template <typename Uy>
    generate_seq_t& operator, (Uy y) {
        elems.push_back(static_cast<Ty>(y));
        return *this;
    }

    template <typename Container>
    operator Container const () const {
        using std::begin;
        using std::end;
        return Container(begin(elems), end(elems));
    }
    
    operator std::valarray<Ty> const () const {
        return std::valarray<Ty> (&elems[0], elems.size());
    }
    
};

/**
 * @brief Shorthand to generate a sequence of elements of a given type.
 * @ingroup independent-features
 * 
 * Example usage:
 * @code
vector<int> vec = seq_<int>(), 4, 5, 6, 7, 8, 9;
list<int> lis = seq_<int>() & -4 & -5 & -6 & -7;

 * @endcode
 */
template <typename Ty>
generate_seq_t<Ty> seq_() {
    return generate_seq_t<Ty>();
}

} // impl::

using impl::seq_;

} } // cxxomfort::library

#endif
