#ifndef CXXOMFORT_IMPL_TO_SIGNED_HPP
#define CXXOMFORT_IMPL_TO_SIGNED_HPP
#include <numeric>
#include <cstddef>
#include <cxxomfort/utils/traits_basic.hpp> // make_signed, make_unsigned

namespace cxxomfort { namespace impl { namespace numeric {

//
// make_signed, make_unsigned
//

#if 0
namespace detail {
	
template <size_t S> struct get_signed {};
template <> struct get_signed< sizeof(signed char) > { typedef signed char type; };
template <> struct get_signed< sizeof(int16_t) > { typedef int16_t type; };
template <> struct get_signed< sizeof(int32_t) > { typedef int32_t type; };
template <> struct get_signed< sizeof(int64_t) > { typedef int64_t type; };

template <size_t S> struct get_unsigned {};
template <> struct get_unsigned< sizeof(unsigned char) > { typedef unsigned char type; };
template <> struct get_unsigned< sizeof(uint16_t) > { typedef uint16_t type; };
template <> struct get_unsigned< sizeof(uint32_t) > { typedef uint32_t type; };
template <> struct get_unsigned< sizeof(uint64_t) > { typedef uint64_t type; };


template <typename T> struct make_signed_impl : get_signed<sizeof(T)> {};
template <> struct make_signed_impl<bool> {};

template <typename T> struct make_signed_impl<T const> 
: std::add_const< typename make_signed_impl<T>::type > {};

template <typename T> struct make_signed_impl<T volatile> 
: std::add_volatile< typename make_signed_impl<T>::type > {};

template <typename T> struct make_signed_impl<T const volatile> 
//: std::add_cv< typename make_signed_impl<T>::type > {};
{
	typedef typename make_signed_impl<T>::type const volatile type; 
};

template <typename T> struct make_unsigned_impl : get_unsigned<sizeof(T)> {};
template <> struct make_unsigned_impl<bool> {};

template <typename T> struct make_unsigned_impl<T const> 
: std::add_const< typename make_unsigned_impl<T>::type > {};

template <typename T> struct make_unsigned_impl<T volatile> 
: std::add_volatile< typename make_unsigned_impl<T>::type > {};

template <typename T> struct make_unsigned_impl<T const volatile> 
// : std::add_cv< typename make_unsigned_impl<T>::type > {};
{ 
	typedef typename make_unsigned_impl<T>::type const volatile type; 
};

} // detail

template <typename T> struct make_signed 
: detail::make_signed_impl<T> { };

template <typename T> struct make_unsigned 
: detail::make_unsigned_impl<T> { };

#endif

#if 0
namespace {
    template <typename T>
    struct to_signed_impl {};

    template <> struct to_signed_impl<char> {
        typedef char type;
        typedef signed char signed_type;
        typedef unsigned char unsigned_type;
    };

    template <> struct to_signed_impl<signed short> {
        typedef short type;
        typedef signed short signed_type;
        typedef unsigned short unsigned_type;
    };
    template <> struct to_signed_impl<unsigned short> : to_signed_impl<signed short>{};

    template <> struct to_signed_impl<signed int> {
        typedef int type;
        typedef signed int signed_type;
        typedef unsigned int unsigned_type;
    };
    template <> struct to_signed_impl<unsigned int> : to_signed_impl<signed int>{};

    template <> struct to_signed_impl<signed long> {
        typedef long type;
        typedef signed long signed_type;
        typedef unsigned long unsigned_type;
    };
    template <> struct to_signed_impl<unsigned long> : to_signed_impl<signed long>{};
    
    template <> struct to_signed_impl<long long> {
        typedef long long type;
        typedef signed long long signed_type;
        typedef unsigned long long unsigned_type;
    };
    template <> struct to_signed_impl<unsigned long long> : to_signed_impl<signed long long>{};
    
} // anon
#endif

#if 0
/**
 * @brief Provides the equivalent signed integral type to an integral type @a T .
 */
template <typename T, typename If= void>
struct to_signed_t {};

template <typename T>
struct to_signed_t<T, typename cxxomfort::traits::enable_if< std::is_scalar<T>::value >::type> 
: public to_signed_impl<T> {};

#endif

/**
 * @brief Returns the equivalent @c signed equivalent of a C++ integer value.
 * @param t A value fo one of C++'s integer types.
 * @return A value of a type similar to @a t 's, except qualified as @c signed (eg.: "signed int").
 * @ingroup cxxo-utils
 */
template <typename T>
typename cxxomfort::traits::make_signed<T>::type 
to_signed (T t) CXXO_NOEXCEPTNOTHROW {
    return static_cast< typename cxxomfort::traits::make_signed<T>::type >(t);
}

/**
 * @brief Returns the equivalent @c unsigned equivalent of a C++ integer value.
 * @param t A value fo one of C++'s integer types.
 * @return A value of a type similar to @a t 's, except qualified as @c unsigned (eg.: "unsigned int").
 * @ingroup cxxo-utils
 */
template <typename T>
typename cxxomfort::traits::make_signed<T>::type 
to_unsigned (T t) CXXO_NOEXCEPTNOTHROW {
    return static_cast< typename cxxomfort::traits::make_signed<T>::type >(t);
}


} } }

#endif
