#ifndef CXXOMFORT_IMPL_tuple_visit_HPP
#define CXXOMFORT_IMPL_tuple_visit_HPP
/*
 * @file 
 * Support for tuple_visit / tuple_foreach
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/tuple.hpp>
#include <cxxomfort/type_traits.hpp>
#include <cxxomfort/functional.hpp> // std::invoke

namespace cxxomfort { namespace impl {
namespace tuple {

/*
 * returns 1 + the index of the first element of tt
 * for which p(get<>...) returns true
 */
template <typename P, typename Tuple>
int tuple_if (P p, Tuple const& tt) {
    (void)p;
    (void)tt;
    return 0;
}

namespace detail {

template <typename F, typename Tuple, int I= 0, int S = std::tuple_size< typename std::remove_const<Tuple>::type >::value>
struct tuple_visit_each_t {
    public:
    //typedef typename F::result_type result_type;
    tuple_visit_each_t (F& f, Tuple& t) : _f(f), _tuple(t) {
        std::invoke(_f, std::get<I>(_tuple));
        tuple_visit_each_t<F,Tuple,I+1,S>(_f,_tuple);
    }

    private:
    F& _f;
    Tuple& _tuple;
    tuple_visit_each_t& operator= (tuple_visit_each_t const&);
};

template <typename F, typename Tuple, int S>
struct tuple_visit_each_t<F,Tuple,S,S> {
    public:
    //typedef typename F::result_type result_type;
    tuple_visit_each_t (F const&, Tuple const&) {
        // intentionally empty
    }

    private:
    tuple_visit_each_t& operator= (tuple_visit_each_t const&);
};

} // detail

#if (defined(__cpp_rvalue_references) && defined(__cpp_lib_integer_sequence) && defined(__cpp_lib_invoke) )

namespace detail {

template <typename F, typename Tuple, size_t... Is> inline 
void tuple_visit (F& fn, Tuple&& tu, std::index_sequence<Is...>) {
	using swallow = int[];
	//(void)swallow{0, (void(os << (Is == 0? "" : ", ") << std::get<Is>(t)), 0)...};
	
	(void)swallow{ 0, (void(
		std::invoke(fn, std::get<Is>(std::forward<Tuple>(tu)) )
	), 0)...
	};
}

} // detail

template <typename F, typename Tuple> inline 
void tuple_visit (F& f, Tuple&& tu) {
    detail::tuple_visit(f,tu, std::make_index_sequence< std::tuple_size< typename std::decay<Tuple>::type >::value >{} );
    //std::cerr<< "[tuple_visit(1)]"<< std::flush;
}

#else

template <typename F, typename Tuple> inline 
Tuple& tuple_visit (F& f, Tuple& t) {
    detail::tuple_visit_each_t<F,Tuple>(f,t);
    //std::cerr<< "[tuple_visit(0)]"<< std::flush;
    return t;
}

#endif

} // namespace tuple
} } // namespace cxxomfort::impl

#endif // 
