#ifndef CXXOMFORT_IMPL_TUPLEMETA_HPP
#define CXXOMFORT_IMPL_TUPLEMETA_HPP

/**
 * Helper utilities to work with tuples.
 * 
 * Interfaces defined here:
 * 
 *  * * @c is_tuple
 *  * * @c tuple_index
 *  * * @c tuple_unshift_type , @c tuple_unshift
 * 
 * */

#include <cxxomfort/base.hpp>
#include <cxxomfort/tuple.hpp>
#include <cxxomfort/utility.hpp> // make_index_sequence in C++11
#include <cxxomfort/type_traits.hpp>
#include "using_tr1_type_traits.hpp"

namespace cxxomfort { namespace impl {


/**
 * @brief type_traits that determines if the type T is a @c std::tuple .
 */
template <typename T> struct is_tuple: traits::false_type {};

#if (CXXOMFORT_CXX_STD >= 2011) || (CXXO_COMPILER_SUPPORT_variadic) 

//! Inherits from @c true_type  if @p T  is a <code>std::tuple</code>.
template <typename... Args> struct is_tuple< std::tuple<Args...> >
: std::true_type {};

#elif (CXXOMFORT_CXX_STD>=1997)

// assume implementation of tuple in tr1 and supports at least 10 elements

template <typename T1> struct is_tuple <std::tuple<T1> >
: traits::true_type {};
template <typename T1, typename T2> 
struct is_tuple <std::tuple<T1,T2> >
: traits::true_type {};
template <typename T1, typename T2, typename T3> 
struct is_tuple <std::tuple<T1,T2,T3> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4> 
struct is_tuple <std::tuple<T1,T2,T3,T4> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4, typename T5> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5> >
: traits::true_type {};

	#if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_MSC && _VARIADIC_MAX < 9)
	#else

template <typename T1, typename T2, typename T3, typename T4, typename T5, typename T6> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5,T6> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5,T6,T7> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5,T6,T7,T8> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> >
: traits::true_type {};

	#endif

#elif (CXXOMFORT_IMPLEMENTS_tuple>=CXXO_IMPLSTATUS_EMULATION())
	// cxxomfort tuple is limited to 5 args

template <typename T1> struct is_tuple <std::tuple<T1> >
: traits::true_type {};
template <typename T1, typename T2> 
struct is_tuple <std::tuple<T1,T2> >
: traits::true_type {};
template <typename T1, typename T2, typename T3> 
struct is_tuple <std::tuple<T1,T2,T3> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4> 
struct is_tuple <std::tuple<T1,T2,T3,T4> >
: traits::true_type {};
template <typename T1, typename T2, typename T3, typename T4, typename T5> 
struct is_tuple <std::tuple<T1,T2,T3,T4,T5> >
: traits::true_type {};

#else // c++03

#endif // c++ standard


#if (CXXOMFORT_IMPLEMENTS_tuple>=CXXO_IMPLSTATUS_EMULATION())

#endif // implements

// 
// tuple_count_type
// 

#if (CXXOMFORT_CXX_STD>=2011 || CXXOMFORT_IMPLEMENTS_tuple)

/*
 * Check if an element of type T is already in Tuple, 
 * if so, returns its index (0...N-1)
 * 
 * @tparam S = size of the tuple Tuple.
 */
template <typename C, size_t N, size_t S, typename Tuple>
struct tuple_element2index_helper {
    enum { rec = (0 <= N && N < S) };
    static const size_t X = tuple_element2index_helper<C,N+1,S,Tuple>::value ;
    typedef typename std::enable_if<rec, bool>::type still_has_arguments_t;
    static const size_t value = 
    (std::is_same<C, typename std::tuple_element<N,Tuple>::type>::value
    ? N : X
    ) ;
};


template <typename C, size_t N, typename Tuple>
struct tuple_element2index_helper<C,N,N,Tuple> {
    enum { value = N+1 };
};



/*
 * Count the number of times type C appears in Tuple
 */
template <typename C, size_t N, size_t S, typename Tuple>
struct tuple_count_type_helper {
    enum { value =
    std::is_same< C, typename std::tuple_element<N,Tuple>::type>::value 
    + tuple_count_type_helper<C,N+1,S,Tuple>::value
    };
};

template <typename C, size_t S, typename Tuple>
struct tuple_count_type_helper<C,S,S,Tuple> {
    enum { value = 0 };
};


// 
// tuple_index
// 


namespace detail {

/*
 * Check if an element of type T is already in Tuple, 
 * if so, returns its index (0...N-1)
 * 
 * @tparam S = size of the tuple Tuple.
 */
template <typename C, size_t N, size_t S, typename Tuple>
struct tuple_element2index_helper {
    enum { rec = (0 <= N && N < S) };
    static const size_t X = tuple_element2index_helper<C,N+1,S,Tuple>::value ;
    typedef typename std::enable_if<rec, bool>::type still_has_arguments_t;
    static const size_t value = 
    (std::is_same<C, typename std::tuple_element<N,Tuple>::type>::value
    ? N : X
    ) ;
};


template <typename C, size_t N, typename Tuple>
struct tuple_element2index_helper<C,N,N,Tuple> {
    enum { value = N+1 };
};

template <typename T1,typename T2>
struct tuple_element2index_helper_pair {
    enum { value= (0==std::is_same<T1,T2>::value) };
    static_assert (value, "Tuple get<type> interface for pair requires pair of distinct types");
};

/*
 * Count the number of times type C appears in Tuple
 */
template <typename C, size_t N, size_t S, typename Tuple>
struct tuple_count_type_helper {
    enum { value =
    std::is_same< C, typename std::tuple_element<N,Tuple>::type>::value 
    + tuple_count_type_helper<C,N+1,S,Tuple>::value
    };
};

template <typename C, size_t S, typename Tuple>
struct tuple_count_type_helper<C,S,S,Tuple> {
    enum { value = 0 };
};

} // namespace detail

/**
 * @brief Counts the number of times that type @a C appears in tuple @a Tuple.
 * @return The member @c value is defined as an integer = the number of times type C appears in Tuple.
 * @post 0 ≤ @a value < tuple_size<Tuple>::value 
 * 
 */
template <typename C, typename Tuple>
struct tuple_count_type {
    enum { value = detail::tuple_count_type_helper<C,0,std::tuple_size<Tuple>::value,Tuple>::value };
};

/**
 * @brief Gives the index <code><0...S-1></code> at which type @a C appears in @a Tuple .
 * 
 * The member @c value defined as the number of times.
 * 
 * The member type @a Sentinel is an implementation-defined type.
 * 
 * @warning If @a C appears more than once, @c value is not defined, and @c Sentinel is the type @c void .
 * @warning If @a C does not appear in the tuple at all, compilation error.
 * 
 * 
 * */
template <typename C, typename Tuple>
struct tuple_index {
    private:
    typedef typename std::remove_reference<Tuple>::type T;
    static const size_t va= tuple_count_type<C,T>::value;
    
    static_assert ((va < 2), "Type (C in template) is repeated in tuple.");
    //static_assert ((va != 0), "Type (C in template) does not exist in tuple.");
    public:
    typedef typename std::conditional< (va>0) , bool, void>::type Sentinel_t;
    enum { 
        //! The value of this trait.
        value = detail::tuple_element2index_helper<C,0UL,std::tuple_size<T>::value,T>::value 
    };
};

#endif

// 
// tuple_unshift_t, tuple_unshift
// adds an element to the beginning of a tuple
// 


template <typename T, typename Tuple>
struct tuple_unshift_t;

#if (CXXOMFORT_CXX_STD>=2011)

/*
template <typename T, typename... TArgs>
std::tuple<T, TArgs...> tuple_unshift 
( T const& v, std::tuple<TArgs...> const& tu) {
	return std::tuple_cat ( std::make_tuple(v), tu);
}

*/

template <typename T, typename... Args>
struct tuple_unshift_t<T, std::tuple<Args...> > {
	typedef std::tuple<Args...> t2;
	typedef std::tuple<T, Args...> type;
	
	static type call (T const& t0, t2 const& tu) {
		using namespace std;
		make_index_sequence< sizeof...(Args)> IS;
		return tuple_cat( make_tuple(t0), tu);
	}
};

#elif (CXXOMFORT_IMPLEMENTS_tuple >= CXXO_IMPLSTATUS_EMULATION())

template <typename T, typename T0>
struct tuple_unshift_t<T, std::tuple<T0> > {
	typedef std::tuple<T,T0> type;
	typedef std::tuple<T0> argument_type;
	static type call (T const& tt, argument_type const& t) {
		return type(tt, std::get<0>(t));//  std::tuple_cat (std::make_tuple(t), tt);
	}
};

template <typename T, typename T0, typename T1>
struct tuple_unshift_t<T, std::tuple<T0,T1> > {
	typedef std::tuple<T,T0,T1> type;
	static type call (T const& tt, std::tuple<T0,T1> const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t));
	}
};

template <typename T, typename T0, typename T1, typename T2>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2> > {
	typedef std::tuple<T0,T1,T2> argument_type;
	typedef std::tuple<T,T0,T1,T2> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t));
	}
};

template <typename T, typename T0, typename T1, typename T2, typename T3>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3> > {
	typedef std::tuple<T0,T1,T2,T3> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t));
	}
};

template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3,T4> > {
	typedef std::tuple<T0,T1,T2,T3,T4> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3,T4> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t), get<4>(t));
	}
};

	#if (CXXOMFORT_IMPLEMENTS_tuple >= CXXO_IMPLSTATUS_NATIVE())

		#if (defined(_VARIADIC_MAX) && (_VARIADIC_MAX<6))
		#else
template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3,T4,T5> > {
	typedef std::tuple<T0,T1,T2,T3,T4,T5> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3,T4,T5> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t), get<4>(t), get<5>(t) );
	}
};

		#endif

		#if (defined(_VARIADIC_MAX) && (_VARIADIC_MAX<7))
		#else
template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3,T4,T5,T6> > {
	typedef std::tuple<T0,T1,T2,T3,T4,T5,T6> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3,T4,T5,T6> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t), get<4>(t), get<5>(t), get<6>(t) );
	}
};

		#endif

		#if (defined(_VARIADIC_MAX) && (_VARIADIC_MAX<8))
		#else
template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3,T4,T5,T6,T7> > {
	typedef std::tuple<T0,T1,T2,T3,T4,T5,T6,T7> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3,T4,T5,T6,T7> type;
	static type call (T const& tt, argument_type const& t) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t), get<4>(t), get<5>(t), get<6>(t), get<7>(t) );
	}
};

		#endif

		#if (defined(_VARIADIC_MAX) && (_VARIADIC_MAX<9))
		#else
template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8>
struct tuple_unshift_t<T, std::tuple<T0,T1,T2,T3,T4,T5,T6,T7,T8> > {
	typedef std::tuple<T0,T1,T2,T3,T4,T5,T6,T7,T8> argument_type;
	typedef std::tuple<T,T0,T1,T2,T3,T4,T5,T6,T7,T8> type;
	static type call (T const& t, argument_type const& tt) {
		using std::get;
		return type(tt, get<0>(t), get<1>(t), get<2>(t), get<3>(t), get<4>(t), get<5>(t), get<6>(t), get<7>(t), get<8>(t) );
	}
};

		#endif

	#endif

#endif

// 
// tuple_push_t, tuple_push
// adds an element to the end of a tuple
// 

template <typename T, typename Tuple>
struct tuple_push_t;

#if (CXXOMFORT_CXX_STD >= 2011)

template <typename T, typename... Args>
struct tuple_push_t<T, std::tuple<Args...> > {
	typedef std::tuple<Args...> t2;
	typedef std::tuple<Args..., T> type;
	static type call (T const& tx, t2 const& tu) {
		using namespace std;
		return tuple_cat(tu, make_tuple(tx));
	}
};

#elif (CXXOMFORT_IMPLEMENTS_tuple >= CXXO_IMPLSTATUS_NATIVE())

template <typename T, typename T0>
struct tuple_push_t<T, std::tuple<T0> > {
	typedef std::tuple<T0, T> type;
	typedef std::tuple<T0> argument_type;
	static type call (T const& t, argument_type const& tu) {
		return type(std::get<0>(tu), t);
	}
};

template <typename T, typename T0, typename T1>
struct tuple_push_t<T, std::tuple<T0,T1> > {
	typedef std::tuple<T0,T1, T> type;
	typedef std::tuple<T0,T1> argument_type;
	static type call (T const& t, argument_type const& tu) {
		using std::get;
		return type(get<0>(tu), get<1>(tu),  t);
	}
};

template <typename T, typename T0, typename T1, typename T2>
struct tuple_push_t<T, std::tuple<T0,T1,T2> > {
	typedef std::tuple<T0,T1,T2, T> type;
	typedef std::tuple<T0,T1,T2> argument_type;
	static type call (T const& t, argument_type const& tu) {
		using std::get;
		return type(get<0>(tu), get<1>(tu), get<2>(tu)
		,  t);
	}
};

template <typename T, typename T0, typename T1, typename T2, typename T3>
struct tuple_push_t<T, std::tuple<T0,T1,T2,T3> > {
	typedef std::tuple<T0,T1,T2,T3, T> type;
	typedef std::tuple<T0,T1,T2,T3> argument_type;
	static type call (T const& t, argument_type const& tu) {
		using std::get;
		return type(get<0>(tu), get<1>(tu), get<2>(tu), get<3>(tu)
		,  t);
	}
};

template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4>
struct tuple_push_t<T, std::tuple<T0,T1,T2,T3,T4> > {
	typedef std::tuple<T0,T1,T2,T3,T4, T> type;
	typedef std::tuple<T0,T1,T2,T3,T4> argument_type;
	static type call (T const& t, argument_type const& tu) {
		using std::get;
		return type(get<0>(tu), get<1>(tu), get<2>(tu), get<3>(tu), get<4>(tu)
		,  t);
	}
};

	#if defined(_VARIADIC_MAX) && (VARIADIC_MAX < 6)
		#pragma message CXXO_WARNING("_VARIADIC_MAX set too low - C++ TR1 indicates at least = 10")
	#else

template <typename T, typename T0, typename T1, typename T2, typename T3, typename T4, typename T5>
struct tuple_push_t<T, std::tuple<T0,T1,T2,T3,T4,T5> > {
	typedef std::tuple<T0,T1,T2,T3,T4,T5, T> type;
	typedef std::tuple<T0,T1,T2,T3,T4,T5> argument_type;
	static type call (T const& t, argument_type const& tu) {
		using std::get;
		return type(get<0>(tu), get<1>(tu), get<2>(tu), get<3>(tu), get<4>(tu), get<5>(tu)
		,  t);
	}
};

	#endif

#endif

// 
// tuple_shift_t, tuple_shift
// removes element from the beginning of a tuple
// 


template <typename Tuple>
struct tuple_shift_t;

#if (CXXOMFORT_CXX_STD >= 2011)

template <typename T, typename... TArgs>
struct tuple_shift_t< std::tuple<T, TArgs...> > {
	typedef std::tuple<TArgs...> type;
};

#elif (CXXOMFORT_IMPLEMENTS_tuple >= CXXO_IMPLSTATUS_NATIVE())

template <typename T, typename T1>
struct tuple_shift_t< std::tuple<T, T1> > {
	typedef std::tuple<T1> type;
	typedef std::tuple<T, T1> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<1>(tu) );
	}
};

template <typename T, typename T1, typename T2>
struct tuple_shift_t< std::tuple<T, T1,T2> > {
	typedef std::tuple<T1,T2> type;
	typedef std::tuple<T, T1,T2> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<1>(tu), get<2>(tu) );
	}
};

template <typename T, typename T1, typename T2, typename T3>
struct tuple_shift_t< std::tuple<T, T1,T2,T3> > {
	typedef std::tuple<T1,T2,T3> type;
	typedef std::tuple<T, T1,T2,T3> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<1>(tu), get<2>(tu), get<3>(tu) );
	}
};

template <typename T, typename T1, typename T2, typename T3, typename T4>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4> > {
	typedef std::tuple<T1,T2,T3,T4> type;
	typedef std::tuple<T, T1,T2,T3,T4> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<1>(tu), get<2>(tu), get<3>(tu), get<4>(tu) );
	}
};

	#if defined(_VARIADIC_MAX) && (VARIADIC_MAX < 6)
		#pragma message CXXO_WARNING("_VARIADIC_MAX set too low - C++ TR1 indicates at least = 10")
	#else

template <typename T, typename T1, typename T2, typename T3, typename T4, typename T5>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4,T5> > {
	typedef std::tuple<T1,T2,T3,T4,T5> type;
};

template <typename T, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4,T5,T6> > {
	typedef std::tuple<T1,T2,T3,T4,T5,T6> type;
};

template <typename T, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4,T5,T6,T7> > {
	typedef std::tuple<T1,T2,T3,T4,T5,T6,T7> type;
};

template <typename T, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4,T5,T6,T7,T8> > {
	typedef std::tuple<T1,T2,T3,T4,T5,T6,T7,T8> type;
};

template <typename T, typename T1, typename T2, typename T3, typename T4, typename T5, typename T6, typename T7, typename T8, typename T9>
struct tuple_shift_t< std::tuple<T, T1,T2,T3,T4,T5,T6,T7,T8,T9> > {
	typedef std::tuple<T1,T2,T3,T4,T5,T6,T7,T8,T9> type;
};

	#endif

#endif

// 
// tuple_pop_t, tuple_pop
// removes element from the beginning of a tuple
// 

template <typename Tuple>
struct tuple_pop_t;

#if (CXXOMFORT_CXX_STD >= 2011)

namespace pop_{
template <typename S, typename... Ts> struct impl_;
template <size_t... I, typename... Ts>
struct impl_ < std::index_sequence<I...>, Ts...> {
	typedef std::index_sequence<I...> IS;
	typedef std::tuple< typename std::tuple_element<I, std::tuple<Ts...> > ::type... > type;
};

} // pop_

template <typename... TArgs>
struct tuple_pop_t< std::tuple<TArgs...> > {
	typedef std::make_index_sequence< sizeof...(TArgs)-1 > IS;
	typedef typename pop_::impl_<IS , TArgs...>::type type;
	typedef std::tuple<TArgs...> argument_type;
	template <size_t... I>
	static type call ( argument_type const& tu, IS = IS{} ) {
		using namespace std;
		return type ( get<I>(tu)... );
	}
	
};

#elif (CXXOMFORT_IMPLEMENTS_tuple >= CXXO_IMPLSTATUS_NATIVE())

template <typename T, typename T1>
struct tuple_pop_t< std::tuple<T1,  T> > {
	typedef std::tuple<T1> type;
	typedef std::tuple<T1,  T> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<0>(tu) );
	}
};

template <typename T, typename T1, typename T2>
struct tuple_pop_t< std::tuple<T1,T2,  T> > {
	typedef std::tuple<T1,T2> type;
	typedef std::tuple<T1,T2,  T> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<0>(tu), get<1>(tu) );
	}
};

template <typename T, typename T1, typename T2, typename T3>
struct tuple_pop_t< std::tuple<T1,T2,T3,  T> > {
	typedef std::tuple<T1,T2,T3> type;
	typedef std::tuple<T1,T2,T3,  T> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<0>(tu), get<1>(tu), get<2>(tu) );
	}
};

template <typename T, typename T1, typename T2, typename T3, typename T4>
struct tuple_pop_t< std::tuple<T1,T2,T3,T4,  T> > {
	typedef std::tuple<T1,T2,T3,T4> type;
	typedef std::tuple<T1,T2,T3,T4,  T> argument_type;
	static type call ( argument_type const& tu) {
		using std::get;
		return type( get<0>(tu), get<1>(tu), get<2>(tu), get<3>(tu) );
	}
};

#endif


}} // cxxomfort::impl

#endif
