#ifndef CXXOMFORT_IMPL_TYPESAFEENUM_HPP
#define CXXOMFORT_IMPL_TYPESAFEENUM_HPP
/**
 @file
 @author Luis Machuca Bezzaza
 @see http://en.wikibooks.org/wiki/More_C%2B%2B_Idioms/Type_Safe_Enum
 @see http://stackoverflow.com/questions/217549/which-typesafe-enum-in-c-are-you-using/11856721#11856721
**/
#include <cxxomfort/base.hpp>
#include <cxxomfort/impl/07-type_traits.hpp>
#include <cxxomfort/impl/using_tr1_type_traits.hpp>
#include <cxxomfort/impl/11-conditional.hpp>
#include <cxxomfort/impl/11-underlying_type.hpp>
//#include <cxxomfort/using.hpp>

/*
 * Usage is as follows:
 */
#if 0 

// example
//struct myenum_def { enum type { member1, member2, member3 }; };
//typedef typesafe_enum<myenum_def> myenum;
// quick and easy encapsulation

#endif

/* Section: declaration of safe_enum */

/*
 * The implementation relies on some type identification measures such as is_enum. If those are not available, 'inner_type' is assumed to be a native type convertible to any enum type.
 * 
 * */

namespace cxxomfort { namespace impl {

namespace detail_enum {
struct safe_enum_tag {};
} // detail_enum


template<typename def, typename inner = typename def::type>
class typesafe_enum : public def /*, public detail_enum::safe_enum_tag */ {
    inner val;
    public: 
    typedef typename def::type enum_type;
    typedef inner inner_type;
#if (CXXOMFORT_CXX_STD>=1997 && CXXO_COMPILER_SUPPORT_type_traits_intrinsic)
    typedef typename std::conditional<
        std::is_enum<inner>::value
        , typename std::underlying_type< /*inner*/ typename def::type >::type , inner
    >::type underlying_type;
#else
	typedef inner underlying_type;
#endif

    public:
    // as per standard sections 3.9.9, 8.5.5, 8.5.7 and 8.5.10 together, see https://stackoverflow.com/questions/17630967/user-defined-c11-enum-class-default-constructor
    // an enum class is def-ctible as if it was value-initialized to 0.
     
    //CXXO_CONSTEXPR typesafe_enum () CXXO_NOEXCEPTNOTHROW 
    //: val(0) {}

    CXXO_CONSTEXPR typesafe_enum (inner_type e = static_cast<inner_type>(0) ) CXXO_NOEXCEPTNOTHROW 
    : val(e) {}

/*
    typesafe_enum (::cxxomfort::detail::explicit_cast<typesafe_enum> const& e)
    : val (e)
    {}
*/

    /*
    template <typename Q>
    CXXO_CONSTEXPR explicit typesafe_enum (Q v
    , typename ::std::enable_if< false==::std::is_same<Q,underlying_type>::value, Q>::type* = nullptr) 
    : val(v) {}
    */

    CXXO_CONSTEXPR 
    enum_type value() const CXXO_NOEXCEPTNOTHROW { return static_cast<enum_type>(val); }
    CXXO_CONSTEXPR 
    inner underlying() const CXXO_NOEXCEPTNOTHROW { return val; }

    CXXO_EXPLICIT_OPERATOR(underlying_type) () const CXXO_NOEXCEPTNOTHROW {
        return val;
    }

#if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_GCC || CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_CLANG)
    template <typename I>
    CXXO_CONSTEXPR CXXO_EXPLICIT_OPERATOR(I) () const CXXO_NOEXCEPTNOTHROW { 
        return static_cast<I>(val); 
    }
#endif

    /*{
        enum { T_is_compatible_type = std::is_same<T, enum_type>::value };
        static_assert (T_is_compatible_type, "typesafe_enum construction invalid - not the enum's tags or the underlying type");
    };*/

    friend CXXO_CONSTEXPR bool
    operator== (typesafe_enum const& vl, typesafe_enum const& vr) CXXO_NOEXCEPTNOTHROW {
        return vl.val == vr.val;
    }
    	
    private:
    struct disallowed_enum_conversion {};
    // we disallow other constructors

};

template< typename def, typename inner>
CXXO_CONSTEXPR inline 
typename typesafe_enum<def,inner>::enum_type     
to_value (typesafe_enum<def,inner> e) CXXO_NOEXCEPTNOTHROW {
	return e.value();
}

template< typename def, typename inner>
CXXO_CONSTEXPR inline 
typename typesafe_enum<def,inner>::underlying_type
to_integer (typesafe_enum<def,inner> e) CXXO_NOEXCEPTNOTHROW {
	return e.underlying();
}


// inequality (uses friend version of equality)
template< typename def, typename inner>
inline CXXO_CONSTEXPR 
bool operator!= (typesafe_enum<def,inner> const& e1, typesafe_enum<def,inner> const& e2) CXXO_NOEXCEPTNOTHROW {
    return !(e1==e2);
}

#if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_DMC)
#else
// construct typesafe_enum from the underlying type (useful for "to-byte"-style helpers)
// usage: TSE e = from_underlying<TSE>(u)
template <typename TSE>
TSE from_underlying ( typename TSE::underlying_type c ) {
    return static_cast<typename TSE::enum_type>(c);
}
#endif

#if (0 && CXXOMFORT_CXX_STD>=1997)

template <bool isenum, typename T>
struct is_typesafeenum_helper;

template <typename T> struct is_typesafeenum_helper<false,T> {
    // is_class == false
    enum { value= false };
};

template <typename T> struct is_typesafeenum_helper<true,T> {
    // is_class == true
    enum { value= std::is_base_of<detail_enum::safe_enum_tag,T>::value 
        && std::is_enum<typename T::type>::value };
};

template <typename T>
struct is_typesafe_enum 
: is_typesafeenum_helper <std::is_class<T>::value, T> {};

template <typename T, typename S>
struct is_typesafe_enum< ::cxxomfort::impl::typesafe_enum<T,S> > 
// : is_scoped_enum_helper < false, ::cxxomfort::extras::safe_enum<T,S> > {};
{ enum { value = true}; };

#endif

#if 0
/* An enumerator is scoped if 
 * it is a native enum but it's not convertible 
 * to its underlying type.
 */

template <bool B, typename T>
struct is_scoped_enum_helper;

template <typename T>
struct is_scoped_enum_helper<false, T> {
    // T is not an integral type
    static const bool value = 
    !std::is_convertible<T, typename std::underlying_type<T>::type>::value
    ;
};

template <typename T>
struct is_scoped_enum_helper<true, T> {
    // T is an integral type
    static const bool value = false;
};

template <typename T>
struct is_scoped_enum 
: is_scoped_enum_helper< std::is_integral<T>::value, T > {};

#endif

} // library
} // cxxomfort::

/* End section: safe_enum */



#ifndef CXXOMFORT_CXX11_MODE
#define CXXO_ENUM_BEGIN(Name,Storage) struct Name##0_def { \
    typedef Storage storage_type; \
    enum type 
#define CXXO_ENUM_END(Name,Storage) ; \
}; \
template <typename T> \
typename std::enable_if< !std::is_same< Name##0_def::type, T>::value, bool>::type \
operator== (Name##0_def::type, T) { \
    enum { v= !std::is_same< Name##0_def::type, T>::value }; \
    typedef char enum_comparison_disallowed[-v];  \
    /* static_assert (!v, "Comparison of scoped enum value"); */ \
} \
typedef ::cxxomfort::extras::safe_enum< Name##0_def , Storage > Name ; \

#else
#define CXXO_ENUM_BEGIN(Name,Storage) enum class Name : Storage
#define CXXO_ENUM_END(Name,Storage) 

#endif // c++11


#endif // SAFEENUM.HPP
