#ifndef CXXOMFORT_IMPL_VALCOMP_HPP
#define CXXOMFORT_IMPL_VALCOMP_HPP

/**
 * @file
 * @brief Implements a generic strcmp-like (three-way) comparator.
 * 
 * */
namespace cxxomfort { namespace impl {

/**
 * @brief Compares two objects and returns a 3-state comparison result the way @c memcmp , @c strcmp do.
 * @returns @c 0 if both arguments compare equal; otherwise a negative or a positive number if the first object compares "less" than the second or viceversa, respectively.
 * @throws Nothing. This function assumes comparators don't throw.
 * 
 * @ingroup cxxo-sup-algorithm
 * 
 * */
template <typename T, typename Less>
int valcmp (T const& v1, T const& v2, Less comp) CXXO_NOEXCEPTNOTHROW {
    return comp(v1,v2);
}

/**
 * @overload valcmp
 * 
 * */
template <typename T>
inline 
int valcmp (T const& v1, T const& v2) CXXO_NOEXCEPTNOTHROW {
    return v1==v2 ? 0 : (v1<v2 ? -1 : +1);
}

}}

#endif
