#ifndef CXXOMFORT_IOS_HPP
#define CXXOMFORT_IOS_HPP
/**
 * @file
 * @brief Implementations and additions tied to Standard Header <ios>.
 * @xrefitem std0ios "<ios>" ""
 *
 */

#include <cxxomfort/base.hpp>
#include <iosfwd>
#include <ios>

// 
// io_errc
// is_error_code_enum
// 

#if (CXXO_CXXSTDHEADER_system_error>0)
#elif (CXXOMFORT_CXX_STD >= 1997 && CXXOMFORT_CXX_STD < 2011)

#include <cxxomfort/utils/typesafe_enum.hpp>
#include <cxxomfort/system_error.hpp>

namespace std {

namespace{
struct my_io_errc { enum type { stream = 1 }; };
}

typedef ::cxxomfort::typesafe_enum<my_io_errc> io_errc;

template<> struct is_error_code_enum<io_errc> 
: std::true_type {};

template<> struct is_error_code_enum< my_io_errc::type> 
: std::true_type {};

} 

#endif

//
// missing ios stuff in pre-C++97
// 

#if (CXXO_COMPILERREQ(CXXO_COMPILERID_OW,<=2000))

namespace std {

ios& boolalpha (ios& s) {  return s; };
ios& noboolalpha (ios& s) {  return s; };

} // std

#endif

/**
 * @page std0ios <ios>
 * @brief Backports related to Standard Header <tt><ios></tt> 
 * 
 * @code
 * #include <cxxomfort/ios.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * 
 * C++03 and onwards don't usually need anything to be done for 
 * <tt><ios></tt>; however for some older compilers that are missing out 
 * on the full ios interface this header provides some
 * fixes and missing std names.
 * 
 * @section Interfaces 
 * 
 * Interfaces defined in this section:
 * 
 * * @c io_errc (C++11).
 * * <tt>is_error_code_enum<io_errc></tt> (C++11).
 * * @c boolalpha (C++98) for Open Watcom.
 * 
 * Non-backport interfaces:
 * 
 * * (none)
 * 
 * @section also See Also
 * 
 * * @see @cppref{header/ios} (cppreference)
 * 
 * */

#endif
