#ifndef CXXOMFORT_LIBRARY_HPP
#define CXXOMFORT_LIBRARY_HPP

#include <cxxomfort/base.hpp>
#include <cxxomfort/cxxo-utils.hpp>

namespace cxxomfort {
namespace library {
}
}

//
// extensions to cstd headers
//

#include "library/array.hpp"
#include "library/algorithm.hpp"
#include "library/bit.hpp"
#include "library/ctime.hpp"
#include "library/functional.hpp"
#include "library/iterator.hpp"
#include "library/limits.hpp"
#include "library/memory.hpp"
#include "library/numeric.hpp"
#include "library/random.hpp"
#include "library/ratio.hpp"
#include "library/string.hpp"
#include "library/tuple.hpp"
#include "library/type_traits.hpp"
#include "library/utility.hpp"

// array_ref - view to a section of contigous memory
#include "library/array_ref.hpp"
// object-like primitive wrapper for fundamentals (int, long, etc)
// #include "library/fundamental.hpp"
// initialization of containers and sequences
#include "library/i12n.hpp"
// local function emulation
//#include "library/localfn.hpp"
// functional plus<> like complements for +=, -=, ...
#include "library/operatorit.hpp"
// sequence helpers
#include "library/sequences.hpp"
// typename to string helpers - typeid_demangle, type_name
#include "library/type_name.hpp"
// typesafe_enum
//#include "library/typesafe_enum.hpp"

/**
 * @page cxxo-sup Library Supplements
 * @brief Implementations and additions proper to the cxxomfort library.
 * 
 * The @b cxxomfort library does not only implement backports to C++ features; it also brings its own set of features to supplement various functionality that, for example, can not be backported to previous version in a meaningful fashion. 
 * 
 * Including the <code><cxxomfort/library.hpp></code> header and invoking the <code>cxxomfort::</code> namespace makes these features available.
 * 
 * @section cxxomfort_supplements Supplements to the Standard Headers
 * 
 * * @ref cxxo-sup-algorithm
 * * @ref cxxo-sup-any
 * * @ref cxxo-sup-bit
 * * @ref cxxo-sup-cstring
 * * @ref cxxo-sup-ctime
 * * @ref cxxo-sup-expected
 * * @ref cxxo-sup-functional
 * * @ref cxxo-sup-iterator
 * * @ref cxxo-sup-memory
 * * @ref cxxo-sup-numeric
 * * @ref cxxo-sup-random
 * * @ref cxxo-sup-ratio
 * * @ref cxxo-sup-string
 * * @ref cxxo-sup-tuple
 * * @ref cxxo-sup-type_traits
 * * @ref cxxo-sup-utility
 * 
 * See also @ref cxxomfort-libraryfeaturemacros .
 * 
 * @section cxxomfort_library_tools Cxxomfort Library Tools
 * 
 * * @subpage cxxo-fundamental "Fundamental"
 * * @subpage cxxo-sequences "Sequence Utilities"
 *   * @ref library/array_ref.hpp @c "array_ref"
 *   * @ref library/fixed_vector.hpp @c "fixed_vector"
 *   * @ref library/static_vector.hpp @c "static_vector"
 *   * @ref library/mstack.hpp @c "mstack"
 * * @subpage cxxo-typesafe_enum "Typesafe @c enum "
 * * @ref cxxo-text 
 * 
 * @section lifted Lifted Utilities
 * 
 * The following tools previously in the library have been lifted to @ref cxxo-utils . They are still accessible with their @c <library> headers for compatibility.
 * 
 * * @subpage cxxo-foreach "Foreach emulation" - see @c @ref CXXO_FOREACH .
 * * @subpage cxxo-i12n "I12N (Container Initialization)"
 * 
 */



#endif
