#ifndef CXXOMFORT_LIBRARY_ARRAY_REF_HPP
#define CXXOMFORT_LIBRARY_ARRAY_REF_HPP
//! @file 

#include <cxxomfort/utils/array_ref.hpp>
#include <cxxomfort/string.hpp>
#include <vector>

#define CXXO_LIB_array_ref CXXO_impl_array_ref

namespace cxxomfort { namespace supplemental { 

    using ::cxxomfort::array_ref;
    using ::cxxomfort::arrayref;
    using ::cxxomfort::carrayref;

// 
// helper functions
// 

template <typename T>
array_ref<T> arrayref (std::vector<T>& v) CXXO_NOEXCEPTNOTHROW {
	return array_ref<T>(v.data(), v.size());
}

template <typename T>
array_ref<T const> carrayref (std::vector<T> const& v) CXXO_NOEXCEPTNOTHROW {
	return array_ref<T const>(v.data(), v.size());
}

static inline
array_ref<char> arrayref (std::string const& s) CXXO_NOEXCEPTNOTHROW {
	return array_ref<char>((char*)(&s[0]), s.size());
}

static inline
array_ref<wchar_t> arrayref (std::wstring const& s) CXXO_NOEXCEPTNOTHROW {
	return array_ref<wchar_t>((wchar_t*)(&s[0]), s.size());
}

static inline
array_ref<char const> carrayref (std::string const& s) CXXO_NOEXCEPTNOTHROW {
	return array_ref<char const>((char const*)(&s[0]), s.size());
}

static inline
array_ref<wchar_t const> carrayref (std::wstring const& s) CXXO_NOEXCEPTNOTHROW {
	return array_ref<wchar_t const>((wchar_t const*)(&s[0]), s.size());
}

} }

namespace cxxomfort { namespace library {
	// DEPRECATED
	//! Mirrors @ref cxxomfort::array_ref .
	using ::cxxomfort::supplemental::array_ref;
	//! Mirrors @ref cxxomfort::arrayref() .
	using ::cxxomfort::supplemental::arrayref;
	//! Mirrors @ref cxxomfort::carrayref() .
	using ::cxxomfort::supplemental::carrayref;
	
} }

/**
 * @file
 * @brief Provides helper functions for cxxomfort's @c array_ref .
 * 
 * This header provides the official Supplemental access point for the following features of cxxomfort, mirroring them in namespace @c cxxomfort::library:: .
 * 
 * * @ref cxxomfort::array_ref .
 * * @ref cxxomfort::arrayref() .
 * * @ref cxxomfort::carrayref() .
 * 
 * This header also adds the following extensions to the factory functions:
 * 
 * * <tt>arrayref( std::vector<T>& ) -> array_ref<T></tt>
 * * <tt>arrayref( std::vector<T> const& ) -> array_ref<T></tt>
 * * <tt>carrayref( std::vector<T> const& ) -> array_ref<T const></tt>
 * * <tt>carrayref( std::basic_string_view<C> ) -> array_ref<C const></tt>
 * 
 * */
#endif
