#ifndef CXXOMFORT_CXXO_MEMORY_SUPP_HPP
#define CXXOMFORT_CXXO_MEMORY_SUPP_HPP
#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>
#include <cxxomfort/memory.hpp>

namespace cxxomfort { namespace library {
} }


// 
// scoped_ptr (in cxxo:utils)
// 

#include <cxxomfort/cxxo-utils.hpp>
#include <cxxomfort/utils/scoped_ptr.hpp>
#include <cxxomfort/impl/functionmeta.hpp>

namespace cxxomfort { namespace library { namespace memory {
	using ::cxxomfort::scoped_ptr;
} } }
	#if (!defined(CXXO_LIB_scoped_ptr))
		//! @xrefitem cxxomfort-libraryfeaturemacros "" "" Announces implementation of @c scoped_ptr .
		#define CXXO_LIB_scoped_ptr 202202UL
	#endif

// 
// noexcept allocator
// from https://thephd.dev/freestanding-noexcept-allocators-vector-memory-hole
// 

namespace cxxomfort { namespace library { namespace memory {

template <typename T>
class noexcept_allocator {
public:
	typedef T 	value_type;
	typedef size_t 	size_type;
	typedef ptrdiff_t 	difference_type;
	typedef T* 	pointer;
	typedef T const* 	const_pointer;
	typedef T& 	reference;
	typedef T const& 	const_reference;
	
	typedef std::true_type 	is_always_equal;

	size_t 	max_size () const CXXO_NOEXCEPTNOTHROW {
		return (std::numeric_limits<size_t>::max() -1 )/sizeof(T); 
	}

	pointer 	allocate (size_t n) CXXO_NOEXCEPT {
		size_t byte_count = n * sizeof(T);
		::cxxomfort::impl::function_caller<void*(size_t) CXXO_NOEXCEPTNOTHROW, malloc> Malloc;
		void* ptr = Malloc(byte_count);
		if (ptr == nullptr) {
			// WHATEVER YOU WANT
			std::terminate();
		}
		return static_cast<T*>(ptr);
	}
	void 	deallocate (pointer ptr, size_t /*n*/) CXXO_NOEXCEPTNOTHROW {
		if (ptr == nullptr) return;
		free(ptr);
	} 
	
	#if (CXXOMFORT_CXX_STD >= 2011)
	template <typename... Args>
	void 	construct (pointer px, Args&&... args) {
		::new((void*)px) T(std::forward<Args>(args)...);
	}
	#else
	template <typename A1>
	void 	construct (pointer px, A1 a1) {
		::new ((void*)px) T(a1);
	}
	template <typename A1, typename A2>
	void 	construct (pointer px, A1 a1, A2 a2) {
		::new ((void*)px) T(a1,a2);
	}
	#endif

	void 	destroy (pointer px) const CXXO_NOEXCEPT {
		px->~T();
	}
	
	void 	validate_max (size_t c, size_t Max) CXXO_NOEXCEPTNOTHROW {
		if (c > Max) {
			fprintf(stderr, "noexcept_allocator::validate_max faill for allocator %p and size %ju\n", (void*)this, c);
			std::terminate();
		}
	}
	
	pointer 	address (reference v) const CXXO_NOEXCEPTNOTHROW {
		return std::addressof(v);
	}
	
	const_pointer 	address (const_reference v) const CXXO_NOEXCEPTNOTHROW {
		return std::addressof(v);
	}
	
	
	template <typename U> 
	struct rebind {
		typedef noexcept_allocator<U> other;
	};
	
}; // class

	#if (!defined(CXXO_LIB_noexcept_allocator))
		#if defined(DOXYGEN_DOC)
		//! @xrefitem cxxomfort-libraryfeaturemacros "" "" Announces implementation of @c noexcept_allocator .
		#define CXXO_LIB_noexcept_allocator YYYYMMUL
		#elif (CXXOMFORT_CXX_STD >= 2011)
		#define CXXO_LIB_noexcept_allocator 201103UL
		#else
		#define CXXO_LIB_noexcept_allocator 199703UL
		#endif
	#endif

} } }

#endif
