#ifndef CXXOMFORT_CXXO_RATIO_SUPP_HPP
#define CXXOMFORT_CXXO_RATIO_SUPP_HPP
#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>
#include <cxxomfort/ratio.hpp> // std::ratio

namespace cxxomfort { namespace supplemental {

    //! Convenience SI typedef
    typedef std::ratio<        1024ULL, 1> kibi;   // 1b10
    //! Convenience SI typedef
    typedef std::ratio<     1048576ULL, 1> mebi;   // 1b20 ~ 10e6
    //! Convenience SI typedef
    typedef std::ratio< mebi::num*1024, 1> gibi;   // 1b30 ~ 10e9
    //! Convenience SI typedef
    typedef std::ratio< gibi::num*1024, 1> tebi;   // 1b40 ~ 10e12
    //! Convenience SI typedef
    typedef std::ratio< tebi::num*1024, 1> pebi;   // 1b50 ~ 10e15
    //! Convenience SI typedef
    typedef std::ratio< pebi::num*1024, 1> exbi;   // 1b60 ~ 10e18


} }
	#define CXXO_LIB_iso80000_bi_prefixes 202208


#endif

/**
 * @page cxxo-sup-ratio Cxxomfort <ratio> Supplements
 *
 * @code
 * #include <cxxomfort/library/ratio.hpp>
 * // to avoid typing explosion
 * namespace stdplus = cxxomfort::supplemental;
 * @endcode
 *
 * @section Interfaces
 *  
 * This component provides supplementary features to the ones present in <code><random></code> that are specific to cxxomfort.
 *
 * * ISO80000 binary prefix types: @c kibi , @c mebi , @c gibi , @c tebi , @c pebi , @c exbi .
 *
 *
 * All interfaces are defined in the namespace 
 * <tt>cxxomfort::supplemental::</tt>.
 * 
 * 
 * @section also See Also
 * 
 * @see @ref std0ratio
 *
 */

