#ifndef CXXOMFORT_LIBRARY_TYPETRAITS_HPP
#define CXXOMFORT_LIBRARY_TYPETRAITS_HPP
/**
 * @file
 *
 * Interfaces defined in this file:
 *
 * * @c is_std_array
 * * @c is_std_basic_string
 * * @c is_std_byte
 * * @c is_std_pair
 * * @c is_tuple
 * * @c is_enumeration_like
 * * @c has_operator_brackets
 * * @c Ic and @link impl/ic_types.hpp integral_constant shortcuts @endlink
 *
 *
 * */
#include <cxxomfort/base.hpp>
#include <cxxomfort/type_traits.hpp>
#include <cxxomfort/tuple.hpp>
#include <cxxomfort/utility.hpp> // pair
#include <cxxomfort/cstddef.hpp> // byte
#include <cxxomfort/array.hpp> // array
#include "sequences.hpp"
#include "../impl/character.hpp"
#include "tuple.hpp"


namespace cxxomfort{ namespace library{
namespace type_traits{

using ::cxxomfort::impl::type_traits::is_character_type;

// 
// is_noexcept, add_noexcept
// 

#if (0 && CXXOMFORT_CXX_STD >= 2017)
template <typename T> struct is_noexcept : std::integral_constant< bool, noexcept(T)> {};
template <typename T> struct add_noexcept : std::conditional <
    is_noexcept<T>::value , T, T noexcept>::type
>::type {};

#else
template <typename T> struct is_noexcept : std::false_type {};
template <typename T> struct add_noexcept { enum { intentionally_empty }; }; // intentionally empty
#endif


/**
 * @brief type_trait that identifies @c std::byte .
 */
template <typename T> struct is_byte: std::false_type {};
template <> struct is_byte< std::byte >: std::true_type {};


// is_fundamental_wrapper


#if (CXXOMFORT_CXX_STD>=1997)
// using others from other facilities
using ::cxxomfort::library::is_std_array;
//using ::cxxomfort::library::string::is_std_basic_string;
using ::cxxomfort::library::tuple::is_tuple;
//using ::cxxomfort::library::is_typesafe_enum;
#endif

/**
 * @brief type_trait that identifies @c enum s , <code>enum class</code> es and typesafe_enums.
 */
/*
template <typename T> struct is_enumeration_like
: std::integral_constant<bool,
    std::is_enum<T>::value || is_typesafe_enum<T>::value
> {};
*/


#if (0) // UNTESTED

// is_specialization_of
// from  https://stackoverflow.com/questions/11251376/how-can-i-check-if-a-type-is-an-instantiation-of-a-given-class-template

#if (CXXOMFORT_CXX_STD>=2011)
template<typename T, template<typename...> class TT>
struct is_specialization_of : std::false_type { };

template<template<typename...> class TT, typename... Ts>
struct is_specialization_of<TT<Ts...>, TT> : std::true_type { };

#else

namespace detail {

template<typename T, template<typename,typename,typename=void> class TT>
struct is_specialization_of_3 : std::false_type { };

//template<typename T, template<typename,typename> class TT>
//struct is_specialization_of_3<T, TT> : std::false_type { };

template<template<typename,typename,typename> class TT, typename A1, typename A2, typename A3>
struct is_specialization_of_3<TT<A1,A2,A3>, TT> : std::true_type { };


template<typename T, template<typename,typename> class TT>
struct is_specialization_of_2 : std::false_type { };

template<template<typename,typename> class TT, typename A1, typename A2>
struct is_specialization_of_2<TT<A1,A2>, TT> : std::true_type { };


template<typename T, template<typename> class TT>
struct is_specialization_of_1 : std::false_type { };

template<template<typename> class TT, typename A1>
struct is_specialization_of_1<TT<A1>, TT> : std::true_type { };


}

#endif

#endif // is_specialization


}}}

/*
 * integral_constant shortcuts
 * */

#include "../impl/ic_types.hpp"

// fixes for is_trivially_constructible<void>

/*
#if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_GCC && CXXOMFORT_CXX_EMULATION == 2011)
namespace std {
    template <> struct is_trivially_copy_constructible<void> : std::false_type {};
    template <> struct is_trivially_move_constructible<void> : std::false_type {};
}
#endif
*/



#endif

/**
 * @page cxxo-sup-type_traits Cxxomfort <type_traits> Supplements
 * 
 * @code
 * #include <cxxomfort/library/type_traits.hpp>
 * namespace traits2 = cxxomfort::library::type_traits;
 * @endcode

 * 
 * This component provides supplementary features for the set of facilities present in <tt><type_traits></tt> that are specific to cxxomfort. Largely they include extended type traits to identify some types and elements not specified in traits in std, and centralizes in a single header traits that can also be found elsewhere.
 * 
 * Interfaces defined here:
 * 
 * 
 * All interfaces are defined in the namespace <tt>cxxomfort::library::type_traits::</tt>.
 * 
 * * @see @ref std0type_traits .
 * 
 */
