#ifndef CXXOMFORT_OSTREAM_HPP
#define CXXOMFORT_OSTREAM_HPP
/**
 * @file cxxomfort/ostream.hpp
 * @brief Implementations, Backports and helpers for Standard header <code>\<ostream\></code>.
 * @xrefitem std0ostream "<ostream>" "Header"
 * 
 * 
 */

#include <cxxomfort/base.hpp>
#include <cxxomfort/cxxo-utils.hpp>
#include <ostream>

#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
	#pragma message CXXO_NOTICE("enabled <ostream> support.")
#endif

/*
 * Outputs a tiny[u]int 
 * */

#if (CXXOMFORT_CXX_STD >= 1997)
namespace cxxomfort {
std::ostream& operator<< (std::ostream& os, tinyint const x) CXXO_NOEXCEPTNOTHROW {
	return os<< "i"<< (signed)to_value(x);
}
std::ostream& operator<< (std::ostream& os, tinyuint const x) CXXO_NOEXCEPTNOTHROW {
	return os<< "u"<< (unsigned)to_value(x);
}

}
#elif (0)
std::ostream& operator<< (std::ostream& os, cxxomfort::tinyint const x) CXXO_NOEXCEPTNOTHROW {
	return os<< (signed)cxxomfort::to_value(x);
}
std::ostream& operator<< (std::ostream& os, cxxomfort::tinyuint const x) CXXO_NOEXCEPTNOTHROW {
	return os<< (unsigned)cxxomfort::to_value(x);
}
#endif

/*
 * basic_string_view
 * */

#include <cxxomfort/string_view.hpp>

#if (CXXOMFORT_IMPLEMENTS_string_view >= CXXO_IMPLSTATUS_NATIVE())
	// nothing to do
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_DMC,>0))
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 2))
		#pragma message CXXO_NOTICE("ostream support for basic_string_view (iostream basic).")
	#endif
namespace std {

ostream& operator<< (ostream& os, std::string_view const v) {
    size_t const l = v.length();
    for (size_t i= 0; i != l; ++i) { os<< v[i]; }
    os.width(0);
    return os;
}

}
#elif (CXXOMFORT_IMPLEMENTS_string_view > 0 && CXXOMFORT_IMPLEMENTS_string_view < CXXO_IMPLSTATUS_NATIVE() )
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 2))
		#pragma message CXXO_NOTICE("ostream support for basic_string_view (C++98~C++17).")
	#endif

namespace cxxomfort { namespace cxxostd {

template <typename Ch, typename ChTraits>
std::basic_ostream<Ch, ChTraits>&
operator<<  (std::basic_ostream<Ch, ChTraits>& os, basic_string_view<Ch, ChTraits> const v) CXXO_NOEXCEPTNOTHROW {
    using namespace std;
    for (size_t i= 0; i < v.size(); ++i) { os<< v[i]; }
    os.width(0);
    return os;
}

} }

#else
#endif // cxx-std for operator<<


// 
// global print
// 

#if 0
namespace cxxomfort { 

struct print_t {
	std::ostream& os;
	print_t () CXXO_NOEXCEPTNOTHROW 
	: os(std::cout) 
	{ }
	
	explicit print_t (std::ostream& o) CXXO_NOEXCEPT 
	: os(o) 
	{ }
	
	#if (CXXOMFORT_CXX_STD >= 2011)
	template <typename T>
	std::ostream& operator() (T&& t) const {
		return os<< std::forward<T>(t);
	}
	#else
	template <typename T>
	std::ostream& operator() (T& const t) const {
		return os<< t;
	}
	
	#endif

	#if (CXXOMFORT_CXX_STD >= 2011)
	template <typename T1, typename... TArgs>
	std::ostream& operator() (T1&& t1, TArgs&&... targs) const {
		return os<< std::forward<T1>(t1) << this->operator()(std::forward<TArgs>(targs)...);
	}
	#else
	template <typename T1, typename T2>
	std::ostream& operator() (T1& const t1, T2 const& t2) const {
		return os<< t1<< t2;
	}
	template <typename T1, typename T2, typename T3>
	std::ostream& operator() (T1& const t1, T2 const& t2, T3 const& t3) const {
		return os<< t1<< t2<< t3;
	}
	template <typename T1, typename T2, typename T3, typename T4>
	std::ostream& operator() (T1& const t1, T2 const& t2, T3 const& t3, T4 const& t4) const {
		return os<< t1<< t2<< t3<< t4;
	}
	template <typename T1, typename T2, typename T3, typename T4, typename T5>
	std::ostream& operator() (T1& const t1, T2 const& t2, T3 const& t3, T4 const& t4, T5 const& t5) const {
		return os<< t1<< t2<< t3<< t4<< t5;
	}
	
	#endif
	

};

static const print_t print = {};

} // cxxomfort

#endif // print


// 
// global name for None_t
// 

namespace cxxomfort {

struct None_namer_t {
	std::string_view name;
};

static inline 
None_namer_t& 	None_namer () {
	static None_namer_t n;
	return n;
}

static inline 
std::string_view None_name () {
	return None_namer().name;
}

static inline 
void None_name (std::string_view s) {
	None_namer().name = s;
}

template <typename Char>
std::basic_ostream<Char>& 	operator<< (std::basic_ostream<Char>& os, ::cxxomfort::None_t const ) {
	return os<< None_name();
}

} // cxxomfort

// 
// 
// 

#endif

/**
 * @page std0ostream <ostream>
 * @brief Backports related to Standard Header <tt><ostream></tt>
 * 
 * @code
 * #include <cxxomfort/ostream.hpp>
 * @endcode
 * 
 * (or automatically by including <tt><cxxomfort/backports.hpp></tt> / <tt><cxxomfort/cxxomfort.hpp></tt>)
 *
 * @section Interfaces
 * 
 * This section does not (for now) implement features of its own; rather, it provides 
 * functionality for other sections that is separated from their implementation sections 
 * so as to avoid a direct dependency on C++ IOstreams when it is not needed.
 * 
 * Interfaces defined in this section:
 * 
 * * <tt>operator<< (ostream&, T const&)</tt> for the following types: 
 *   * <tt>unique_ptr<T></tt>
 *   * <tt>shared_ptr<T></tt>
 *   * @link stdbyte <tt>std::byte</tt> @endlink
 *   * @link cxxomfort::cxxostd::basic_string_view <tt>std::basic_string_view<Char_Type></tt> @endlink eg.: <tt>string_view</tt>
 * 
 *
 * @section also See Also
 * 
 * @see @cppref{header/ostream} (cppreference)
 * 
 * */
