#ifndef CXXOMFORT_STRING_HPP
#define CXXOMFORT_STRING_HPP
/**
 * @file 
 * @brief Implementations and additions tied to <tt>\<string\></tt>.
 * @xrefitem std0string "" ""
 */

#include <cxxomfort/base.hpp>
#include <string>

#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
    #pragma message CXXO_NOTICE("enabled <string> support.")
#endif

#undef  CXXOMFORT_IMPLEMENTS_string 
#define CXXOMFORT_IMPLEMENTS_string CXXO_IMPLSTATUS_NATIVE()

#if (CXXOMFORT_CXX_STD >= 1997)
	#include "cuchar.hpp"
	#include <cstring>

// 
// to_string 
// 

	#include "impl/11-basic_string_charn.hpp"
	#include "impl/11-to_string.hpp"

	#if (CXXOMFORT_CXX_STD < 2011 && CXXOMFORT_CXX_EMULATION < 2011)

    // GCC already has a specialization for string, wstring
		#if (CXXOMFORT_COMPILER_ID != CXXO_COMPILERID_GCC \
		&& CXXOMFORT_COMPILER_ID != CXXO_COMPILERID_CLANG \
		&& CXXOMFORT_COMPILER_ID != CXXO_COMPILERID_MSC  )
	// compilers that don't have hash<> 
		#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<1500))
		#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_OW,<9999))
		#elif (0)    
		#endif // gcc

	#endif // pre-2011


#else // CXX_STD < 1997

	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("<string> for compilers without full strings / iostreams")
	#endif
	//#include "impl/11-to_string.pre97.hpp"

// 
// missing operator<< under Watcom
//
	#if (0 && CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_OW)

		#include <ostream>

		#if 1
namespace std {
ostream& operator<< (ostream& os, string const& s) {
	for (unsigned long i= 0; i < s.length(); ++i) os<< s[i];
	return os;
}
} // std
        #endif

	#endif

#endif // CXX_STD

// 
// std::hash< std::basic_string >
// 

#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_COMPILER_SUPPORT_std_hash_basicstring)
    #undef  CXXOMFORT_IMPLEMENTS_stdhash_string
    #define CXXOMFORT_IMPLEMENTS_stdhash_string CXXO_IMPLSTATUS_NATIVE()
#elif (CXXOMFORT_CXX_STD >= 1997)
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("hash<basic_string> (1)")
	#endif
    #include <functional> // std::hash
    #include <cxxomfort/functional.hpp> // std::hash
    #undef  CXXOMFORT_IMPLEMENTS_stdhash_string
    #define CXXOMFORT_IMPLEMENTS_stdhash_string CXXO_IMPLSTATUS_BACKPORT()
namespace std {

    template<typename Char> 
    struct hash< basic_string<Char> > {
        typedef basic_string<Char> argument_type;
        typedef size_t result_type;
        
        result_type 
        operator()(argument_type const& s) const CXXO_NOEXCEPTNOTHROW {
            using namespace std;
            size_t const MS = (sizeof(size_t) - sizeof(Char) + 1) * CHAR_BIT;
            size_t r= s.length();
            size_t wi= 1;
            for ( size_t i= 0; i < s.length(); ++i, wi<<=1 ) {
                r ^= (static_cast<size_t>(s[i]) * wi);
                if (wi<<2 > MS ) { wi= 1; }
            }
            return r;
        }    
    };
    
} // std

#endif

// 
// std::hash< basic_string > 
// C++11 hash <u16string> , hash <u32string>
// 

#if (CXXOMFORT_CXX_STD>=2011)
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=406) && CXXOMFORT_CXX_EMULATION==2011)
#elif (CXXOMFORT_CXX_STD>=1997 /* && CXXO_COMPILER_SUPPORT_std_hash_basicstring==0 */)
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("hash for u16string, u32string")
	#endif
    #include <cxxomfort/functional.hpp>

namespace std {

template <> 
struct hash<std::u16string> {
	typedef std::u16string argument_type;
	typedef size_t result_type;
	result_type operator() (argument_type const& s) const CXXO_NOEXCEPTNOTHROW   {
		using namespace std;
		size_t const MS = (sizeof(size_t) - sizeof(char16_t) + 1) * CHAR_BIT;
		size_t r= s.length(), wi= 1;
		for ( size_t i= 0; i < s.length(); ++i, wi<<=1 ) {
			r ^= (static_cast<size_t>(s[i]) * wi);
			if (wi<<2 > MS ) { wi= 1; }
		}
		return r;
	}    
};

template<> 
struct hash< std::u32string > {
	typedef std::u32string argument_type;
	typedef size_t result_type;
	result_type operator()(argument_type const& s) const CXXO_NOEXCEPTNOTHROW   {
		using namespace std;
		size_t const MS = (sizeof(size_t) - sizeof(char32_t) + 1) * CHAR_BIT;
		size_t r= s.length(), wi= 1;
		for ( size_t i= 0; i < s.length(); ++i, wi<<=1 ) {
			r ^= (static_cast<size_t>(s[i]) * wi);
			if (wi<<2 > MS ) { wi= 1; }
		}
		return r;
	}    
};

}

#endif


// 
// "s" helper
// 

namespace cxxomfort { namespace fix {

CXXOMFORT_CXX11_CODE(inline, ) namespace string_helpers {

#if defined(DOXYGEN_DOC)
/**
 * @brief Creates a string from a character array or string literal.
 * @ingroup cxxomfort-fix
 * @xrefitem cxxomfort-fix "String Helpers" "String Helpers" Create a string from a character array or string literal.
 * 
 * This helper function supplements the <tt>""s</tt> user-defined literal helper pre C++14.
 * 
 * @code
 * using cxxomfort::fix::s;
 * s("Hello World"); // creates std::string
 * s(L"Hello World"); // creates std::wstring
 * s(u"Hello World") // C++11, creates std::u16string
 * @endcode
 * 
 * */
static const _implementation_defined_t_ s={};
	
#elif (CXXOMFORT_CXX_STD>=1997)

struct s_helper {
    template <size_t N>
    std::string operator() (char const (&str)[N]) const {
        return std::string(str);
    }
    template <size_t N>
    std::wstring operator() (wchar_t const (&str)[N]) const {
        return std::wstring(str);
    }

	#if (defined(__cpp_unicode_character_types))
	template <size_t N>
	std::u16string operator() (char16_t const (&str)[N] const {
		return std::u16string(str);
	}
	template <size_t N>
	std::u32string operator() (char32_t const (&str)[N] const {
		return std::u32string(str);
	}
	#endif
};

	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("s helper (obj)")
	#endif

CXXO_CONSTEXPR static const s_helper s={};
	
	#else

		#if (CXXOMFORT_NOTICES>2)
			#pragma message CXXO_NOTICE("s helper (funct)")
		#endif
template <size_t N>
std::string s (char const (&str)[N]) {
    return std::string(str);
}
template <size_t N>
std::wstring s (wchar_t const (&str)[N]) {
    return std::wstring(str);
}
	#endif

} // string_helpers
    
} } // cxxomfort::fix

// 
// "s" literal
// 

#if defined(DOXYGEN_DOC)
	/**
	 * @ingroup std00featuremacros
	 * @cppfeature{<tt>""s</tt> user-defined literal}
	 * */
	#define __cpp_lib_string_udls YYYY00L
#endif

#if (defined(__cpp_lib_string_udls) && __cpp_lib_string_udls>=201304)
	#undef  CXXO_COMPILER_SUPPORT_udl_s_basic_string
	#define CXXO_COMPILER_SUPPORT_udl_s_basic_string 1
    #undef  CXXOMFORT_IMPLEMENTS_udl_s
    #define CXXOMFORT_IMPLEMENTS_udl_s CXXO_IMPLSTATUS_NATIVE()
    
#elif defined(CXXOFLAG_NO_STD_USING)
#elif (CXXOMFORT_CXX_STD==2011 && CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,<800))
	// can't define UDLs not starting with underscore
#elif (CXXOMFORT_CXX_STD==2011 && CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,<600))
	// can't define UDLs not starting with underscore
#elif (CXXOMFORT_CXX_STD >= 2011 && CXXO_COMPILER_SUPPORT_udl_s_basic_string==0)
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("s UDL")
	#endif

/*  
 * note: this adds a UDL that is not preceded by '_' and thus reserved by the Standard. If the compiler emits a warning about this, it means the feature is working correctly!
 * 
 * */
namespace std { 
CXXOMFORT_CXX11_CODE(inline, /* */ ) namespace literals { 
CXXOMFORT_CXX11_CODE(inline, /* */ ) namespace string_literals {

    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=400))
        #pragma clang diagnostic push
        #pragma clang diagnostic ignored "-Wuser-defined-literals"
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=700))
        #pragma GCC diagnostic push
        #pragma GCC diagnostic ignored "-Wno-literal-suffix"
    #endif

std::string operator ""s (const char* str, std::size_t len) noexcept {
	return std::string(str,len);
}

std::wstring operator ""s (const wchar_t* str, std::size_t len) noexcept {
	return std::wstring(str,len);
}
	
	#if (defined(__cpp_unicode_literals))
std::u16string operator ""s (const char16_t* str, std::size_t len) noexcept {
	return std::u16string(str,len);
}
std::u32string operator ""s (const char32_t* str, std::size_t len) noexcept {
	return std::u32string(str,len);
}
	#endif

    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=700))
        #pragma GCC diagnostic pop
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=400))
        #pragma clang diagnostic pop
    #endif
} // string_literals
} } // std::literals
    
    #undef  CXXOMFORT_IMPLEMENTS_udl_s
    #define CXXOMFORT_IMPLEMENTS_udl_s CXXO_IMPLSTATUS_BACKPORT()
	#define __cpp_lib_string_udls (CXXOMFORT_CXX_STD*100)

#endif

// 
// "_s" literal (fix)
// 


#if (defined(__cpp_user_defined_literals))
namespace cxxomfort { namespace fix {
CXXOMFORT_CXX11_CODE(inline, ) namespace string_literals {

std::string operator "" _s (const char* str, std::size_t len) {
	return std::string(str,len);
}

std::wstring operator "" _s (const wchar_t* str, std::size_t len) {
	return std::wstring(str,len);
}

std::u16string operator "" _s (const char16_t* str, std::size_t len) {
	return std::u16string(str,len);
}

std::u32string operator "" _s (const char32_t* str, std::size_t len) {
	return std::u32string(str,len);
}

}
} } // cxxomfort::fix
#endif

// 
// C++20 free function interfaces for string
// 

#include "impl/string_helpers.hpp"
#include "impl/20-string_helpers.hpp"

#endif // file

/**
 * @page std0string <string>
 * @brief Backports related to Standard header <tt><string></tt>
 * 
 * 
 * @code
 * #include <cxxomfort/string.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * 
 * @section Interfaces
 * 
 * Interfaces defined in this section:
 * 
 * * @link cxxomfort::cxxostd::to_string <tt>to_string()</tt> @endlink (C++11) - convert C++ integers/floats into strings.
 * * Overloads include @c to_string(short) , @c to_string(long) , @c to_string(double) etc, for the compilers that missed some overloads (eg.: MSVC 2010).
 * * @c strtoll() , @c strtoull() function family (C++11) - convert strings into @c long s , for the compilers that missed them (MSVC).
 * * <tt>hash<></tt> specialization for @c std::basic_string where not available.
 * * @ref s_literal "User-Defined Literal Support" (C++14) - backports <tt>""s</tt> to C++11.
 * 
 * Interfaces not in std:
 * 
 * * @ref s_helper "String helper" , for systems where UDL is not available.
 * * @ref cxx20_nonmember_string "String nonmember helpers" , to supplement the new methods in C++20.
 * 
 * @section s_literal "s" user-defined literal
 * 
 * C++14 onwards defines <tt>operator"" s</tt> as the user-defined literal to create @c basic_string objects. 
 * The cxxomfort library backports the @c string and @c wstring capable versions to C++11, directly into namespace std (as <tt>std::literals::s</tt>).
 * 
 * @code
 * using namespace std::literals; // or using namespace cxxomfort::fix
 * auto s2 = L"Hello World"s; // is a std::wstring
 * @endcode
 * 
 * @warning The compiler might issue a warning <strong>or an error</strong> in C++11 mode that the literal <tt>""s</tt> is reserved. While this means the library can add the backport correctly, it could be that the compiler is blocking its use. It is compiler-dependent whether it is possible to disable this warning or error (see eg.: https://stackoverflow.com/questions/31509434/gcc-does-not-honor-pragma-gcc-diagnostic-to-silence-warnings ).
 * 
 * @note To try and bypass this error use one of the following options: 
 * * add the cxxomfort include directory to system headers eg.: <tt>-isystem</tt>.
 * * code intended to be cross-Standard compatible should be using <tt>cxxomfort::fix::s()</tt> instead (see the next section).
 * 
 * 
 * @section s_helper "s" helper
 * 
 * The helper function @link cxxomfort::fix::s s() @endlink creates a @c basic_string . It is provided to serve as a supplement to the "user-defined literal" <tt>""s</tt> where it is not available:
 * 
 * @code
 * using cxxomfort::fix::s; // or using namespace cxxomfort::fix
 * auto s1 = s("Hello World");
 * using namespace std::literals;
 * auto s2 = "Hello World"s;
 * static_assert( std::is_same< decltype(s1) , decltype(s2) >::value, "?");
 * @endcode
 * 
 * @section cxx20_string_members "basic_string nonmember helpers"
 * 
 * C++20 adds new member functions to <tt>basic_string</tt> and <tt>basic_string_view</tt>. Since it is not possible in C++ to extend a class with new methods, and the interface for basic_string is @e already crap (see []), cxxomfort implements them as nonmember accessors à-la-<tt><iterator></tt>.
 * 
 * For relevant documentation see @ref cxxo_nonmember_string_accessors .
 * 
 * @section also "See Also"
 * 
 * @see @cppref{header/string} (cppreference)
 * @see @ref std0string_view "string_view backports"
 * @see @ref cxxo_nonmember_string_accessors
 * @see @ref cxxo-sup-string
 * 
 *
 * */
