#ifndef CXXOMFORT_STRING_VIEW_HPP
#define CXXOMFORT_STRING_VIEW_HPP

#include <cxxomfort/base.hpp>

#if (defined(DOXYGEN_DOC))
	/**
	 * @ingroup std00featuremacros
	 * @cppfeature{ @ref std::basic_string_view }
	 * */
	#define __cpp_lib_string_view 0L

namespace std { 

namespace literals {
	/**
	 * @brief UDL for @c string_view s.
	 * @ingroup std0string_view
	 * @cxx17b User defined literal for <tt>basic_string_view</tt>.
	 * @xrefitem std0string_view "" "" User defined literal for <tt>basic_string_view</tt>.
	 * */
	template <typename Char>
	std::basic_string_view<Char> operator ""sv (Char const*, size_t n) {}
}}

#endif

#if (CXXO_CXXSTDHEADER_string_view)
	#include <string_view>
	#define CXXOMFORT_IMPLEMENTS_header_string_view CXXO_IMPLSTATUS_NATIVE()
	#define CXXOMFORT_IMPLEMENTS_string_view CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD >= 1997)
    #include <string>
	//#include "system_error.hpp"
	#include "impl/17-string_view.hpp"
	#define CXXOMFORT_IMPLEMENTS_header_string_view CXXO_IMPLSTATUS_BACKPORT()
	#define __cpp_lib_string_view (CXXOMFORT_CXX_STD*100)

namespace std {
    using cxxomfort::cxxostd::basic_string_view;
    using cxxomfort::cxxostd::string_view;
    using cxxomfort::cxxostd::wstring_view;
}


#else // C++ pre 97

	/* pre-C++98 compilers likely don't have a concept of basic_string and instead implement an individual "string" type (eg.: OpenWatcom, STLPort)
	 * */
	#include "impl/17-string_view.hpp"
	#define CXXOMFORT_IMPLEMENTS_header_string_view CXXO_IMPLSTATUS_EMULATION()
	#define __cpp_lib_string_view 1

namespace std {
    using cxxomfort::cxxostd::basic_string_view;
    using cxxomfort::cxxostd::string_view;
    using cxxomfort::cxxostd::wstring_view;
}

#endif

// 
// "sv" literal
// 

#ifdef CXXOFLAG_NO_STD_USING
#elif (CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view > 0)
    #undef  CXXOMFORT_IMPLEMENTS_udl_sv
    #define CXXOMFORT_IMPLEMENTS_udl_sv CXXO_IMPLSTATUS_NATIVE()
#elif (CXXOMFORT_CXX_STD==2011 && CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,<800))
	// can't define UDLs ot starting with underscore
#elif ( (CXXOMFORT_CXX_STD >= 2011 && CXXO_COMPILER_SUPPORT_user_defined_literals > 0) && CXXO_COMPILER_SUPPORT_udl_sv_basic_string_view==0)
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("sv UDL")
	#endif
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=400))
        #pragma clang diagnostic push
        #pragma clang diagnostic ignored "-Wuser-defined-literals"
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=700))
        #pragma GCC diagnostic push
        #pragma GCC diagnostic ignored "-Wno-literal-suffix"
    #endif

namespace std {
CXXOMFORT_CXX11_CODE(inline, /* */ ) namespace literals {
CXXOMFORT_CXX11_CODE(inline, /* */ ) namespace string_literals {

constexpr std::string_view 
operator "" sv (const char* str, std::size_t len) noexcept {
    return std::string_view(str,len);
}

constexpr std::wstring_view 
operator "" sv (const wchar_t* str, std::size_t len) noexcept {
    return std::wstring_view(str,len);
}

} // string_literals
} } // std::literals
    #if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=700))
        #pragma GCC diagnostic pop
    #elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=400))
        #pragma clang diagnostic pop
    #endif

    #undef  CXXOMFORT_IMPLEMENTS_udl_sv
    #define CXXOMFORT_IMPLEMENTS_udl_sv CXXO_IMPLSTATUS_BACKPORT()
    
#endif

// 
// sv helper
// 

#if (CXXOMFORT_CXX_STD >= 1997)
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("sv helper (1)")
	#endif

namespace cxxomfort { namespace fix {
CXXOMFORT_CXX11_CODE(inline, ) namespace string_helpers {

/**
 * @brief Creates a view from a character array.
 * @ingroup cxxomfort-fix
 * @xrefitemstd0string_view "" "" Creates a view from a character array
 * 
 * This helper function supplements the <tt>""sv</tt> user-defined literal helper pre C++17.
 * 
 * */
template <typename Char, size_t N>
CXXO_CONSTEXPR std::basic_string_view<Char, std::char_traits<Char> > 
sv (Char const(&arr)[N]) CXXO_NOEXCEPT {
    return std::basic_string_view<Char, std::char_traits<Char> > (arr, N-1); // "NULL" at the end is counted
}

/**
 * @brief Creates a view of a std::string.
 * @ingroup cxxomfort-fix
 * 
 * @note This is offered only for completeness to resolve ambiguities; in cxxomfort >= <tt>20200814</tt>, cxxomfort's <tt>basic_string_view</tt> can implicitly construct from a <tt>basic_string</tt> where required.
 * 
 * */
template <typename Char, typename CT>
std::basic_string_view<Char,CT> 
sv (std::basic_string<Char,CT> const& st) {
    return std::basic_string_view<Char,CT> (st.data(), st.size());
}

} // string_helpers
} } // cxxomfort::fix

#elif 1
	#if (CXXOMFORT_NOTICES>2)
		#pragma message CXXO_NOTICE("sv helper (2)")
	#endif
namespace cxxomfort { namespace fix { namespace string_helpers {

static inline std::string_view 
sv (std::string const& s) {
	return std::string_view(s.data(), s.size());
}

static inline std::wstring_view 
sv (std::wstring const& s) {
	return std::wstring_view(s.data(), s.size());
}

static inline std::string_view 
sv (char const* s) {
	return std::string_view(s);
}

static inline std::wstring_view 
sv (wchar_t const* s) {
	return std::wstring_view(s);
}

} // string_helpers
} }
#endif

// 
// "_sv" literal (fix)
// 

#if (defined(__cpp_user_defined_literals))
namespace cxxomfort { namespace fix {
CXXOMFORT_CXX11_CODE(inline, ) namespace string_literals {

CXXO_CONSTEXPR std::string_view 
operator "" _sv (const char* str, std::size_t len) noexcept {
    return std::string_view(str,len);
}

CXXO_CONSTEXPR std::wstring_view 
operator "" _sv (const wchar_t* str, std::size_t len) noexcept {
    return std::wstring_view(str,len);
}

}
}}
#endif

// 
// std::hash
// 

#if (CXXOMFORT_IMPLEMENTS_string_view>=CXXO_IMPLSTATUS_NATIVE())
	#undef  CXXOMFORT_IMPLEMENTS_stdhash_string_view
	#define CXXOMFORT_IMPLEMENTS_stdhash_string_view CXXO_IMPLSTATUS_NATIVE()
#elif (CXXOMFORT_IMPLEMENTS_string_view>=CXXO_IMPLSTATUS_BACKPORT())
	#undef  CXXOMFORT_IMPLEMENTS_stdhash_string_view
	#define CXXOMFORT_IMPLEMENTS_stdhash_string_view CXXO_IMPLSTATUS_BACKPORT()

	#include <cxxomfort/functional.hpp> // hash
namespace std {

/*
 * cppreference indicates that hash<basic_string_view> == hash<basic_string> 
 * since we can not see the implementation, all we can do is forward by creating the required basic_string from the basic_string_view, which incurs in allocation
 * */
template <typename Char, typename CT>
struct hash< std::basic_string_view<Char,CT> > 
: private hash <std::basic_string<Char,CT> >
{
    private:
    typedef hash <std::basic_string<Char,CT> > base_type;
    
    public:
    typedef std::basic_string_view<Char,CT> argument_type;
    typedef size_t result_type;
    result_type operator() (argument_type const& s) const CXXO_NOEXCEPT {
        // we are allowed to be noexcept because the only reasonable exception is out-of-mem
        return base_type()(basic_string<Char,CT>(begin(s),end(s)));
    }
    
};

} // std

#endif


// 
// string helpers
// 
    #if (CXXOMFORT_CXX_STD>=1997)
        #include <string>
        #include "impl/string_helpers.hpp"
        #include "impl/20-string_helpers.hpp"
        #include "impl/23-string_helpers.hpp"
    #endif

#endif

/**
 * @page std0string_view <string_view>
 * @brief Backports related to Standard Header <tt><string_view></tt> 
 * 
 * @code
 * #include <cxxomfort/string_view.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * #include <cxxomfort/ostream.hpp>
 * @endcode
 * 
 * @section Interfaces
 * 
 * Interfaces defined here:
 * 
 * * @link cxxomfort::cxxostd::basic_string_view @c basic_string_view @endlink - view to a string-like object (C++17).
 * * the related typedefs eg.: @c string_view , @c wstring_view .
 * * @c CXXOMFORT_IMPLEMENTS_stdheader_string_view - implementation status for the above.
 * * @ref sv_literal "User-Defined Literal Support" (C++17) - backports <tt>""sv</tt> to C++11 upto C++14.
 * 
 * For differences between the implementation of <tt>string_view</tt> 
 * here vs standard, check the notices at <tt> @link cxxomfort::cxxostd::basic_string_view basic_string_view @endlink </tt>
 * 
 * Fixes / Interfaces not in std:
 * 
 * * @ref sv_helper - see <tt>cxxomfort::fix::sv()</tt>
 * 
 * @section sv_literal "sv" user-defined literal
 * 
 * C++17 onwards defines <tt>operator"" sv</tt> as the user-defined literal to create @c basic_string_view objects. The cxxomfort library backports the @c string_view and @c wstring_view capable versions of this user-defined literal.
 * 
 * @code
 * using namespace std::literals; // or using namespace cxxomfort::fix
 * auto s2 = "Hello World"sv; // post-C++11 in cxxomfort, depending on compiler
 * @endcode
 * 
 * @warning The compiler might issue a warning <strong>or an error</strong> in C++11/14 mode that the literal <tt>""sv</tt> is reserved. While this means the library is functioning correctly in that it can add the literal operator, it is compiler-dependent whether it is possible to disable this warning or error (see eg.: https://stackoverflow.com/questions/31509434/gcc-does-not-honor-pragma-gcc-diagnostic-to-silence-warnings ).
 * 
 * @note To try and bypass this error use one of the following options: 
 * * add the cxxomfort include directory to system headers eg.: <tt>-isystem</tt>.
 * * code intended to be cross-Standard compatible should use @ref sv_helper instead (see the next section).
 * 
 * @section sv_helper "sv" helper
 * 
 * The helper function @link cxxomfort::fix::sv() sv() @endlink creates a @c basic_string_view . It is provided to serve as a supplement to the "user-defined literal" <tt>""sv</tt> where it is not available:
 * 
 * @code
 * using cxxomfort::fix::sv; // or using namespace cxxomfort::fix
 * auto s1 = sv("Hello World");
 * using namespace std::literals;
 * auto s2 = "Hello World"sv;
 * static_assert( std::is_same< decltype(s1) , decltype(s2) >::value, "?");
 * @endcode
 * 
 * @section cxx20_nonmember_string_view "basic_string_view nonmember helpers"
 * 
 * C++20 and C++23 add new member functions to <tt>basic_string</tt> and <tt>basic_string_view</tt>. Since it is not possible in C++ to extend a class with new methods, and the interface for basic_string is @e already crap (see []), cxxomfort implements them as nonmember accessors à-la-<tt><iterator></tt>.
 * 
 * For relevant documentation see @ref nonmember_string_accessors .
 * 
 * @section std0string_view_also "See Also"
 * 
 * * @cppref{header/string_view} , @ref @cppref{string/basic_string_view}
 * * @ref std0string "<string> backports"
 * 
 * 
 * */

// @include{doc} impl/17-string_view.hpp
