#ifndef CXXOMFORT_TUPLE_HPP
#define CXXOMFORT_TUPLE_HPP
/**
 * @file
 * @brief Implementations and backports related to <code><tuple></code>.
 * @ingroup std0tuple
 * 
 */

#include <cxxomfort/base.hpp>

#undef  CXXOMFORT_IMPLEMENTS_header_tuple
#define CXXOMFORT_IMPLEMENTS_header_tuple 0
#undef  CXXOMFORT_IMPLEMENTS_tuple 

#if (CXXOMFORT_CXX_STD>=2011)
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES > 2)
		#pragma message CXXO_NOTICE("tuple using C++11 <tuple>")
	#endif
	#include <tuple>
	#undef  CXXOMFORT_IMPLEMENTS_header_tuple
	#define CXXOMFORT_IMPLEMENTS_header_tuple CXXO_IMPLSTATUS_NATIVE()
	#define CXXOMFORT_IMPLEMENTS_tuple CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_EMULATION >= 2011 && CXXO_CXXSTDHEADER_tuple)
	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES > 2)
		#pragma message CXXO_NOTICE("tuple using c++0x <tuple>")
	#endif
	#include <tuple>
	#undef  CXXOMFORT_IMPLEMENTS_header_tuple
	#define CXXOMFORT_IMPLEMENTS_header_tuple CXXO_IMPLSTATUS_NATIVE()
	#define CXXOMFORT_IMPLEMENTS_tuple CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD>=1997 && CXXOFLAG_tr1)

	#include "utility.hpp" // index_sequence
	#if (CXXOFLAG_directory_tr1 && CXXO_CXXSTDHEADER_tr1_tuple)
		#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES > 2)
			#pragma message CXXO_NOTICE("tuple TR1 using <tr1/tuple>")
		#endif
		#include <tr1/tuple> // d'uh
	#else
		#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES > 2)
			#pragma message CXXO_NOTICE("tuple TR1 using <tuple>")
		#endif
		#include <tuple>
	#endif
	#undef  CXXOMFORT_IMPLEMENTS_header_tuple
	#define CXXOMFORT_IMPLEMENTS_header_tuple CXXO_IMPLSTATUS_BACKPORT()
	#define CXXOMFORT_IMPLEMENTS_tuple CXXO_IMPLSTATUS_NATIVE()

	#include <cxxomfort/array.hpp> // array<> ->  tuple<> conversions

	#if (defined(CXXOMFORT_NOTICES) && CXXOMFORT_NOTICES > 1)
		#pragma message CXXO_NOTICE("enabled <tuple> support")
	#endif

	// for TR1, we need to bring names into namespace std
	#if (CXXO_COMPILER_SUPPORT_tr1_tuple)
	namespace std {
		using tr1::tuple;
		using tr1::tuple_size;
		using tr1::tuple_element;
		using tr1::get;
		using tr1::tie;
		using tr1::ignore;
		using tr1::make_tuple;
		// using tr1::tuple_cat; // not available!

	}

	// backport tuple_cat
	namespace std {
		template <typename A1, typename B1> inline 
		tuple<A1,B1> tuple_cat (tuple<A1> a, tuple<B1> b) {
			return tuple<A1,B1>( get<0>(a), get<0>(b) );
		}

		template <typename A1, typename A2, typename B1> inline 
		tuple<A1,A2,B1> tuple_cat (tuple<A1,A2> a, tuple<B1> b) {
			return tuple<A1,A2,B1>( get<0>(a), get<1>(a), get<0>(b) );
		}
		template <typename A1, typename A2, typename A3, typename B1> inline 
		tuple<A1,A2,A3,B1> tuple_cat (tuple<A1,A2,A3> a, tuple<B1> b) {
			return tuple<A1,A2,A3,B1>( get<0>(a), get<1>(a), get<2>(a), get<0>(b) );
		}

		template <typename A1, typename B1, typename B2> inline 
		tuple<A1,B1,B2> tuple_cat (tuple<A1> a, tuple<B1,B2> b) {
			return tuple<A1,B1,B2>( get<0>(a), get<0>(b), get<1>(b) );
		}
		template <typename A1, typename B1, typename B2, typename B3> inline 
		tuple<A1,B1,B2,B3> tuple_cat (tuple<A1> a, tuple<B1,B2,B3> b) {
			return tuple<A1,B1,B2,B3>( get<0>(a), get<0>(b), get<1>(b), get<2>(b) );
		}

		template <typename A1, typename A2, typename B1, typename B2> inline 
		tuple<A1,A2,B1,B2> tuple_cat (tuple<A1,A2> a, tuple<B1,B2> b) {
			return tuple<A1,A2,B1,B2>( get<0>(a), get<1>(a), get<0>(b), get<1>(b) );
		}

	} // std
	

	#endif

#elif (defined(CXXOFLAG_USE_INTERNAL_TR1))
	#include <cxxomfort/impl/07-tuple.hpp>
	#pragma message CXXO_NOTICE("<tuple> cxxomfort internal TR1")
	#define CXXOMFORT_IMPLEMENTS_tuple CXXO_IMPLSTATUS_EMULATION()

namespace std {
	using ::cxxomfort::cxxostd::tuple;
	using ::cxxomfort::cxxostd::tuple_size;
	using ::cxxomfort::cxxostd::tuple_element;
	using ::cxxomfort::cxxostd::make_tuple;
	using ::cxxomfort::cxxostd::get;
	using ::cxxomfort::cxxostd::ignore;
	using ::cxxomfort::cxxostd::tie;
}

#else // CXX_STD < 1997, or without TR1
	#pragma message CXXO_NOTICE("<tuple> not supported")
	#define CXXOMFORT_IMPLEMENTS_tuple 0

#endif // CXX_STD

// 
// support stuff
// 

#if (CXXOMFORT_CXX_STD >= 1997 && CXXOMFORT_IMPLEMENTS_tuple>0)
	#include "impl/tuplemeta.hpp"
#endif

#include "impl/14-tuple_get_type.hpp" // std::get<T> (tuple<>)
#include "impl/17-make_from_tuple.hpp" // std::make_from_tuple<T>(tuple<>)
#include "impl/17-tuple_apply.hpp" // std::apply (Function, tuple<>)

#endif

/**
 * @page std0tuple <tuple>
 * @brief Backports related to Standard header <tt><tuple></tt> 
 * 
 * @code
 * #include <cxxomfort/tuple.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * @endcode
 * 
 * @section Interfaces
 * 
 * Interfaces defined here:
 * 
 * * @c tuple<> , @c tuple_size<> , @c tuple_element<> (C++03) when using cxxomfort's TR1 implementation.
 * * <tt>get<I>(tuple)</tt> (C++03) when using cxxomfort's TR1 implementation.
 * * <tt>tie()</tt> (C++03) when using cxxomfort's TR1 implementation. 
 * * @c get<tuple>() ( @ref std_get_type_tuple ) (C++14) - get elements of a tuple by type.
 * * @c apply() (C++17) - call a functor, unpacking a tuple for the arguments.
 * * @c make_from_tuple() (C++17) - construct an object unpacking a tuple for constructor arguments.
 * 
 * @section tuple_also See Also:
 * 
 * * @ref std0utility , @ref cxxo-sup-tuple ,
 * * @cppref{header/tuple}
 * 
 * 
 * 
 * */

// @include impl/17-make_from_tuple.hpp
