#ifndef CXXOMFORT_TYPE_TRAITS_HPP
#define CXXOMFORT_TYPE_TRAITS_HPP
/**
 * @file
 * @brief Implementations and backports related to <code><type_traits></code>.
 * @cxx1b 
 * @xrefitem std0type_traits "" ""
 * 
 */

#include <cxxomfort/base.hpp>
#include "impl/07-type_traits.hpp"

// 
// header 
// 

#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
    #pragma message CXXO_NOTICE("enabled <type_traits> support.")
#endif

#undef  CXXOMFORT_IMPLEMENTS_header_type_traits

#if (CXXOMFORT_CXX_STD >= 2011)
    #include <type_traits>
    #undef  CXXOMFORT_IMPLEMENTS_integral_constant
    #define CXXOMFORT_IMPLEMENTS_integral_constant CXXO_IMPLSTATUS_NATIVE()
    #define CXXOMFORT_IMPLEMENTS_header_type_traits CXXO_IMPLSTATUS_NATIVE()
    
#endif
#include "impl/lift_type_traits.hpp"
// bring in enable_if, conditional from C++11
#include "impl/11-conditional.hpp"

#if (CXXOMFORT_CXX_STD >= 2011)
    #include "impl/11-type_traits.hpp" // maybe supplement missing traits
#elif (CXXOMFORT_CXX_STD>=1997 && CXXO_COMPILER_SUPPORT_tr1_type_traits>0)
	#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 2))
		#pragma message CXXO_NOTICE("use compiler's <type_traits> if it can be found.")
	#endif

    #include "impl/11-type_traits.hpp"
    #include "impl/11-is_tn_xble.hpp"

#else // CXX_STD < 1997 / no <type_traits>
    #include "impl/11-type_traits.hpp"
    #include "impl/11-is_tn_xble.hpp"


#endif // CXX_STD

// search for potentially missing constructible/destructible traits
// 
#include "impl/11-is_tn_xble.hpp"

// patch missing is_trivially_... in GCC 4.8 C++11
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,<=408) && CXXOMFORT_CXX_STD>=2011)
    #include "impl/11-is_tn_xble.hpp"
#endif

#if (CXXOMFORT_CXX_STD>= 1997 && CXXOMFORT_CXX_STD<2011)
    #if (CXXOMFORT_CXX_EMULATION >= 2007)
    #else
    #endif
#endif


// 
// fix: to_integer for integral_constant 
// 

#if (CXXOMFORT_CXX_STD>=1997 || defined(DOXYGEN_DOC) )
/*
 * "to_integral", "to_bool" conversion helpers for 
 * integral_constant in the contexts where they don't feature the 
 * integral conversion operator (eg.: pre-C++11).
 * */

namespace cxxomfort { namespace fix {

/**
 * @brief Returns the value corresponding to a <tt>integral_constant</tt>.
 * @ingroup std0type_traits
 * @cxxo-utils Recovers the value corresponding to a <tt>integral_constant</tt>. 
 * 
 * This function is a helper for versions of C++ pre-C++14 which introduced the <tt>integral_constant</tt> conversion operator and function call.
 * 
 * @see @cppref{types/integral_constant}
 * 
 * */
template <class Ty, Ty u>
CXXO_CONSTEXPR inline 
Ty to_integer (std::integral_constant<Ty,u> const) CXXO_NOEXCEPTNOTHROW {
	return u;
}

/**
 * @brief Returns the value corresponding to a <tt>integral_constant</tt>.
 * @ingroup std0type_traits
 * @xrefitem cxxomfort-fixes "" ""  Value corresponding to a <tt>integral_constant</tt>. 
 * 
 * This function is a helper for versions of C++ pre-C++14 which introduced the <tt>integral_constant</tt> conversion operator and function call.
 * 
 * @see @cppref{types/integral_constant}
 * 
 * */
template <class Ty, Ty u> CXXO_CONSTEXPR inline 
Ty to_value (std::integral_constant<Ty,u> const) CXXO_NOEXCEPTNOTHROW {
	return u;
}


//! Helper function to return the value corresponding to a <tt>false_type</tt>.
//! @cxxo-utils Returns the value of a <tt>std::false_type</tt> (see also @c to_integer() )
CXXO_CONSTEXPR static inline 
bool to_bool (std::false_type) CXXO_NOEXCEPTNOTHROW { return false; }
//! Helper function to return the value corresponding to a <tt>true_type</tt>.
//! @cxxo-utils Returns the value of a <tt>std::true_type</tt> (see also @c to_integer() )
CXXO_CONSTEXPR static inline 
bool to_bool (std::true_type) CXXO_NOEXCEPTNOTHROW { return true; }


} } // cxxomfort::fix

	#define CXXO_IC_value(ICx) ICx.value
#else
	#define CXXO_IC_value(ICx) ICx.value
#endif


#if (CXXOMFORT_CXX_STD>=1997)


    /*
     * remove_cvref, from c++20
     * */

    #if (CXXOMFORT_CXX_STD>=2020)
        #define CXXOMFORT_IMPLEMENTS_p0550 CXXO_IMPLSTATUS_NATIVE()
    #else
        #define CXXOMFORT_IMPLEMENTS_p0550 CXXO_IMPLSTATUS_BACKPORT()
namespace std {
	using ::cxxomfort::traits::remove_cvref;
}

    #endif

// #define CXXOMFORT_IMPLEMENTS_std_remove_cvref CXXOMFORT_IMPLEMENTS_p0550


/*
 * TR1 traits missing in c++03 in implementations
 * is_literal_type, is_trivial, etc
 */


    #if 0
    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_GCC)

        #if (!defined(__GXX_EXPERIMENTAL_CXX0X__) )
            #define CXXO_LACKS_make_signed 1
        #endif
    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_CLANG)
        #if (CXXOMFORT_CXX_STD < 2011)
            #define CXXO_LACKS_make_signed 1
        #endif
    #elif (CXXOMFORT_COMPILER_ID == CXXO_COMPILERID_MSC)
        #if (CXXOMFORT_COMPILER_VERSION < 1400)
            #define CXXO_LACKS_make_signed 1
        #endif
    #else
    #endif

#if defined(CXXO_LACKS_make_signed)
namespace std { namespace tr1 {
    using cxxomfort::traits::make_signed;
    using cxxomfort::traits::make_unsigned;
} }
#endif // bring missing make_signed, make_unsigned to tr1

//
// lvalue_reference traits mirrored in C++03 mode
//

#if (CXXOMFORT_CXX_STD < 2011 && CXXOMFORT_CXX_EMULATION < 2011)
    #define CXXO_MIRROR_LVALUE_REFERENCE 1
#else
    #if (defined(__GNUC__) && !defined(__GXX_EXPERIMENTAL_CXX0X__))
        namespace std { using namespace tr1; }

        #define CXXO_MIRROR_LVALUE_REFERENCE 1
    #endif
#endif
#if (defined(CXXO_MIRROR_LVALUE_REFERENCE))
namespace std {
    template <typename T> struct is_lvalue_reference
    : cxxomfort::traits::is_reference<T> {};

    template <typename T>
    struct add_lvalue_reference
    : cxxomfort::traits::add_reference<T> {};

    template <typename T>
    struct remove_lvalue_reference
    : cxxomfort::traits::remove_reference<T> {};


    template <typename T>
    struct is_rvalue_reference { static const bool value = false; };

}
#endif
#undef CXXO_MIRROR_LVALUE_REFERENCE



/*
 * remove_all_extents (C++11)
 * */
#if (CXXOMFORT_CXX_STD>=2011)
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC, >= 1600 )) // MSVC 2010 onwards defines remove_all_extents
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >= 406 ) && CXXOMFORT_CXX_EMULATION==2011)
#else
namespace std {
	using cxxomfort::traits::remove_all_extents;
}
#endif

/*
 * traits from C++14
 * * is_final
 * * is_null_pointer_constant
 * */
#include "impl/14-type_traits.hpp"

/*
 * traits from C++17
 * * bool_constant
 * * make_void
 * * void_t
 */
#include "impl/17-type_traits.hpp"

namespace cxxomfort { namespace fix {
template <bool B> CXXO_CONSTEXPR inline 
bool to_value (std::bool_constant<B> const) CXXO_NOEXCEPTNOTHROW { 
	return B;
}

}}

// invoke_result
#include "impl/17-invoke.hpp"

/*
 * endian (c++20)
 */
#include "impl/20-type_traits.hpp"
#include "impl/20-endian.hpp"


//
// is_trivially_... missing in GCC 4.7 despite announcing C++11 mode
//

#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC , < 404) && CXXOMFORT_CXX_EMULATION==2011)
namespace std {
    template <typename T> struct is_trivially_default_constructible
    : integral_constant<bool, __has_trivial_constructor(T)> {};

    template <typename T> struct is_trivially_copy_constructible
    : integral_constant<bool, __has_trivial_copy(T)> {};

    template <typename T> struct is_trivially_move_constructible
    : integral_constant<bool, is_scalar<T>::value > {};

    template <typename T> struct is_trivially_copy_assignable
    : integral_constant<bool, __has_trivial_assign(T)> {};

    template <typename T> struct is_trivially_destructible
    : integral_constant<bool, __has_trivial_destructor(T)> {};

}
#endif
#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC , == 407) && CXXOMFORT_CXX_EMULATION==2011)
namespace std {
    template <typename T> struct is_trivially_destructible
    : integral_constant<bool, __has_trivial_destructor(T)> {};
}
#endif    

// is_...._move_constructible - for C++03
	#if (CXXOMFORT_CXX_STD > 1997)
		#include "impl/11-move_traits.hpp"
	#endif

#if 0 // -- is_constructible
//
// "variadic" is_constructible, is_assignable implementation
//

//#include "impl/is_constructible.hpp"


namespace cxxomfort {
namespace detail {

    template<std::size_t N>
    struct dummy;

    typedef struct { char a; } yes_t;
    typedef struct { char b[2]; } no_t;


#if defined(_MSC_VER)
// MSVC does not support the SFINAE required for the constructor expressions
// to work (error C2564 function-style conversion takes only one argument)
// so for MSVC we just make the is_constructible traits return false
// for all >1-argument cases.
// At most what we can do is to provide a couple of specializations,
// given we are implementors and thus are not limited by
// the specializations rule on namespace std.

#define CONSTRUCTIBLE_IMPL_FORM_BEGIN(...) \

#define CONSTRUCTIBLE_IMPL_FORM_END(...) \
    public:           \
    static const bool value = sizeof(test< __VA_ARGS__ >(0)) \
    == sizeof(yes_t); \


template <typename T, typename A1, typename A2, typename A3, typename A4, typename A5>
struct constructible5 : false_type {};

template <typename T, typename A1, typename A2, typename A3, typename A4>
struct constructible4 : false_type {};

template <typename T, typename A1, typename A2, typename A3>
struct constructible3 : false_type {};

template <typename T, typename A1, typename A2>
struct constructible2 : false_type {};

template<typename T, typename A1>
struct constructible1 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN();

    template<typename Tp1, typename Arg1_>
    static typename
    std::enable_if<(sizeof(Tp1( declval<Arg1_>()), 1) > 0), yes_t>::type test(int);

    template<typename, typename>
    static no_t test(...);

    CONSTRUCTIBLE_IMPL_FORM_END(T,A1);
};

template <typename T>
struct constructible0 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN();

    template<typename Tp1>
    static typename
    std::enable_if<(sizeof(Tp1(), 1) > 0), yes_t>::type test(int);

    template<typename, typename>
    static no_t test(...);

    CONSTRUCTIBLE_IMPL_FORM_END(T);
};


template<typename T, typename Arg1>
struct assignable {
    //typedef typename std::add_const<Arg1>::type A1Const;

    template<typename Tp1, typename Arg1_>
    static typename
    std::enable_if<(sizeof(declval<Tp1>()= declval<Arg1_>(), 1) > 0), yes_t>::type test(int);

    template<typename, typename>
    static no_t test(...);

    public:
    static const bool value = sizeof(test<T, Arg1>(0)) == sizeof(yes_t);
};

#else
// not on MSVC

#define CONSTRUCTIBLE_IMPL_FORM_BEGIN()          \
    template <typename X>                        \
    static yes_t                                 \
    test(dummy<sizeof(     \

#define CONSTRUCTIBLE_IMPL_FORM_END()            \
    )>*);                                        \
                                                 \
    template<typename X>                         \
    static no_t                                  \
    test(...);                                   \
                                                 \
    static const bool value = sizeof(            \
        test<T>(0)) == sizeof(yes_t);               \
    typedef ::cxxomfort::traits::                \
        integral_constant<bool,value> type;      \


template <typename T, typename A1, typename A2, typename A3, typename A4, typename A5>
struct constructible5 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    (X) ( std::declval<A1>(), std::declval<A2>(), std::declval<A3>(), std::declval<A4>(), std::declval<A5>() )
    CONSTRUCTIBLE_IMPL_FORM_END();
};

template <typename T, typename A1, typename A2, typename A3, typename A4>
struct constructible4 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    (X) ( std::declval<A1>(), std::declval<A2>(), std::declval<A3>(), std::declval<A4>() )
    CONSTRUCTIBLE_IMPL_FORM_END();
};

template <typename T, typename A1, typename A2, typename A3>
struct constructible3 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    (X) ( std::declval<A1 const>(), std::declval<A2 const>(), std::declval<A3 const>() )
    CONSTRUCTIBLE_IMPL_FORM_END();
};

template <typename T, typename A1, typename A2>
struct constructible2 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    X ( std::declval<A1 const>(), std::declval<A2 const>() )
    CONSTRUCTIBLE_IMPL_FORM_END();
};

template <typename T, typename A1>
struct constructible1 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    X ( std::declval<A1 const>() )
    CONSTRUCTIBLE_IMPL_FORM_END();
};

template <typename T, typename U>
struct assignable {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    std::declval<X>() = std::declval<X>()
    CONSTRUCTIBLE_IMPL_FORM_END()
};

#if (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_GCC && CXXOMFORT_COMPILER_VERSION<406)
template <typename U> struct assignable<void,U> {
    static const bool value= false;
};
#endif

template <typename T>
struct constructible0 {
    CONSTRUCTIBLE_IMPL_FORM_BEGIN()
    X
    CONSTRUCTIBLE_IMPL_FORM_END();
};



#endif



#undef CONSTRUCTIBLE_IMPL_FORM_BEGIN
#undef CONSTRUCTIBLE_IMPL_FORM_END

} } // cxxomfort::

#endif // -- is_constructible

// 
// traits from here on require at least C++03 TR1 <type_traits> support
// 


// is_destructible
#if (CXXOMFORT_CXX_STD >= 2011) \
|| CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1700)
#elif (CXXO_COMPILER_SUPPORT_tr1_type_traits)
namespace std {

    template <typename T>
    struct is_destructible
    : integral_constant<bool, 
    (is_pod<T>::value || is_class<T>::value) && !is_void<T>::value > 
    {};

}
#else
namespace std {
	template <typename T>
	struct is_destructible 
	: integral_constant<bool, (is_fundamental<T>::value && !(is_void<T>::value))> {} ;
}

#endif




#else // CXX_STD < 1997

	#if (0 && CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_OW)
		#include <cxxomfort/config/ow/type_traits>
	#endif

	#include <cxxomfort/impl/07-type_traits.hpp>
	
namespace std {
	using namespace cxxomfort::traits;
}


    #endif // CXX_STD >= 1997

// 
// is_scoped_enum (C++23)
// 

#if (defined(__cpp_lib_is_scoped_enum))
#elif (CXXOMFORT_CXX_STD >= 2011 || CXXO_COMPILER_SUPPORT_enum_class)
namespace cxxomfort { namespace cxxostd { 
namespace impl {

template <typename Enum>
struct _is_scoped_enum_impl 
: std::integral_constant<bool, 
(!std::is_convertible<Enum, typename std::underlying_type<Enum>::type>::value)
> {};

} 
} }

namespace std {
	template <typename T>
	struct is_scoped_enum
	: std::conditional< std::is_enum<T>::value 
	, ::cxxomfort::cxxostd::impl::_is_scoped_enum_impl<T>, std::false_type>::type 
	{ };
}
#else
	// no enum-class support
namespace std {
	template <typename> struct is_scoped_enum : public false_type {};
}
#endif


// 
// cxxomfort extension for eg.: is_enum
// 

#include <cxxomfort/cxxo-utils.hpp>

namespace cxxomfort {
template <typename E>
struct enumeration_underlying_type <E, typename ::cxxomfort::traits::enable_if<std::is_enum<E>::value>::type > 
: std::underlying_type<E> {};

}

#endif // #include guard

/**
 * @page std0type_traits <type_traits>
 * @brief Backports related to Standard Header <tt><type_traits></tt>
 * 
 * @code
 * #include <cxxomfort/type_traits.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * #include <cxxomfort/tuple.hpp>
 * #include <cxxomfort/library.hpp>
 * @endcode
 * 
 * 
 * @section Interfaces
 * 
 * Interfaces provided by this header include:
 * 
 * * The family of basic traits defined in C++ TR1, if they were not already available. Eg.: @cppref{is_integer} , @cppref{add_const} .
 * * @c is_lvalue_reference - brought to C++03 as a mirror to @c is_reference (C++11).
 * * @c is_rvalue_reference - identifies rvalue references (always @e false in C++03).
 * * @ref @c std::decay "decay<T>" - perform implicit decay on a type.
 * * @ref @c std::common_type  - obtains the type that a list of types decays or converts to (see note below).
 * * @ref @c std::underlying_type "underlying_type<Enum>" - underlying integral for an @c enum .
 * * @e is_trivially_ family of traits.
 * * @e is_nothrow_ family of traits.
 * * @ref @c is_trivial , @ref @c is_literal_type (C++11).
 * * @ref @c is_null_pointer (C++14) - identifies @c std::nullptr_t .
 * * @ref @c void_t , @ref @c make_void - generic, type-absorbing @c void transformations (C++17).
 * * @ref @c cxxomfort::cxxostd::bool_constant "std::bool_constant"  (C++17).
 * * @c @ref remove_cvref (C++20).
 * 
 * Non-backport interfaces (<tt>cxxomfort::fix</tt>):
 * 
 * * <tt>to_value()</tt> overloads to check the value of a type_trait object: see @ref typetrait_accessors .
 * * <tt>to_bool()</tt> for <tt>false_type</tt>, <tt>true_type</tt> (see above).
 * * @link cxxomfort::fix::common_type @c common_type @endlink - 
 * implements "variadic" @c common_type in MSVC 2010, MSVC 2012 where it has issues, or refers to @c std::common_type otherwise.
 * 
 * @section typetrait_emulation type_traits emulations
 * 
 * The implementation of every type_traits helper utility in Cxxomfort depends on what is detected about the capabilities of the compiler.
 * 
 * Several type traits refer to information that can not be computed by the language but has to be provided by "compiler magic". Usually those traits refer to a compiler intrinsic with a similar name: <tt>is_pod<T></tt> refers to something like <tt>__is_pod(T)</tt> provided magically by the compiler.
 * 
 * If possible, cxxomfort will supplement a missing type_trait with one invoking the corresponding intrinsic if that is found. An example is @c __is_final in MSVC.
 * 
 * Other type traits refer to information that can often times be computed in the version of the Standards where they appear, but for which intrinsics with incomplete or different implementation was found in earlier Standards. An example is the triviality traits (@c is_trivially_copy_constructible , etc) which pre-C++11 would rely on an intrinsic (like @c __has_trivial_copy ) that has different meaning and capabilities.
 * 
 * In those cases, cxxomfort tries to provide a barebones implementation that invokes the intrinsic when found; if not found, or if the behaviour is too different, a simpler approximation that at least provides no false positives is implemented instead. For example, in pre-C++11 compilers with the @c __has_trivial_copy(T) intrinsic @c is_trivially_copy_constructible uses it, but in certain patological cases or if not found, @c is_trivially_copy_constructible will instead check if the type applies for @c is_scalar .
 * 
 * A quick summary of the simpler approximations, based on information from cppreference, is provided here:
 * 
 * <table><tr>
 * <th>Type Trait</th><th>Intrinsic if found</th><th>Alternative Approximation</th></tr>
 * <tr>
 * <th><tt>is_trivially_xxx_constructible</tt></th> <td><tt>__has_trivial_xxx, __has_trivial_constructor</tt></td> <td><tt>is_fundamental</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_trivially_copy_assignable</tt></th> <td><tt>__has_trivial_assign</tt></td> <td><tt>is_fundamental</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_trivially_destructible</tt></th> <td><tt>__has_trivial_destructor</tt></td> <td><tt>is_fundamental && !is_void</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_nothrow_xxx_constructible</tt></th> <td><tt>__has_nothrow_xxx</tt></td> <td><tt>is_scalar</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_standard_layout</tt></th> <td><tt>__is_standard_layout</tt></td> <td><tt>is_scalar</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_trivial</tt></th> <td><tt>__is_trivial</tt></td> <td><tt>is_scalar || (is_class && is_trivially_default_constructible && is_trivially_default_copyable && is_trivially_destructible)</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_literal_type</tt></th> <td><tt>__is_literal_type</tt></td> <td><tt>is_scalar || is_reference</tt></td>
 * </tr>
 * <tr>
 * <th><tt>has_virtual_destructor</tt></th> <td><tt>__has_virtual_destructor</tt></td> <td><tt>false_type</tt></td>
 * </tr>
 * <tr>
 * <th><tt>is_final</tt></th> <td><tt>__is_final</tt></td> <td><tt>false_type</tt></td>
 * </tr>
 * </table>
 * 
 * Other type traits refer to information that changes meaning between versions of the Standard and that is not easily, or at all, computable without "compiler magic". In those cases, cxxomfort simply does not provide a supplement. An example is C++17's @c is_aggregate , as the definition of an Aggregate has changed in @e every version of the Standard and even if a @c __is_aggregate(T) was found, it would only give a proper meaning for the exact compiler and Standards version where it was found.
 * 
 * 
 * @section typetrait_accessors type_traits accessors
 * 
 * cxxomfort provides for <tt>std::integral_constant</tt> and its related types, including the various type traits implemented via it, the following special functions to check the value of an @c integral_constant or derived object generically. This is intended to supplement the implicit conversion operator and the @c .value() member introduced in later versions of the Standard.
 * 
 * @code
 * // The following invocations are equivalent:
 * 
 * is_scalar<float>::value;  
 * is_scalar<float>{}.value();               // C++11 onward
 * static_cast<bool>(is_scalar<float>{});    // C++14 onward
 * cxxomfort::fix::to_value(is_scalar<float>()); // cxxomfort
 * 
 * // The following invocations are equivalent:
 * 
 * typedef std::array<int, 4> Ar;
 * rank<Ar>::value; 
 * rank<Ar>{}.value(); // C++11 onward
 * static_cast<size_t>(rank<Ar>{}); // C++14 onward
 * using cxxomfort::fix::to_value;
 * to_value(rank<Ar>());  // cxxomfort
 * 
 * // the following is valid only for true_type, false_type
 * 
 * using cxxomfort::fix::to_bool;
 * to_bool(is_scalar<float>()); // ok, is a integral_constant of bool
 * to_bool(rank<A>()); // fails, is an integral_constant of integer type
 * 
 * @endcode
 * 
 * @section Notes 
 * 
 * @attention General notes:
 * 
 * * A number of traits depend on <b>compiler intrinsics</b>, including @c is_empty and @c is_pod . If they are not available, the library does not compile correctly.
 * * In compilers without at least @c \_\_typeof__ support, @c common_type may not resolve correctly.
 * * In older compilers @c underlying_type may not resolve correctly.
 * 
 * @section also See Also
 * 
 * * @see @ref @cppref{header/type_traits}
 * * See also @link cxxo-sup-type_traits Supplemental type_traits @endlink for other traits and transformations available in cxxomfort.
 * 
 */
