#ifndef CXXOMFORT_TYPE_INDEX_HPP
#define CXXOMFORT_TYPE_INDEX_HPP
/**
 * @file
 * @brief Implementations and backports tied to Standard Header <code><typeindex></code>.
 * 
 * Interfaces defined in this file:
 * 
 * * @c type_index (C++11)
 * * @c hash<type_index> (C++11) (for C++03 only)
 *
 * @see https://en.cppreference.com/w/cpp/header/typeindex
 * 
 */

#include <cxxomfort/base.hpp>

#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_CXXSTDHEADER_typeindex>0)
    #include <typeindex>
    #undef  CXXOMFORT_IMPLEMENTS_typeindex 
    #define CXXOMFORT_IMPLEMENTS_typeindex CXXO_IMPLSTATUS_NATIVE()

#elif (CXXOMFORT_CXX_STD>=1997)

	#include <cxxomfort/impl/11-typeindex.hpp>
	#undef  CXXOMFORT_IMPLEMENTS_typeindex
	#define CXXOMFORT_IMPLEMENTS_typeindex CXXO_IMPLSTATUS_BACKPORT()

    #if (defined( CXXOFLAG_NO_STD_USING ) && CXXOFLAG_NO_STD_USING>0)
    #else

namespace std {
    using ::cxxomfort::cxxostd::type_index;
}
	#endif // no_std_using

#include <functional> // hash

#else // CXX_STD < 1997
	#include <cxxomfort/impl/11-typeindex.hpp>
	#undef  CXXOMFORT_IMPLEMENTS_typeindex
	#define CXXOMFORT_IMPLEMENTS_typeindex CXXO_IMPLSTATUS_EMULATION()

    #if (defined( CXXOFLAG_NO_STD_USING ) && CXXOFLAG_NO_STD_USING>0)
    #else

namespace std {
    using ::cxxomfort::cxxostd::type_index;
}
	#endif // no_std_using

#endif


#if (CXXOMFORT_CXX_STD >= 2011 || CXXO_CXXSTDHEADER_typeindex>0)
	// already has specialization for std::hash
#elif (CXXOMFORT_CXX_STD>=1997)
CXXO_STDTR1_OPEN() 
    template<>
    struct hash< type_index > {
        typedef type_index argument_type;
        typedef size_t result_type;
        result_type operator()(argument_type const& s) const CXXO_NOEXCEPTNOTHROW   {
            //cout<< "[hash<type_index> for "<< s.name()<< "]"<< flush;
            return s.hash_code();
        }    
    };
    
CXXO_STDTR1_CLOSE()
#endif // no_std_using

#endif

/**
 * @page std0typeindex <typeindex>
 * @brief Backports related to Standard header <tt>\<typeindex\></tt> 
 * 
 * @code
 * #include <cxxomfort/typeindex.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * #include <cxxomfort/library.hpp>
 * @endcode
 * 
 * @section Interfaces 
 * 
 * From C++11 onwards, <tt><typeindex></tt> provides an utility that 
 * allows using <tt>type_info</tt> information in a way that is indexable 
 * as criteria for eg.: a <tt>std::map</tt>.
 * 
 * Interfaces defined in this section:
 * 
 * * @c std::type_index - Hashable form of @c <tt>std::type_info</tt> (see @c @ref cxxomfort::cxxostd::typeindex "typeindex" ).
 * * <tt>std::hash<type_index></tt> - for C++03 only.
 * 
 * @section also See Also
 * 
 * * @see @cppref{header/typeindex} 
 * 
 */

