#ifndef CXXOMFORT_UTILITY_HPP
#define CXXOMFORT_UTILITY_HPP
/**
 * @file
 * @brief Implementations related to Standard Header <tt>utility</tt>.
 * @ingroup std0utility
 * @xrefitem std0utility "<utility>" "Header"
 * 
 */

#include <cxxomfort/base.hpp>
#include <utility>

#if (defined(CXXOMFORT_NOTICES) && (CXXOMFORT_NOTICES > 1))
	#pragma message CXXO_NOTICE("enabled <utility> support.")
#endif

#include "impl/07-type_traits.hpp"


// 
// declval (C++11)
// 

#if defined(DOXYGEN_DOC)
namespace std {
	/**
	 * @cxx11b Creates an unevaluated object.
	 * @xrefitem std0utility "" "" Creates an unevaluated object.
	 * */
	template <typename T> T& declval() {}
}
#elif (CXXOMFORT_CXX_STD >= 2011 || CXXOMFORT_CXX_EMULATION >= 2011)
#else
	#if (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,<170))
		#define CXXO_ADD_declval 
	#endif
	#if (CXXO_COMPILERREQ(CXXO_COMPILERID_GCC, >0) && CXXOMFORT_CXX_EMULATION<2011)
		#define CXXO_ADD_declval
	#elif (CXXOMFORT_CXX_EMULATION < 2011)
		#define CXXO_ADD_declval
	#endif

	#if defined(CXXO_ADD_declval)
namespace std { 
    template <typename T>  
    typename cxxomfort::traits::add_reference<T>::type 
    declval();
} //std::
		#undef CXXO_ADD_declval
	#endif

#endif


// 
// exchange (C++14)
// 
#include "impl/n3668-exchange.hpp"

// 
// integer_sequence, etc (C++14)
// 
#include "impl/14-integer_sequence.hpp"

// 
// as_const (C++17)
// 
#include "impl/17-as_const.hpp"


// 
// in_place_t, etc (C++17)
// 

#if defined(DOXYGEN_DOC)
namespace std {
	//! @cxx11b Tag type for in-place construction.
	//! @xrefitem std0utility "" "" Tag type for in-place construction.
	struct in_place_t {};
	//! Tag element
	constexpr const in_place_t in_place = {};
}
#elif (defined( CXXOFLAG_NO_STD_USING ) && CXXOFLAG_NO_STD_USING>0)
#elif (CXXOMFORT_CXX_STD>=2017)
#elif (defined(__cpp_variable_templates))
	// with inline variables we can use the definitions from cppreference

namespace std {
	
	struct in_place_t {
		explicit in_place_t() = default;
	};
	constexpr const in_place_t in_place {};

	template <typename T> struct in_place_type_t {
		explicit in_place_type_t() = default;
	};
	template <typename T>
	constexpr const in_place_type_t<T> in_place_type {};

} // std

#else
	#define CXXO_ADD_inplace 1

#endif

#ifdef CXXO_ADD_inplace

// c++<14 does not have inline variables so we use the 
// overloaded function name reference from 
// https://vector-of-bool.github.io/2016/10/30/standard-in-place.html

namespace cxxomfort { namespace cxxostd { namespace impl {
	template< typename T >
	struct in_place_type_t { };

	template< size_t N>
	struct in_place_index_t { };
        
} } }

namespace std {
	struct in_place_t { };
	in_place_t const in_place = {};
	
	template <typename T>
	struct in_place_type_t { };
	
	template <size_t I>
	struct in_place_index_t { };

	template <typename T>
	void in_place_type (in_place_type_t<T> = in_place_type_t<T>() ) CXXO_NOEXCEPTNOTHROW { }

	template <size_t I>
	void in_place_index (in_place_index_t<I> = in_place_index_t<I>() ) CXXO_NOEXCEPTNOTHROW { }
	
} // std

	#undef CXXO_ADD_inplace
#endif

// 
// ---
// 


/*
#if (CXXOMFORT_CXX_STD >= 1997)

//
// in_place_t, etc
//


namespace std {


    namespace detail {

        template< typename T >
        struct in_place_type_tag {
        };

        template< size_t N>
        struct in_place_index_tag {};

    } // detail


// 
// in_place_type , in_place_index
//

		#if (CXXOMFORT_CXX_STD == 2014)
    // here we have template variables

//    template <class T>
//    CXXOMFORT_CXX17_CODE(inline, ) constexpr in_place_type_t<T> in_place_type = {};

//    template <size_t I>
//    CXXOMFORT_CXX17_CODE(inline, ) constexpr in_place_index_t<I> in_place_index{};

//                #define CXXO_in_place_type_t(T) ::std::in_place_type_t<T>
    
		#else

// c++<14 does not have inline variables so we use the 
// overloaded function name reference from 
// https://vector-of-bool.github.io/2016/10/30/standard-in-place.html


struct in_place_tag { };

template <typename T>
void in_place_type (detail::in_place_type_tag<T> = detail::in_place_type_tag<T>() ) CXXO_NOEXCEPTNOTHROW {
}

template <size_t I>
void in_place_index (detail::in_place_index_tag<I> = detail::in_place_index_tag<I>() ) CXXO_NOEXCEPTNOTHROW {
}

			#define CXXO_in_place_type_t(T) void(&)( ::std::detail::in_place_type_tag<T> )
		#endif


} // std

    #endif // std_using

// 
// in_place_t, etc
// 

#if (CXXOMFORT_CXX_STD >= 2017)
	#define CXXO_has_in_place 1
#endif
#if (CXXOMFORT_CXX_STD == 2017 && CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=400) && defined(__GLIBCXX__) ) 
	#undef CXXO_has_in_place
#endif


*/

// 
// in_range  (C++20)
// 

#if defined(DOXYGEN_DOC)
namespace cxxomfort { namespace cxxostd {

/**
 * @brief Determines if a value is in the acceptable range of a type.
 * @tparam R A native C type, or a type for which @c std::numeric_limits is specialized.
 * @return A bool.
 * @ingroup cxx20-backports
 * */
template< class R, class T >
CXXO_CONSTEXPR 
bool in_range( T t ) noexcept;

} } 

#elif (CXXOMFORT_CXX_STD >= 2020)
#else
    #include <limits>
namespace cxxomfort { namespace cxxostd {

template< class R, class T >
CXXO_CONSTEXPR 
bool in_range( T t ) CXXO_NOEXCEPTNOTHROW {
    return t >= std::numeric_limits<R>::min() &&
    t <= std::numeric_limits<R>::max();
}

} } 

namespace std {
    using ::cxxomfort::cxxostd::in_range;
}
#endif


// in_place_type, in_place_index using inline variables or emulating via function address

namespace std {

#if (CXXOMFORT_CXX_STD >= 2014)
	#if defined(_LIBCPP_VERSION)
	#else
	
    //template <class T>
    //constexpr in_place_type_t<T> in_place_type {};

    //template <size_t I>
    //constexpr in_place_index_t<I> in_place_index {};
    
	#define CXXO_in_place_type_t(T) ::std::in_place_type_t<T>
    #endif
#else
#endif

}


// 
// std::unreachable (C++23)
// 

#if defined(DOXYGEN_DOC)
namespace std {
	/**
	 * @brief Invokes UB (Undefined Behaviour) to mark program termination.
	 * @xrefitem std0utility "" "" Invokes undefined behaviour.
	 * @cxx23b Invokes undefined behaviour.
	 * */
	 [[noreturn]] inline void unreachable() {}
	 
}
	/**
	 * @ingroup std00featuremacros
	 * @cppfeature{@ref @c std::unreachable() }
	 * */
	#define __cpp_lib_unreachable YYYY00L

#elif (defined(__cpp_lib_unreachable))
#elif (CXXOMFORT_CXX_STD >= 2011)
	#define __cpp_lib_unreachable (CXXOMFORT_CXX_STD*100)
namespace std {
	[[noreturn]] inline void unreachable() {
	#if (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=340) || CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=480))
	__builtin_unreachable();
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>0))
	__assume(false);
	#endif
	}
}
#elif (CXXOMFORT_CXX_STD >= 1997)
namespace std {
	#if (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=340))
	__attribute__((noreturn)) inline void unreachable() { __builtin_unreachable(); }
		#define  __cpp_lib_unreachable 199703
	#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1700))
	inline void unreachable() { __assume(false); }
		#define  __cpp_lib_unreachable 199703
	#endif
}
#else
namespace std {
	inline void unreachable () {}
}
#endif


#endif

/**
 * @page std0utility <utility>
 * @brief Backports related to Standard Header <tt><utility></tt> 
 * 
 * @code
 * #include <cxxomfort/utility.hpp>
 * // or as part of:
 * #include <cxxomfort/backports.hpp>
 * #include <cxxomfort/type_traits.hpp>
 * #include <cxxomfort/tuple.hpp>
 * #include <cxxomfort/library.hpp>
 * @endcode
 * 
 * @section Interfaces
 * 
 * Interfaces defined in this file:
 * 
 * * @c move() (C++11) - moves an object from an "rvalue reference".
 * * @c declval() (C++11) - declare variables in an unevaluated context.
 * * @c exchange() (C++14) - propagate a modification to two variables.
 * * @link cxxomfort::cxxostd::integer_sequence @c integer_sequence @endlink , @link cxxomfort::cxxostd::index_sequence @c index_sequence @endlink  (C++14) (from C++11 onward only).
 * * @c as_const() , @c as_ptr_const() (C++17) - a <tt>const</tt> view of data.
 * * @c in_place_t , @c in_place_type_t (C++17).
 * * @c in_range() (C++20) - check if a type fits within the constraints of an integer type..
 * * @c unreachable() (C++23) — invokes UB in order to mark program termination.
 * * The @ref @c __cpp_lib_exchange_function macro if @c exchange() is implemented.
 * * The @ref @c __cpp_lib_integer_sequence macro if @c #integer_sequence is implemented.
 *
 * @section also See Also
 * 
 * See also @ref std0type_traits , @ref std0tuple , @cppref{header/utility}
 * 
 * See also @ref cxxo-sup-utility (supplemental).
 * 
 * */
