#ifndef CXXOMFORT_UTILS_ATTRIBUTE_HPP
#define CXXOMFORT_UTILS_ATTRIBUTE_HPP

// 
// basic attribute dispatcher
// 

#if defined(DOXYGEN_DOC)
	/**
	 * @brief Marks a declaration with attribute @a attr and gives it arguments @a args... .
	 * 
	 * It is basically equivalent to <tt>[[attr(args...)]]</tt> in C++>=11, or a compiler-dependent construct on previous C++ versions.
	 * 
	 * */
	#define CXXO_ATTRIBUTE(attr, args...) implementation_defined_t
	/**
	 * @brief Marks a declaration with a simple attribute @a attr .
	 * 
	 * It is basically equivalent to <tt>[[attr]]</tt> in C++>=11, or a compiler-dependent construct on previous C++ versions.
	 * 
	 * */
	#define CXXO_ATTR(attr) implementation_defined_t
#else
	#define CXXO_ATTRIBUTE(attr,...) CXXO_ATTRIBUTE_##attr(__VA_ARGS__)
	#define CXXO_ATTR(attr) CXXO_ATTR_attr()
#endif

// 
// direct attribute dispatcher (for eg.: namespaced attributes)
// 
#if (CXXOMFORT_CXX_STD >= 2011)
	#define CXXO_ATTRIBUTE_FULL(...) [[ CXXO_EXPAND(__VA_ARGS__) ]]
#else
	#define CXXO_ATTRIBUTE_FULL(...)
#endif

// 
// shorthand attribute dispatcher
// 

#if (CXXOMFORT_CXX_STD >= 2011)
	#define CXXO_ATTR(attr) [[attr]]
#elif (CXXOMFORT_COMPILER_ID==CXXO_COMPILERID_GCC)
	#define CXXO_ATTR(attr) __attribute__ ((attr)) 
#else
	#define CXXO_ATTR(attr) CXXO_ATTR0_attr()
#endif



// 
// noreturn (C++11)
// 
#if (CXXOMFORT_CXX_STD >= 2011)
	#define CXXO_ATTR0_noreturn() [[noreturn]]
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_CLANG,>=320) || CXXO_COMPILERREQ(CXXO_COMPILERID_GCC,>=408) )
	#define CXXO_ATTR0_noreturn() __attribute__((noreturn))
#elif (CXXO_COMPILERREQ(CXXO_COMPILERID_MSC,>=1200))
	#define CXXO_ATTR0_noreturn() __declspec(noreturn)
#endif



// 
// deprecated("reason") (C++14)
// 
#if (CXXOMFORT_CXX_STD >= 2014)
	#define CXXO_ATTRIBUTE_deprecated(...)  [[deprecated(__VA_ARGS__)]] 
#elif (CXXOMFORT_CXX_STD >= 2011)
	#define CXXO_ATTRIBUTE_deprecated(...) [[deprecated]] 
#else
	#define CXXO_ATTRIBUTE_deprecated(...) CXXO_ATTR(deprecated) 
#endif

// 
// nodiscard (C++17)
// 
#if (CXXOMFORT_CXX_STD>=2017)
	#define CXXO_ATTRIBUTE_nodiscard(...) [[nodiscard(__VA_ARGS__)]]
#else
	// cxxomfort does NOT "backport nodiscard via warn_unused_result; for rationale see https://quuxplusone.github.io/blog/2019/01/25/nodiscard-philosophies/
	#define CXXO_ATTRIBUTE_nodiscard(...)
#endif

//
// pure (non-STD)
// 

#define CXXO_ATTRIBUTE_pure(...) CXXO_ATTR(pure)


#endif // file
