#ifndef CXXOMFORT_UTILS_CHAR_TRAITS_HPP
#define CXXOMFORT_UTILS_CHAR_TRAITS_HPP

/**
 * @file 
 * @brief cxxomfort char_traits without the need to pull <string> and allocator
 * @ingroup cxxomfort-utils
 * */

#include <cxxomfort/config.hpp>
#include <cxxomfort/base.hpp>
#include <cstring>
#include <cwchar>
#include <ios>

namespace cxxomfort {

template <typename Char> 
struct char_traits { };

template <typename C, typename UC, typename I, typename P, typename Off=std::streamoff, typename State=std::mbstate_t>
struct ct_impl {
	typedef C 	char_type;
	typedef I 	int_type;
	typedef P 	pos_type;
	typedef Off 	off_type; 
	typedef State 	state_type;
	typedef bool 	comparison_category; // C++20, unused

	static CXXO_CONSTEXPR14 
	void 	assign (char_type& to, char_type const& x) CXXO_NOEXCEPTNOTHROW { to= x; }
	static CXXO_CONSTEXPR14 
	char_type* 	assign (char_type* to, size_t n, char_type const a) {
		for (size_t i= 0; i != n; ++i) { to[i]= a ;}
	}
	
	static CXXO_CONSTEXPR 
	bool 	eq (char_type c1, char_type c2) CXXO_NOEXCEPT { return static_cast<UC>(c1) == static_cast<UC>(c2); }
	static CXXO_CONSTEXPR 
	bool 	lt (char_type c1, char_type c2) CXXO_NOEXCEPT { return static_cast<UC>(c1) < static_cast<UC>(c2); }

	static 
	char_type* 	move (char_type* dest, char_type const* src, size_t n) {
		for (size_t i=0; i != n; ++i) { dest[i]= src[i]; }
		return dest;
	}
	static 
	char_type* 	copy (char_type* dest, char_type const* src, size_t n) {
		for (size_t i=0; i != n; ++i) { dest[i]= src[i]; }
		return dest;
	}
	static 
	int 	compare (char_type const* s1, char_type const* s2, size_t n) {
		//int r= 0;
		size_t i= 0;
		for (;i!=n;++i) {
			if (s1[i] <s2[i]) return -1;
			else if (s1[i] > s2[i]) return +1;
		}
		return 0;
	}
	static 
	size_t 	length (char_type const* s) {
		size_t r= 0;
		while (s[r]) { ++r; }
		return r;
	}
	
};

template <> struct char_traits <char> 
: ct_impl<char, unsigned char, int, std::streampos> {};

template <> struct char_traits <unsigned char> 
: ct_impl<unsigned char, unsigned char, int, std::streampos> {};

template <> struct char_traits <wchar_t> 
: ct_impl<wchar_t, wchar_t, std::wint_t, std::wstreampos> {};



} // namespace

#endif // file
