#ifndef CXXOMFORT_UTIL_PF_LITERALS_HPP
#define CXXOMFORT_UTIL_PF_LITERALS_HPP

/**
 * @file
 * @brief Literal suffixes for native types, printf-style.
 * @ingroup cxxomfort-utils
 * */
#if (CXXOMFORT_CXX_STD>=1997)
    #include <cxxomfort/config.hpp>
    #include <cxxomfort/cstddef.hpp>
 
namespace cxxomfort { namespace pl {

// 
// function-like, for all C++ versions
// 

#define DEF_SHORTCUT(name_,type_,argtype_) \
CXXO_CONSTEXPR static inline type_ name_ (argtype_ const x) CXXO_NOEXCEPTNOTHROW { return static_cast<type_>(x); }

DEF_SHORTCUT(zi,ssize_t,ssize_t);
DEF_SHORTCUT(zu,size_t,size_t);
DEF_SHORTCUT(ji,intmax_t,intmax_t);
DEF_SHORTCUT(ju,uintmax_t,uintmax_t);
DEF_SHORTCUT(li,long,intmax_t);
DEF_SHORTCUT(lu,unsigned long,uintmax_t);
DEF_SHORTCUT(hi,short,int);
DEF_SHORTCUT(hu,unsigned short,unsigned int);
DEF_SHORTCUT(t,ptrdiff_t,intmax_t);
DEF_SHORTCUT(f,float,long double);
DEF_SHORTCUT(lf,double,long double);
DEF_SHORTCUT(Lf,long double,long double);

#undef DEF_SHORTCUT

    #if (CXXOMFORT_CXX_STD >= 2011)

namespace literals {

constexpr inline
ssize_t operator"" _zi (unsigned long long int x) noexcept { return static_cast<ssize_t>(x); }
constexpr inline
size_t operator"" _zu (unsigned long long int x) noexcept { return static_cast<size_t>(x); }
constexpr inline
intmax_t operator"" _ji (unsigned long long int x) noexcept { return static_cast<intmax_t>(x); }
constexpr inline
uintmax_t operator"" _ju (unsigned long long int x) noexcept { return static_cast<uintmax_t>(x); }
constexpr inline
signed long operator"" _li (unsigned long long int x) noexcept { return static_cast<signed long>(x); }
constexpr inline
unsigned long operator"" _lu (unsigned long long int x) noexcept { return static_cast<unsigned long>(x); }
constexpr inline
signed short operator"" _hi (unsigned long long int x) noexcept { return static_cast<signed short>(x); }
constexpr inline
unsigned short operator"" _hu (unsigned long long int x) noexcept { return static_cast<unsigned short>(x); }

constexpr inline
ptrdiff_t operator"" _t (unsigned long long int x) noexcept { return static_cast<ptrdiff_t>(x); }

constexpr inline
long double operator"" _Lf (long double x) noexcept { return static_cast<long double>(x); }
constexpr inline
double operator"" _lf (long double x) noexcept { return static_cast<double>(x); }
constexpr inline
float operator"" _f (long double x) noexcept { return static_cast<float>(x); }


} // pl::literals

    #else
	namespace literals {}
    #endif
    
    
} } // cxxomfort::pl

#endif

#endif
