#ifndef CXXOMFORT_UTILS_SCOPEDPTR_HPP
#define CXXOMFORT_UTILS_SCOPEDPTR_HPP

/**
 * @file 
 * @brief Includes cxxomfort's @c scoped_ptr implementation
 * @ingroup cxxomfort-utils
 * */

#include <cxxomfort/config.hpp>
//#include "type_traits.hpp"
#include <cxxomfort/memory.hpp>


namespace cxxomfort {

template <typename T, typename Deleter = std::default_delete<T> >
struct scoped_ptr {
    public:
    typedef T* pointer;
        
    explicit scoped_ptr (T* p= nullptr) CXXO_NOEXCEPTNOTHROW
    : ptr(p)
    { }

    ~scoped_ptr() {
        reset();
    }
    
    T*         get () const {
        return ptr;
    }

    void       reset (T* p= nullptr) {
        if (ptr) delete ptr;
        ptr= p;
    }
    
    T&         operator*() const CXXO_NOEXCEPTNOTHROW { return *get(); }
    T*         operator->() const CXXO_NOEXCEPTNOTHROW { return get(); }

    private:
    pointer ptr;
    scoped_ptr (scoped_ptr const&);
    scoped_ptr& operator= (scoped_ptr const&);

};

} // namespace

#endif // file
