#ifndef CXXOMFORT_UTILS_SIESTA_HPP
#define CXXOMFORT_UTILS_SIESTA_HPP
//!  @file
#include <cxxomfort/base.hpp>
#include <cxxomfort/utils/traits_basic.hpp>
#include <cxxomfort/cerrno.hpp>
#include <cstdlib>
#include <cstring>
#include <utility>
#include <cassert>
#include <cxxomfort/cstring.hpp>
#include <cxxomfort/impl/07-type_traits.hpp>
#include "typesafe_enum.hpp"
// #define CXXOMFORT_UTILS_TERROR_HPP // as in "throwable error"


#if defined(_WIN32)
	#if !defined(WIN32_LEAN_AND_MEAN)
		#define WIN32_LEAN_AND_MEAN
	#endif
	//#include <windows.h>
#else
	#include <unistd.h>
#endif

#if (defined(__cpp_exceptions) || defined(_CPPUNWIND) || defined(__EXCEPTIONS) )
	#define CXXO_SIESTA_WITH_EXCEPTIONS
#endif

namespace cxxomfort { 
//! Namespace that contains the copy implementation for "siesta", the status code component
namespace siesta {

CXXO_OPEN_TYPESAFE_ENUM(errc,int) {
	success = 0,
	_unknown = -1,
	#include <cxxomfort/impl/errc-list.hpp>
} CXXO_CLOSE_TYPESAFE_ENUM(errc, int);

#if (CXXO_COMPILERSUPPORT_enum_class>0)
#else
#endif

} }

#include "siesta-status_code.hpp"

namespace cxxomfort { namespace siesta {


typedef status_code<erased<intptr_t> > system_code;

// 
// status_error 
// 

template<> class status_error<void> 
//CXXOMFORT_CXX11_CODE( : public std::exception , ) {
: public std::exception {
	protected:
	#if (CXXO_COMPILER_SUPPORT_default_delete)
	status_error () = default;
	status_error (status_error const&) = default;
	~status_error () CXXOMFORT_CXX11_CODE(override,) = default;
	#else
	status_error () CXXO_NOEXCEPTNOTHROW {}
	~status_error () CXXO_NOEXCEPTNOTHROW CXXOMFORT_CXX11_CODE(override,) {}
	#endif
	
	virtual 
	status_code<void> const& _do_code () const CXXO_NOEXCEPT = 0; 
	
	public:
	typedef void 	domain_type;
	typedef status_code<void> 	status_code_type;
	
	status_code<void> const& 	code () const CXXO_NOEXCEPT { return _do_code(); }
	
};

// status_error for a given domain

template <typename Domain>
class status_error : public status_error<void>  {
	status_code<Domain> _code;
	
	public:
	typedef Domain 	domain_type;
	typedef status_code<Domain> 	status_code_type;
	
	#if (CXXO_COMPILER_SUPPORT_rvref)
	explicit status_error (status_code<Domain> code)
	: _code(static_cast<status_code<Domain> &&>(code)) // should move
	{ }
	#else
	explicit status_error (status_code<Domain> code) CXXO_NOEXCEPTNOTHROW 
	: _code((code))  // should move
	{ 
	}
	#endif
	
	#if (CXXOMFORT_CXX_STD >= 2011)
	#else
	~status_error () CXXO_NOEXCEPTNOTHROW {
	}
	#endif
	
	virtual 
	char const* 	what () const CXXO_NOEXCEPTNOTHROW CXXOMFORT_CXX11_CODE(override,) {
		return _code.domain().name();
	}
	
	status_code_type const& 	code () const CXXO_NOEXCEPTNOTHROW { return _code; }
	
	protected:
	virtual 
	status_code<void> const& _do_code () const 
	CXXO_NOEXCEPT CXXOMFORT_CXX11_CODE(override final,) {
		return _code;
	}

	
};

// 
// generic_code support 
// 

//! Code domain for status codes that are transmissed via @a errc values .
class _generic_code_domain
: public quick_status_code_domain< errc,  0x1122334455667788> {
	template <class> friend class status_code;
	//typedef quick_status_code_domain< errc,  0x1122334455667788> _Base;
	public:
	typedef errc 	value_type;
	public:
	_generic_code_domain () CXXOMFORT_CXX11_CODE(=default, {});
	//explicit _generic_code_domain (typename _Base::unique_id_type id = 0x1122334455667788) CXXO_NOEXCEPTNOTHROW 
	//: _Base(id) {}
	#if (CXXOMFORT_CXX_STD>=2011)
	_generic_code_domain (_generic_code_domain const&) = default;
	#else
	#endif
	~_generic_code_domain () CXXOMFORT_CXX11_CODE(= default, {});
	
	static inline _generic_code_domain const& get() CXXO_NOEXCEPT;
	
	virtual char const* 	name () const CXXO_NOEXCEPT CXXOMFORT_CXX11_CODE(override,) { return "generic domain"; }
	
	protected:
	
	virtual bool _do_failure (status_code<void> const& code) const CXXO_NOEXCEPT CXXOMFORT_CXX11_CODE(override,) {
		return static_cast<generic_code const&>(code).value() != errc(errc::success); 
	}
	/*
	virtual bool _do_equivalent (status_code<void> const& c1, status_code<void> const& c2) const CXXO_NOEXCEPT {
		generic_code const& C1 = static_cast<generic_code const&>(c1);
		generic_code const& C2 = static_cast<generic_code const&>(c2);
		if (c2.domain() == *this) {
			return C1.value() == C2.value(); 
		}
		return false;
	}
	* */
	virtual generic_code _generic_code (status_code<void> const& code) const CXXO_NOEXCEPT CXXOMFORT_CXX11_CODE(override,) {
		assert(code.domain() == *this); 
		generic_code const& gc = static_cast<generic_code const&>(code);
		return gc;
	} 
	
	virtual char* _do_message_to (char* pb, size_t pl, status_code<void> const& code) const CXXO_NOEXCEPT CXXOMFORT_CXX11_CODE(override,) {
		using ::cxxomfort::to_value;
		generic_code const& gerr = static_cast<generic_code const&>(code);
		int const i= to_value(gerr.value());
		::cxxomfort::fix::strerror_r(i, pb, pl);
		return pb;
	}
	#if (defined(CXXO_SIESTA_WITH_EXCEPTIONS))
	virtual void _do_throw_exception (status_code<void> const& code) const CXXOMFORT_CXX11_CODE(override, ) {
		assert(code.domain() == *this);
		generic_code const& c = static_cast<generic_code const&>(code);
		throw status_error<_generic_code_domain>(c);
	}
	#endif
	
	
};
_generic_code_domain const& _generic_code_domain::get () CXXO_NOEXCEPT {
	CXXO_CONSTEXPR static _generic_code_domain const p; return p;
}

static inline generic_code make_status_code (errc c) CXXO_NOEXCEPT {
	return generic_code(std::in_place, c);
}


// 
// int code support 
// 

class _int_code_domain
: public quick_status_code_domain< errc,  0x99> {
	template <class> friend class status_code;
	typedef quick_status_code_domain< errc,  0x99> _Base;
	public:
	typedef int 	value_type;
	public:
	_int_code_domain () {} //: _Base() {}
	//explicit _generic_code_domain (typename _Base::unique_id_type id = 0x1122334455667788) CXXO_NOEXCEPTNOTHROW 
	//: _Base(id) {}
	#if (CXXOMFORT_CXX_STD>=2011)
	_int_code_domain (_int_code_domain const&) = default;
	#else
	#endif
	~_int_code_domain () CXXOMFORT_CXX11_CODE(= default, {});
	
	virtual char const* 	name () const CXXO_NOEXCEPT { return "int-error domain"; }
	
	protected:
	
	virtual bool _do_failure (status_code<void> const& code) const CXXO_NOEXCEPT {
		return static_cast<system_code const&>(code).value() != static_cast<int>(errc::success); 
	}
	/*
	virtual bool _do_equivalent (status_code<void> const& c1, status_code<void> const& c2) const CXXO_NOEXCEPT {
		generic_code const& C1 = static_cast<generic_code const&>(c1);
		generic_code const& C2 = static_cast<generic_code const&>(c2);
		if (c2.domain() == *this) {
			return C1.value() == C2.value(); 
		}
		return false;
	}
	* */
};

} }

#if (defined(DOXYGEN_DOC) || defined(_WIN32))
	#include <winsock2.h>
	#include <basetsd.h>
	#include <winerror.h>
	#include <winbase.h>

namespace cxxomfort { namespace siesta {

class _win32_code_domain;

namespace win32 {
	typedef unsigned long DWORD;
	extern DWORD __stdcall GetLastError();
	extern DWORD __stdcall FormatMessageW(DWORD dwFlags, const void *lpSource, DWORD dwMessageId, DWORD dwLanguageId, wchar_t *lpBuffer, DWORD nSize, void /*va_list*/ *Arguments);
	extern int __stdcall WideCharToMultiByte(unsigned int CodePage, DWORD dwFlags, const wchar_t *lpWideCharStr, int cchWideChar, char *lpMultiByteStr, int cbMultiByte, const char *lpDefaultChar, int *lpUsedDefaultChar);

}

// 
// win32 / winapi code support
// 

//! Status code delivered by WinAPI functions.
typedef status_code<_win32_code_domain> win32_code;

class _win32_code_domain : public quick_status_code_domain<win32::DWORD, 0x3a63786f00009800> {
	template <class> friend class status_code;
	public:
	static inline _win32_code_domain const& get() CXXO_NOEXCEPT;
	virtual char const* 	name () const CXXO_NOEXCEPT { return "Windows win32 domain"; }
	virtual bool _do_failure (status_code<void> const& code) const CXXO_NOEXCEPT {
		return static_cast<win32_code const &>(code).value() != 0;
	}
	virtual generic_code _generic_code (status_code<void> const& code) const CXXO_NOEXCEPT {
		//typedef generic_code::value_type V;
		assert(code.domain() == *this);
		win32_code const& wc = static_cast<win32_code const&>(code);
		#if (CXXO_COMPILER_SUPPORT_enum_class)
		return generic_code(errc(wc.value()));
		#else
		return generic_code(errc::type(wc.value()));
		#endif
	}
	virtual char* _do_message_to (char* p, size_t l, status_code<void> const& code) const CXXO_NOEXCEPT {
		win32_code const& cs= static_cast<win32_code const&>(code);
		win32::DWORD c= cs.value();
		//std::cerr<< "[win32::_do_message_to for c="<< c<< "]"<< std::flush;
		wchar_t wbuffer[32768];
		win32::DWORD wlen = FormatMessageW(0x00001000 /*FORMAT_MESSAGE_FROM_SYSTEM*/ | 0x00000200 /*FORMAT_MESSAGE_IGNORE_INSERTS*/, nullptr, c, 0, wbuffer, 32768, nullptr);
		//std::cerr<< "[FMW len="<< wlen<< "]"<< std::flush;
		char cbuffer[32768];
		size_t bytes = WideCharToMultiByte(65001 /*CP_UTF8*/, 0, wbuffer, (int) (wlen + 1), cbuffer, 32768, nullptr, nullptr);
		//std::cerr<< "[FMW to char len="<< bytes<< "]"<< std::flush;
		if (bytes > 0) {
			char* c_end= strchr(cbuffer, 0);
			while (c_end[-1] == 0x0a || c_end[-1]==0x0d) {--c_end; --bytes;}
			*c_end= 0;
			memcpy(p, cbuffer, l < bytes ? l : bytes);
		}
		return p;
	}
	#if defined(CXXO_SIESTA_WITH_EXCEPTIONS)
	virtual void _do_throw_exception (status_code<void> const& code) const {
		assert(code.domain() == *this);
		win32_code const& c = static_cast<win32_code const&>(code);
		throw status_error<_win32_code_domain>(c);
	}
	#endif
};

_win32_code_domain const& _win32_code_domain::get () CXXO_NOEXCEPT {
	CXXO_CONSTEXPR static _win32_code_domain const p; return p;
}

} }
#endif

// 
// posix (errno) code support 
// 

namespace cxxomfort { namespace siesta {

class _posix_code_domain;
//! Status code delivered by POSIX functions.
typedef status_code<_posix_code_domain> posix_code;
typedef status_error<_posix_code_domain> posix_error;

class _posix_code_domain
: public quick_status_code_domain<int, 0x3a63786f00000101> {
	protected:
	template <class> friend class status_code;
	typedef quick_status_code_domain< errc,  0x3a63786f00000101> _Base;
	static inline _posix_code_domain const& get() CXXO_NOEXCEPT ;
	public:
	virtual char const* 	name () const CXXO_NOEXCEPT { return "POSIX domain"; }
	virtual bool _do_failure (status_code<void> const & code) const CXXO_NOEXCEPT {
		assert(code.domain() == *this); // NOLINT
		return static_cast<const posix_code &>(code).value() != 0; // NOLINT
	}
	virtual generic_code _generic_code (status_code<void> const& code) const CXXO_NOEXCEPT {
		//typedef generic_code::value_type V;
		assert(code.domain() == *this);
		posix_code const& pc = static_cast<posix_code const&>(code);
		#if (CXXO_COMPILER_SUPPORT_enum_class)
		return generic_code(errc(pc.value()));
		#else
		return generic_code(errc::type(pc.value()));
		#endif
	}
	virtual char* _do_message_to (char* p, size_t l, status_code<void> const& code) const CXXO_NOEXCEPT {
		posix_code const& cs= static_cast<posix_code const&>(code);
		int e= (cs.value());
		char const* es= strerror(e);
		size_t elen= strlen(es);
		//std::cerr<< "[posix_code_domain::_do_message_to with value="<< e<< "]"<< std::flush;
		memcpy(p, es, l < elen ? l : elen);
		return p;
	}
	#if defined(CXXO_SIESTA_WITH_EXCEPTIONS)
	virtual void _do_throw_exception (status_code<void> const& code) const {
		assert(code.domain() == *this);
		posix_code const& c = static_cast<posix_code const&>(code);
		throw status_error<_posix_code_domain>(c);
	}
	#endif
};
_posix_code_domain const& _posix_code_domain::get () CXXO_NOEXCEPT {
	CXXO_CONSTEXPR static _posix_code_domain const p; return p;
}

// 
// message domain for cxxomfort 
// 

#if defined(DOXYGEN_DOC)
/**
 * @enum status_messages 
 * @brief Status message codes for @a Siesta .
 * */
enum class status_messages {
#else
CXXO_OPEN_TYPESAFE_ENUM(status_messages, int) {
#endif
	success = 0, //!< Operation completed successfully
	unknown = -1, 
	reserved = 2,
	// workflow status codes
	default_constructor_failure, //!< A default constructor has failed
	copy_constructor_failure, //!< A copy constructor has failed
	use_before_ready, //!< Invoke object methods before it is fully initialized
	use_after_finish, //!< Invoke object methods after it is released / decommissioned
	object_without_value, //!< Attempt to retrieve the value of an object that has None
	still_processing, //!< Invoke object method successful, but the result value is not yet available
	// arithmetic errors
	arithmetic_error, //!< General arithmetic error
	domain_error, //!< An arithmetic argument is not in arithmetic domain
	range_error, //!< Result out of intended arithmetic range
	overflow, //!< Overflow
	// text errors
	text_error, //!< General text error
	empty_string, //!< A text argument is empty
	encoding_error, //!< A text argument has the wrong encoding
	// memory errors
	no_more_memory, //!< A local allocator is exhausted
	out_of_range, //!< Attempt to acces an element out of range
	// runtime errors
	runtime_error, //!< A general runtime error
	// functionality warnings
	
#if !defined(DOXYGEN_DOC)
} CXXO_CLOSE_TYPESAFE_ENUM(status_messages, int);
#else
};
#endif


class _siesta_code_domain;
#if defined(DOXYGEN_DOC)
//! Status code delivered by POSIX functions.
class siesta_code {};
#endif
typedef status_code<_siesta_code_domain> siesta_code;

typedef status_error<_siesta_code_domain> siesta_error;

class _siesta_code_domain
: public quick_status_code_domain<status_messages, 0x3a63786f00000001> {
	template <class> friend class status_code;
	//typedef quick_status_code_domain< errc,  0x3a63786f00000101> _Base;
	static inline _siesta_code_domain const& get() CXXO_NOEXCEPT ;
	public:
	virtual char const* 	name () const CXXO_NOEXCEPT { return "Siesta domain"; }
	virtual bool _do_failure (status_code<void> const & code) const CXXO_NOEXCEPT {
		assert(code.domain() == *this); // NOLINT
		return static_cast<const siesta_code &>(code).value() != status_messages(status_messages::success); // NOLINT
	}
	virtual generic_code _generic_code (status_code<void> const& code) const CXXO_NOEXCEPT {
		//typedef generic_code::value_type V;
		assert(code.domain() == *this);
		siesta_code const& pc = static_cast<siesta_code const&>(code);
		#if (CXXO_COMPILER_SUPPORT_enum_class)
		return generic_code(errc(pc.value()));
		#else
		int pv = to_value(pc.value());
		return generic_code(errc::type(pv)); // return generic_code(errc(errc::type((pc.value()))));
		#endif
	}
	virtual char* _do_message_to (char* p, size_t l, status_code<void> const& code) const CXXO_NOEXCEPT {
		siesta_code const& cs= static_cast<siesta_code const&>(code);
		status_messages const e= (cs.value());
		int ei = to_value(e);
		char const* es= nullptr;
		switch(ei) {
			case int(status_messages::default_constructor_failure): es= "default constructor failure"; break;
			case int(status_messages::copy_constructor_failure): es="copy constructor failure"; break;
			case int(status_messages::object_without_value): es="object does not have a value"; break;
			default: es= "unknown";
			break;
		}
		size_t elen= strlen(es);
		//std::cerr<< "[posix_code_domain::_do_message_to with value="<< e<< "]"<< std::flush;
		memcpy(p, es, l < elen ? l : elen);
		return p;
	}
	#if defined(CXXO_SIESTA_WITH_EXCEPTIONS)
	virtual void _do_throw_exception (status_code<void> const& code) const {
		assert(code.domain() == *this);
		siesta_code const& c = static_cast<siesta_code const&>(code);
		throw status_error<_siesta_code_domain>(c);
	}
	#endif

};
_siesta_code_domain const& _siesta_code_domain::get () CXXO_NOEXCEPT {
	CXXO_CONSTEXPR static _siesta_code_domain const p; return p;
}




} // siesta
} 

/**
 * @page cxxo-siesta 
 * @brief Siesta, a trimmed-down value "status code" message type from SG14.
 * @xrefitem cxxo-utils "" "" Siesta, a trimmed down "status code" implementation.
 * */
#endif

